@@ -6,13 +6,13 @@
 
 #include "InterfaceInitFuncs.h"
 #include "mozilla/a11y/PDocAccessible.h"
-#include "Accessible-inl.h"
+#include "LocalAccessible-inl.h"
 #include "HyperTextAccessible-inl.h"
 #include "nsMai.h"
-#include "ProxyAccessible.h"
+#include "RemoteAccessible.h"
+#include "AccAttributes.h"
 
 #include "nsIAccessibleTypes.h"
-#include "nsIPersistentProperties2.h"
 #include "nsISimpleEnumerator.h"
 #include "nsUTF8Utils.h"
 
@@ -25,11 +25,9 @@ using namespace mozilla::a11y;
 
 static const char* sAtkTextAttrNames[ATK_TEXT_ATTR_LAST_DEFINED];
 
-void
-ConvertTextAttributeToAtkAttribute(const nsACString& aName,
-                                   const nsAString& aValue,
-                                   AtkAttributeSet** aAttributeSet)
-{
+void ConvertTextAttributeToAtkAttribute(const nsACString& aName,
+                                        const nsAString& aValue,
+                                        AtkAttributeSet** aAttributeSet) {
   // Handle attributes where atk has its own name.
   const char* atkName = nullptr;
   nsAutoString atkValue;
@@ -62,120 +60,116 @@ ConvertTextAttributeToAtkAttribute(const nsACString& aName,
 
   if (atkName) {
     AtkAttribute* objAttr =
-      static_cast<AtkAttribute*>(g_malloc(sizeof(AtkAttribute)));
+        static_cast<AtkAttribute*>(g_malloc(sizeof(AtkAttribute)));
     objAttr->name = g_strdup(atkName);
     objAttr->value = g_strdup(NS_ConvertUTF16toUTF8(atkValue).get());
     *aAttributeSet = g_slist_prepend(*aAttributeSet, objAttr);
   }
 }
 
-static AtkAttributeSet*
-ConvertToAtkTextAttributeSet(nsTArray<Attribute>& aAttributes)
-{
-  AtkAttributeSet* objAttributeSet = nullptr;
-  for (size_t i = 0; i < aAttributes.Length(); ++i) {
-    AtkAttribute* objAttr = (AtkAttribute *)g_malloc(sizeof(AtkAttribute));
-    objAttr->name = g_strdup(aAttributes[i].Name().get());
-    objAttr->value =
-      g_strdup(NS_ConvertUTF16toUTF8(aAttributes[i].Value()).get());
-    objAttributeSet = g_slist_prepend(objAttributeSet, objAttr);
-    ConvertTextAttributeToAtkAttribute(aAttributes[i].Name(),
-                                       aAttributes[i].Value(),
-                                       &objAttributeSet);
-  }
-  return objAttributeSet;
-}
-
-static AtkAttributeSet*
-ConvertToAtkTextAttributeSet(nsIPersistentProperties* aAttributes)
-{
-  if (!aAttributes)
+static AtkAttributeSet* ConvertToAtkTextAttributeSet(
+    AccAttributes* aAttributes) {
+  if (!aAttributes) {
+    // This can happen if an Accessible dies in the content process, but the
+    // parent hasn't been udpated yet.
     return nullptr;
+  }
 
   AtkAttributeSet* objAttributeSet = nullptr;
-  nsCOMPtr<nsISimpleEnumerator> propEnum;
-  nsresult rv = aAttributes->Enumerate(getter_AddRefs(propEnum));
-  NS_ENSURE_SUCCESS(rv, nullptr);
 
-  bool hasMore = false;
-  while (NS_SUCCEEDED(propEnum->HasMoreElements(&hasMore)) && hasMore) {
-    nsCOMPtr<nsISupports> sup;
-    rv = propEnum->GetNext(getter_AddRefs(sup));
-    NS_ENSURE_SUCCESS(rv, objAttributeSet);
-
-    nsCOMPtr<nsIPropertyElement> propElem(do_QueryInterface(sup));
-    NS_ENSURE_TRUE(propElem, objAttributeSet);
-
-    nsAutoCString name;
-    rv = propElem->GetKey(name);
-    NS_ENSURE_SUCCESS(rv, objAttributeSet);
+  for (auto iter : *aAttributes) {
+    nsAutoString name;
+    iter.NameAsString(name);
 
     nsAutoString value;
-    rv = propElem->GetValue(value);
-    NS_ENSURE_SUCCESS(rv, objAttributeSet);
+    iter.ValueAsString(value);
 
     AtkAttribute* objAttr = (AtkAttribute*)g_malloc(sizeof(AtkAttribute));
-    objAttr->name = g_strdup(name.get());
+    objAttr->name = g_strdup(NS_ConvertUTF16toUTF8(name).get());
     objAttr->value = g_strdup(NS_ConvertUTF16toUTF8(value).get());
     objAttributeSet = g_slist_prepend(objAttributeSet, objAttr);
-
-    ConvertTextAttributeToAtkAttribute(name, value, &objAttributeSet);
   }
 
   // libatk-adaptor will free it
   return objAttributeSet;
 }
 
-static void
-ConvertTexttoAsterisks(AccessibleWrap* accWrap, nsAString& aString)
-{
+static void ConvertTexttoAsterisks(AccessibleWrap* accWrap,
+                                   nsAString& aString) {
   // convert each char to "*" when it's "password text"
-  if (accWrap->NativeRole() == roles::PASSWORD_TEXT) {
+  if (accWrap->IsPassword()) {
     DOMtoATK::ConvertTexttoAsterisks(aString);
   }
 }
 
 extern "C" {
 
-static gchar*
-getTextCB(AtkText *aText, gint aStartOffset, gint aEndOffset)
-{
+static gchar* getTextCB(AtkText* aText, gint aStartOffset, gint aEndOffset) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   nsAutoString autoStr;
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
-    if (!text || !text->IsTextRole() || text->IsDefunct())
-      return nullptr;
+    if (!text || !text->IsTextRole() || text->IsDefunct()) return nullptr;
 
-    return DOMtoATK::NewATKString(text, aStartOffset, aEndOffset,
-         accWrap->NativeRole() == roles::PASSWORD_TEXT ?
-           DOMtoATK::AtkStringConvertFlags::ConvertTextToAsterisks :
-           DOMtoATK::AtkStringConvertFlags::None);
+    return DOMtoATK::NewATKString(
+        text, aStartOffset, aEndOffset,
+        accWrap->IsPassword()
+            ? DOMtoATK::AtkStringConvertFlags::ConvertTextToAsterisks
+            : DOMtoATK::AtkStringConvertFlags::None);
 
-  } else if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  } else if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     return DOMtoATK::NewATKString(proxy, aStartOffset, aEndOffset,
-         DOMtoATK::AtkStringConvertFlags::None);
+                                  DOMtoATK::AtkStringConvertFlags::None);
   }
 
   return nullptr;
 }
 
-static gchar*
-getTextAfterOffsetCB(AtkText *aText, gint aOffset,
-                     AtkTextBoundary aBoundaryType,
-                     gint *aStartOffset, gint *aEndOffset)
-{
-    nsAutoString autoStr;
+static gint getCharacterCountCB(AtkText* aText);
+
+// Note: this does not support magic offsets, which is fine for its callers
+// which do not implement any.
+static gchar* getCharTextAtOffset(AtkText* aText, gint aOffset,
+                                  gint* aStartOffset, gint* aEndOffset) {
+  gint end = aOffset + 1;
+  gint count = getCharacterCountCB(aText);
+
+  if (aOffset > count) {
+    aOffset = count;
+  }
+  if (end > count) {
+    end = count;
+  }
+  if (aOffset < 0) {
+    aOffset = 0;
+  }
+  if (end < 0) {
+    end = 0;
+  }
+  *aStartOffset = aOffset;
+  *aEndOffset = end;
+
+  return getTextCB(aText, aOffset, end);
+}
+
+static gchar* getTextAfterOffsetCB(AtkText* aText, gint aOffset,
+                                   AtkTextBoundary aBoundaryType,
+                                   gint* aStartOffset, gint* aEndOffset) {
+  if (aBoundaryType == ATK_TEXT_BOUNDARY_CHAR) {
+    return getCharTextAtOffset(aText, aOffset + 1, aStartOffset, aEndOffset);
+  }
+
+  nsAutoString autoStr;
   int32_t startOffset = 0, endOffset = 0;
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
-    if (!text || !text->IsTextRole())
-      return nullptr;
+    if (!text || !text->IsTextRole()) return nullptr;
 
-    text->TextAfterOffset(aOffset, aBoundaryType, &startOffset, &endOffset, autoStr);
+    text->TextAfterOffset(aOffset, aBoundaryType, &startOffset, &endOffset,
+                          autoStr);
     ConvertTexttoAsterisks(accWrap, autoStr);
-  } else if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  } else if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     proxy->GetTextAfterOffset(aOffset, aBoundaryType, autoStr, &startOffset,
                               &endOffset);
   }
@@ -187,22 +181,24 @@ getTextAfterOffsetCB(AtkText *aText, gint aOffset,
   return DOMtoATK::Convert(autoStr);
 }
 
-static gchar*
-getTextAtOffsetCB(AtkText *aText, gint aOffset,
-                  AtkTextBoundary aBoundaryType,
-                  gint *aStartOffset, gint *aEndOffset)
-{
+static gchar* getTextAtOffsetCB(AtkText* aText, gint aOffset,
+                                AtkTextBoundary aBoundaryType,
+                                gint* aStartOffset, gint* aEndOffset) {
+  if (aBoundaryType == ATK_TEXT_BOUNDARY_CHAR) {
+    return getCharTextAtOffset(aText, aOffset, aStartOffset, aEndOffset);
+  }
+
   nsAutoString autoStr;
   int32_t startOffset = 0, endOffset = 0;
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
-    if (!text || !text->IsTextRole())
-      return nullptr;
+    if (!text || !text->IsTextRole()) return nullptr;
 
-    text->TextAtOffset(aOffset, aBoundaryType, &startOffset, &endOffset, autoStr);
+    text->TextAtOffset(aOffset, aBoundaryType, &startOffset, &endOffset,
+                       autoStr);
     ConvertTexttoAsterisks(accWrap, autoStr);
-  } else if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  } else if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     proxy->GetTextAtOffset(aOffset, aBoundaryType, autoStr, &startOffset,
                            &endOffset);
   }
@@ -214,9 +210,7 @@ getTextAtOffsetCB(AtkText *aText, gint aOffset,
   return DOMtoATK::Convert(autoStr);
 }
 
-static gunichar
-getCharacterAtOffsetCB(AtkText* aText, gint aOffset)
-{
+static gunichar getCharacterAtOffsetCB(AtkText* aText, gint aOffset) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
@@ -226,30 +220,31 @@ getCharacterAtOffsetCB(AtkText* aText, gint aOffset)
     return DOMtoATK::ATKCharacter(text, aOffset);
   }
 
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     return DOMtoATK::ATKCharacter(proxy, aOffset);
   }
 
   return 0;
 }
 
-static gchar*
-getTextBeforeOffsetCB(AtkText *aText, gint aOffset,
-                      AtkTextBoundary aBoundaryType,
-                      gint *aStartOffset, gint *aEndOffset)
-{
+static gchar* getTextBeforeOffsetCB(AtkText* aText, gint aOffset,
+                                    AtkTextBoundary aBoundaryType,
+                                    gint* aStartOffset, gint* aEndOffset) {
+  if (aBoundaryType == ATK_TEXT_BOUNDARY_CHAR) {
+    return getCharTextAtOffset(aText, aOffset - 1, aStartOffset, aEndOffset);
+  }
+
   nsAutoString autoStr;
   int32_t startOffset = 0, endOffset = 0;
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
-    if (!text || !text->IsTextRole())
-      return nullptr;
+    if (!text || !text->IsTextRole()) return nullptr;
 
-    text->TextBeforeOffset(aOffset, aBoundaryType,
-                           &startOffset, &endOffset, autoStr);
+    text->TextBeforeOffset(aOffset, aBoundaryType, &startOffset, &endOffset,
+                           autoStr);
     ConvertTexttoAsterisks(accWrap, autoStr);
-  } else if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  } else if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     proxy->GetTextBeforeOffset(aOffset, aBoundaryType, autoStr, &startOffset,
                                &endOffset);
   }
@@ -261,31 +256,27 @@ getTextBeforeOffsetCB(AtkText *aText, gint aOffset,
   return DOMtoATK::Convert(autoStr);
 }
 
-static gint
-getCaretOffsetCB(AtkText *aText)
-{
+static gint getCaretOffsetCB(AtkText* aText) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
     if (!text || !text->IsTextRole()) {
-      return 0;
+      return -1;
     }
 
     return static_cast<gint>(text->CaretOffset());
   }
 
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     return static_cast<gint>(proxy->CaretOffset());
   }
 
-  return 0;
+  return -1;
 }
 
-static AtkAttributeSet*
-getRunAttributesCB(AtkText *aText, gint aOffset,
-                   gint *aStartOffset,
-                   gint *aEndOffset)
-{
+static AtkAttributeSet* getRunAttributesCB(AtkText* aText, gint aOffset,
+                                           gint* aStartOffset,
+                                           gint* aEndOffset) {
   *aStartOffset = -1;
   *aEndOffset = -1;
   int32_t startOffset = 0, endOffset = 0;
@@ -297,8 +288,8 @@ getRunAttributesCB(AtkText *aText, gint aOffset,
       return nullptr;
     }
 
-    nsCOMPtr<nsIPersistentProperties> attributes =
-      text->TextAttributes(false, aOffset, &startOffset, &endOffset);
+    RefPtr<AccAttributes> attributes =
+        text->TextAttributes(false, aOffset, &startOffset, &endOffset);
 
     *aStartOffset = startOffset;
     *aEndOffset = endOffset;
@@ -306,21 +297,19 @@ getRunAttributesCB(AtkText *aText, gint aOffset,
     return ConvertToAtkTextAttributeSet(attributes);
   }
 
-  ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText));
+  RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText));
   if (!proxy) {
     return nullptr;
   }
 
-  AutoTArray<Attribute, 10> attrs;
+  RefPtr<AccAttributes> attrs;
   proxy->TextAttributes(false, aOffset, &attrs, &startOffset, &endOffset);
   *aStartOffset = startOffset;
   *aEndOffset = endOffset;
   return ConvertToAtkTextAttributeSet(attrs);
 }
 
-static AtkAttributeSet*
-getDefaultAttributesCB(AtkText *aText)
-{
+static AtkAttributeSet* getDefaultAttributesCB(AtkText* aText) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
@@ -328,29 +317,27 @@ getDefaultAttributesCB(AtkText *aText)
       return nullptr;
     }
 
-    nsCOMPtr<nsIPersistentProperties> attributes = text->DefaultTextAttributes();
+    RefPtr<AccAttributes> attributes = text->DefaultTextAttributes();
     return ConvertToAtkTextAttributeSet(attributes);
   }
 
-  ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText));
+  RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText));
   if (!proxy) {
     return nullptr;
   }
 
-  AutoTArray<Attribute, 10> attrs;
+  RefPtr<AccAttributes> attrs;
   proxy->DefaultTextAttributes(&attrs);
   return ConvertToAtkTextAttributeSet(attrs);
 }
 
-static void
-getCharacterExtentsCB(AtkText *aText, gint aOffset,
-                      gint *aX, gint *aY,
-                      gint *aWidth, gint *aHeight,
-                      AtkCoordType aCoords)
-{
-  if(!aX || !aY || !aWidth || !aHeight) {
+static void getCharacterExtentsCB(AtkText* aText, gint aOffset, gint* aX,
+                                  gint* aY, gint* aWidth, gint* aHeight,
+                                  AtkCoordType aCoords) {
+  if (!aX || !aY || !aWidth || !aHeight) {
     return;
   }
+  *aX = *aY = *aWidth = *aHeight = -1;
 
   nsIntRect rect;
   uint32_t geckoCoordType;
@@ -368,7 +355,7 @@ getCharacterExtentsCB(AtkText *aText, gint aOffset,
     }
 
     rect = text->CharBounds(aOffset, geckoCoordType);
-  } else if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  } else if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     rect = proxy->CharBounds(aOffset, geckoCoordType);
   } else {
     return;
@@ -380,13 +367,13 @@ getCharacterExtentsCB(AtkText *aText, gint aOffset,
   *aHeight = rect.height;
 }
 
-static void
-getRangeExtentsCB(AtkText *aText, gint aStartOffset, gint aEndOffset,
-                  AtkCoordType aCoords, AtkTextRectangle *aRect)
-{
+static void getRangeExtentsCB(AtkText* aText, gint aStartOffset,
+                              gint aEndOffset, AtkCoordType aCoords,
+                              AtkTextRectangle* aRect) {
   if (!aRect) {
     return;
   }
+  aRect->x = aRect->y = aRect->width = aRect->height = -1;
 
   nsIntRect rect;
   uint32_t geckoCoordType;
@@ -397,14 +384,14 @@ getRangeExtentsCB(AtkText *aText, gint aStartOffset, gint aEndOffset,
   }
 
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
-  if(accWrap) {
+  if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
     if (!text || !text->IsTextRole()) {
       return;
     }
 
     rect = text->TextBounds(aStartOffset, aEndOffset, geckoCoordType);
-  } else if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  } else if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     rect = proxy->TextBounds(aStartOffset, aEndOffset, geckoCoordType);
   } else {
     return;
@@ -416,28 +403,24 @@ getRangeExtentsCB(AtkText *aText, gint aStartOffset, gint aEndOffset,
   aRect->height = rect.height;
 }
 
-static gint
-getCharacterCountCB(AtkText *aText)
-{
+static gint getCharacterCountCB(AtkText* aText) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* textAcc = accWrap->AsHyperText();
-    return
-      textAcc->IsDefunct() ? 0 : static_cast<gint>(textAcc->CharacterCount());
+    return !textAcc || textAcc->IsDefunct()
+               ? 0
+               : static_cast<gint>(textAcc->CharacterCount());
   }
 
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     return proxy->CharacterCount();
   }
 
   return 0;
 }
 
-static gint
-getOffsetAtPointCB(AtkText *aText,
-                   gint aX, gint aY,
-                   AtkCoordType aCoords)
-{
+static gint getOffsetAtPointCB(AtkText* aText, gint aX, gint aY,
+                               AtkCoordType aCoords) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
@@ -445,27 +428,25 @@ getOffsetAtPointCB(AtkText *aText,
       return -1;
     }
 
-    return static_cast<gint>(
-      text->OffsetAtPoint(aX, aY,
-                          (aCoords == ATK_XY_SCREEN ?
-                           nsIAccessibleCoordinateType::COORDTYPE_SCREEN_RELATIVE :
-                           nsIAccessibleCoordinateType::COORDTYPE_WINDOW_RELATIVE)));
+    return static_cast<gint>(text->OffsetAtPoint(
+        aX, aY,
+        (aCoords == ATK_XY_SCREEN
+             ? nsIAccessibleCoordinateType::COORDTYPE_SCREEN_RELATIVE
+             : nsIAccessibleCoordinateType::COORDTYPE_WINDOW_RELATIVE)));
   }
 
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
-    return static_cast<gint>(
-      proxy->OffsetAtPoint(aX, aY,
-                           (aCoords == ATK_XY_SCREEN ?
-                            nsIAccessibleCoordinateType::COORDTYPE_SCREEN_RELATIVE :
-                            nsIAccessibleCoordinateType::COORDTYPE_WINDOW_RELATIVE)));
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+    return static_cast<gint>(proxy->OffsetAtPoint(
+        aX, aY,
+        (aCoords == ATK_XY_SCREEN
+             ? nsIAccessibleCoordinateType::COORDTYPE_SCREEN_RELATIVE
+             : nsIAccessibleCoordinateType::COORDTYPE_WINDOW_RELATIVE)));
   }
 
   return -1;
 }
 
-static gint
-getTextSelectionCountCB(AtkText *aText)
-{
+static gint getTextSelectionCountCB(AtkText* aText) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
@@ -476,17 +457,15 @@ getTextSelectionCountCB(AtkText *aText)
     return text->SelectionCount();
   }
 
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     return proxy->SelectionCount();
   }
 
   return 0;
 }
 
-static gchar*
-getTextSelectionCB(AtkText *aText, gint aSelectionNum,
-                   gint *aStartOffset, gint *aEndOffset)
-{
+static gchar* getTextSelectionCB(AtkText* aText, gint aSelectionNum,
+                                 gint* aStartOffset, gint* aEndOffset) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   int32_t startOffset = 0, endOffset = 0;
   if (accWrap) {
@@ -501,7 +480,7 @@ getTextSelectionCB(AtkText *aText, gint aSelectionNum,
 
     return getTextCB(aText, *aStartOffset, *aEndOffset);
   }
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     nsString data;
     proxy->SelectionBoundsAt(aSelectionNum, data, &startOffset, &endOffset);
     *aStartOffset = startOffset;
@@ -514,11 +493,8 @@ getTextSelectionCB(AtkText *aText, gint aSelectionNum,
 }
 
 // set methods
-static gboolean
-addTextSelectionCB(AtkText *aText,
-                   gint aStartOffset,
-                   gint aEndOffset)
-{
+static gboolean addTextSelectionCB(AtkText* aText, gint aStartOffset,
+                                   gint aEndOffset) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
@@ -528,17 +504,14 @@ addTextSelectionCB(AtkText *aText,
 
     return text->AddToSelection(aStartOffset, aEndOffset);
   }
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     return proxy->AddToSelection(aStartOffset, aEndOffset);
   }
 
   return FALSE;
 }
 
-static gboolean
-removeTextSelectionCB(AtkText *aText,
-                      gint aSelectionNum)
-{
+static gboolean removeTextSelectionCB(AtkText* aText, gint aSelectionNum) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
@@ -548,17 +521,15 @@ removeTextSelectionCB(AtkText *aText,
 
     return text->RemoveFromSelection(aSelectionNum);
   }
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     return proxy->RemoveFromSelection(aSelectionNum);
   }
 
   return FALSE;
 }
 
-static gboolean
-setTextSelectionCB(AtkText *aText, gint aSelectionNum,
-                   gint aStartOffset, gint aEndOffset)
-{
+static gboolean setTextSelectionCB(AtkText* aText, gint aSelectionNum,
+                                   gint aStartOffset, gint aEndOffset) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
@@ -568,16 +539,14 @@ setTextSelectionCB(AtkText *aText, gint aSelectionNum,
 
     return text->SetSelectionBoundsAt(aSelectionNum, aStartOffset, aEndOffset);
   }
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     return proxy->SetSelectionBoundsAt(aSelectionNum, aStartOffset, aEndOffset);
   }
 
   return FALSE;
 }
 
-static gboolean
-setCaretOffsetCB(AtkText *aText, gint aOffset)
-{
+static gboolean setCaretOffsetCB(AtkText* aText, gint aOffset) {
   AccessibleWrap* accWrap = GetAccessibleWrap(ATK_OBJECT(aText));
   if (accWrap) {
     HyperTextAccessible* text = accWrap->AsHyperText();
@@ -589,21 +558,65 @@ setCaretOffsetCB(AtkText *aText, gint aOffset)
     return TRUE;
   }
 
-  if (ProxyAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
+  if (RemoteAccessible* proxy = GetProxy(ATK_OBJECT(aText))) {
     proxy->SetCaretOffset(aOffset);
     return TRUE;
   }
 
   return FALSE;
 }
+
+static gboolean scrollSubstringToCB(AtkText* aText, gint aStartOffset,
+                                    gint aEndOffset, AtkScrollType aType) {
+  AtkObject* atkObject = ATK_OBJECT(aText);
+  AccessibleWrap* accWrap = GetAccessibleWrap(atkObject);
+  if (accWrap) {
+    HyperTextAccessible* text = accWrap->AsHyperText();
+    if (!text || !text->IsTextRole() ||
+        !text->IsValidRange(aStartOffset, aEndOffset)) {
+      return FALSE;
+    }
+    text->ScrollSubstringTo(aStartOffset, aEndOffset, aType);
+    return TRUE;
+  }
+
+  RemoteAccessible* proxy = GetProxy(atkObject);
+  if (proxy) {
+    proxy->ScrollSubstringTo(aStartOffset, aEndOffset, aType);
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
+static gboolean scrollSubstringToPointCB(AtkText* aText, gint aStartOffset,
+                                         gint aEndOffset, AtkCoordType aCoords,
+                                         gint aX, gint aY) {
+  AtkObject* atkObject = ATK_OBJECT(aText);
+  AccessibleWrap* accWrap = GetAccessibleWrap(atkObject);
+  if (accWrap) {
+    HyperTextAccessible* text = accWrap->AsHyperText();
+    if (!text || !text->IsTextRole() ||
+        !text->IsValidRange(aStartOffset, aEndOffset)) {
+      return FALSE;
+    }
+    text->ScrollSubstringToPoint(aStartOffset, aEndOffset, aCoords, aX, aY);
+    return TRUE;
+  }
+
+  RemoteAccessible* proxy = GetProxy(atkObject);
+  if (proxy) {
+    proxy->ScrollSubstringToPoint(aStartOffset, aEndOffset, aCoords, aX, aY);
+    return TRUE;
+  }
+
+  return FALSE;
+}
 }
 
-void
-textInterfaceInitCB(AtkTextIface* aIface)
-{
+void textInterfaceInitCB(AtkTextIface* aIface) {
   NS_ASSERTION(aIface, "Invalid aIface");
-  if (MOZ_UNLIKELY(!aIface))
-    return;
+  if (MOZ_UNLIKELY(!aIface)) return;
 
   aIface->get_text = getTextCB;
   aIface->get_text_after_offset = getTextAfterOffsetCB;
@@ -620,14 +633,20 @@ textInterfaceInitCB(AtkTextIface* aIface)
   aIface->get_n_selections = getTextSelectionCountCB;
   aIface->get_selection = getTextSelectionCB;
 
-    // set methods
+  // set methods
   aIface->add_selection = addTextSelectionCB;
   aIface->remove_selection = removeTextSelectionCB;
   aIface->set_selection = setTextSelectionCB;
   aIface->set_caret_offset = setCaretOffsetCB;
 
+  if (IsAtkVersionAtLeast(2, 32)) {
+    aIface->scroll_substring_to = scrollSubstringToCB;
+    aIface->scroll_substring_to_point = scrollSubstringToPointCB;
+  }
+
   // Cache the string values of the atk text attribute names.
-  for (uint32_t i = 0; i < ArrayLength(sAtkTextAttrNames); i++)
+  for (uint32_t i = 0; i < ArrayLength(sAtkTextAttrNames); i++) {
     sAtkTextAttrNames[i] =
-      atk_text_attribute_get_name(static_cast<AtkTextAttribute>(i));
+        atk_text_attribute_get_name(static_cast<AtkTextAttribute>(i));
+  }
 }
