@@ -6,7 +6,7 @@
 
 #include "Logging.h"
 
-#include "Accessible-inl.h"
+#include "LocalAccessible-inl.h"
 #include "AccEvent.h"
 #include "DocAccessible.h"
 #include "nsAccessibilityService.h"
@@ -16,12 +16,13 @@
 #include "nsDocShellLoadTypes.h"
 #include "nsIChannel.h"
 #include "nsIInterfaceRequestorUtils.h"
-#include "nsTraceRefcnt.h"
 #include "nsIWebProgress.h"
 #include "prenv.h"
 #include "nsIDocShellTreeItem.h"
 #include "mozilla/PresShell.h"
+#include "mozilla/StackWalk.h"
 #include "mozilla/dom/BorrowedAttrInfo.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/Element.h"
 #include "mozilla/dom/HTMLBodyElement.h"
 #include "mozilla/dom/Selection.h"
@@ -31,6 +32,8 @@ using namespace mozilla::a11y;
 
 using mozilla::dom::BorrowedAttrInfo;
 
+MOZ_DEFINE_MALLOC_SIZE_OF(AccessibleLoggingMallocSizeOf)
+
 ////////////////////////////////////////////////////////////////////////////////
 // Logging helpers
 
@@ -51,6 +54,7 @@ static ModuleRep sModuleMap[] = {{"docload", logging::eDocLoad},
                                  {"platforms", logging::ePlatforms},
                                  {"text", logging::eText},
                                  {"tree", logging::eTree},
+                                 {"treeSize", logging::eTreeSize},
 
                                  {"DOMEvents", logging::eDOMEvents},
                                  {"focus", logging::eFocus},
@@ -127,9 +131,10 @@ static void LogDocShellTree(dom::Document* aDocumentNode) {
     treeItem->GetInProcessRootTreeItem(getter_AddRefs(rootTreeItem));
     printf(
         "in-process docshell hierarchy, parent: %p, root: %p, "
-        "is tab document: %s;",
+        "is top level: %s;",
         static_cast<void*>(parentTreeItem), static_cast<void*>(rootTreeItem),
-        (nsCoreUtils::IsTabDocument(aDocumentNode) ? "yes" : "no"));
+        (nsCoreUtils::IsTopLevelContentDocInProcess(aDocumentNode) ? "yes"
+                                                                   : "no"));
   }
 }
 
@@ -155,8 +160,11 @@ static void LogDocState(dom::Document* aDocumentNode) {
   printf(", %sinitial", aDocumentNode->IsInitialDocument() ? "" : "not ");
   printf(", %sshowing", aDocumentNode->IsShowing() ? "" : "not ");
   printf(", %svisible", aDocumentNode->IsVisible() ? "" : "not ");
-  printf(", %svisible considering ancestors",
-         aDocumentNode->IsVisibleConsideringAncestors() ? "" : "not ");
+  printf(
+      ", %svisible considering ancestors",
+      nsCoreUtils::IsDocumentVisibleConsideringInProcessAncestors(aDocumentNode)
+          ? ""
+          : "not ");
   printf(", %sactive", aDocumentNode->IsActive() ? "" : "not ");
   printf(", %sresource", aDocumentNode->IsResourceDoc() ? "" : "not ");
 
@@ -233,9 +241,6 @@ static void LogShellLoadType(nsIDocShell* aDocShell) {
     case LOAD_NORMAL_REPLACE:
       printf("normal replace; ");
       break;
-    case LOAD_NORMAL_EXTERNAL:
-      printf("normal external; ");
-      break;
     case LOAD_HISTORY:
       printf("history; ");
       break;
@@ -248,9 +253,6 @@ static void LogShellLoadType(nsIDocShell* aDocShell) {
     case LOAD_NORMAL_BYPASS_PROXY_AND_CACHE:
       printf("normal bypass proxy and cache; ");
       break;
-    case LOAD_NORMAL_ALLOW_MIXED_CONTENT:
-      printf("normal allow mixed content; ");
-      break;
     case LOAD_RELOAD_NORMAL:
       printf("reload normal; ");
       break;
@@ -263,15 +265,15 @@ static void LogShellLoadType(nsIDocShell* aDocShell) {
     case LOAD_RELOAD_BYPASS_PROXY_AND_CACHE:
       printf("reload bypass proxy and cache; ");
       break;
-    case LOAD_RELOAD_ALLOW_MIXED_CONTENT:
-      printf("reload allow mixed content; ");
-      break;
     case LOAD_LINK:
       printf("link; ");
       break;
     case LOAD_REFRESH:
       printf("refresh; ");
       break;
+    case LOAD_REFRESH_REPLACE:
+      printf("refresh replace; ");
+      break;
     case LOAD_RELOAD_CHARSET_CHANGE:
       printf("reload charset change; ");
       break;
@@ -307,14 +309,17 @@ static void LogRequest(nsIRequest* aRequest) {
     aRequest->GetLoadFlags(&loadFlags);
     printf("    request load flags: %x; ", loadFlags);
     if (loadFlags & nsIChannel::LOAD_DOCUMENT_URI) printf("document uri; ");
-    if (loadFlags & nsIChannel::LOAD_RETARGETED_DOCUMENT_URI)
+    if (loadFlags & nsIChannel::LOAD_RETARGETED_DOCUMENT_URI) {
       printf("retargeted document uri; ");
+    }
     if (loadFlags & nsIChannel::LOAD_REPLACE) printf("replace; ");
-    if (loadFlags & nsIChannel::LOAD_INITIAL_DOCUMENT_URI)
+    if (loadFlags & nsIChannel::LOAD_INITIAL_DOCUMENT_URI) {
       printf("initial document uri; ");
+    }
     if (loadFlags & nsIChannel::LOAD_TARGETED) printf("targeted; ");
-    if (loadFlags & nsIChannel::LOAD_CALL_CONTENT_SNIFFERS)
+    if (loadFlags & nsIChannel::LOAD_CALL_CONTENT_SNIFFERS) {
       printf("call content sniffers; ");
+    }
     if (loadFlags & nsIChannel::LOAD_BYPASS_URL_CLASSIFIER) {
       printf("bypass classify uri; ");
     }
@@ -325,14 +330,15 @@ static void LogRequest(nsIRequest* aRequest) {
 
 static void LogDocAccState(DocAccessible* aDocument) {
   printf("document acc state: ");
-  if (aDocument->HasLoadState(DocAccessible::eCompletelyLoaded))
+  if (aDocument->HasLoadState(DocAccessible::eCompletelyLoaded)) {
     printf("completely loaded;");
-  else if (aDocument->HasLoadState(DocAccessible::eReady))
+  } else if (aDocument->HasLoadState(DocAccessible::eReady)) {
     printf("ready;");
-  else if (aDocument->HasLoadState(DocAccessible::eDOMLoaded))
+  } else if (aDocument->HasLoadState(DocAccessible::eDOMLoaded)) {
     printf("DOM loaded;");
-  else if (aDocument->HasLoadState(DocAccessible::eTreeConstructed))
+  } else if (aDocument->HasLoadState(DocAccessible::eTreeConstructed)) {
     printf("tree constructed;");
+  }
 }
 
 static void GetDocLoadEventType(AccEvent* aEvent, nsACString& aEventType) {
@@ -347,10 +353,11 @@ static void GetDocLoadEventType(AccEvent* aEvent, nsACString& aEventType) {
     AccStateChangeEvent* event = downcast_accEvent(aEvent);
     if (event->GetState() == states::BUSY) {
       aEventType.AssignLiteral("busy ");
-      if (event->IsStateEnabled())
+      if (event->IsStateEnabled()) {
         aEventType.AppendLiteral("true");
-      else
+      } else {
         aEventType.AppendLiteral("false");
+      }
     }
   }
 }
@@ -537,7 +544,7 @@ void logging::OuterDocDestroy(OuterDocAccessible* aOuterDoc) {
 
 void logging::FocusNotificationTarget(const char* aMsg,
                                       const char* aTargetDescr,
-                                      Accessible* aTarget) {
+                                      LocalAccessible* aTarget) {
   MsgBegin(sFocusTitle, "%s", aMsg);
   AccessibleNNode(aTargetDescr, aTarget);
   MsgEnd();
@@ -558,25 +565,26 @@ void logging::FocusNotificationTarget(const char* aMsg,
 
   if (aTargetThing) {
     nsCOMPtr<nsINode> targetNode(do_QueryInterface(aTargetThing));
-    if (targetNode)
+    if (targetNode) {
       AccessibleNNode(aTargetDescr, targetNode);
-    else
+    } else {
       printf("    %s: %p, window\n", aTargetDescr,
              static_cast<void*>(aTargetThing));
+    }
   }
 
   MsgEnd();
 }
 
 void logging::ActiveItemChangeCausedBy(const char* aCause,
-                                       Accessible* aTarget) {
+                                       LocalAccessible* aTarget) {
   SubMsgBegin();
   printf("    Caused by: %s\n", aCause);
   AccessibleNNode("Item", aTarget);
   SubMsgEnd();
 }
 
-void logging::ActiveWidget(Accessible* aWidget) {
+void logging::ActiveWidget(LocalAccessible* aWidget) {
   SubMsgBegin();
 
   AccessibleNNode("Widget", aWidget);
@@ -587,7 +595,7 @@ void logging::ActiveWidget(Accessible* aWidget) {
   SubMsgEnd();
 }
 
-void logging::FocusDispatched(Accessible* aTarget) {
+void logging::FocusDispatched(LocalAccessible* aTarget) {
   SubMsgBegin();
   AccessibleNNode("A11y target", aTarget);
   SubMsgEnd();
@@ -598,12 +606,13 @@ void logging::SelChange(dom::Selection* aSelection, DocAccessible* aDocument,
   SelectionType type = aSelection->GetType();
 
   const char* strType = 0;
-  if (type == SelectionType::eNormal)
+  if (type == SelectionType::eNormal) {
     strType = "normal";
-  else if (type == SelectionType::eSpellCheck)
+  } else if (type == SelectionType::eSpellCheck) {
     strType = "spellcheck";
-  else
+  } else {
     strType = "unknown";
+  }
 
   bool isIgnored = !aDocument || !aDocument->IsContentLoaded();
   printf(
@@ -619,11 +628,11 @@ void logging::TreeInfo(const char* aMsg, uint32_t aExtraFlags, ...) {
     va_start(vl, aExtraFlags);
     const char* descr = va_arg(vl, const char*);
     if (descr) {
-      Accessible* acc = va_arg(vl, Accessible*);
+      LocalAccessible* acc = va_arg(vl, LocalAccessible*);
       MsgBegin("TREE", "%s; doc: %p", aMsg, acc ? acc->Document() : nullptr);
       AccessibleInfo(descr, acc);
       while ((descr = va_arg(vl, const char*))) {
-        AccessibleInfo(descr, va_arg(vl, Accessible*));
+        AccessibleInfo(descr, va_arg(vl, LocalAccessible*));
       }
     } else {
       MsgBegin("TREE", "%s", aMsg);
@@ -638,12 +647,13 @@ void logging::TreeInfo(const char* aMsg, uint32_t aExtraFlags, ...) {
 }
 
 void logging::TreeInfo(const char* aMsg, uint32_t aExtraFlags,
-                       const char* aMsg1, Accessible* aAcc, const char* aMsg2,
-                       nsINode* aNode) {
+                       const char* aMsg1, LocalAccessible* aAcc,
+                       const char* aMsg2, nsINode* aNode) {
   if (IsEnabledAll(logging::eTree | aExtraFlags)) {
     MsgBegin("TREE", "%s; doc: %p", aMsg, aAcc ? aAcc->Document() : nullptr);
     AccessibleInfo(aMsg1, aAcc);
-    Accessible* acc = aAcc ? aAcc->Document()->GetAccessible(aNode) : nullptr;
+    LocalAccessible* acc =
+        aAcc ? aAcc->Document()->GetAccessible(aNode) : nullptr;
     if (acc) {
       AccessibleInfo(aMsg2, acc);
     } else {
@@ -654,31 +664,32 @@ void logging::TreeInfo(const char* aMsg, uint32_t aExtraFlags,
 }
 
 void logging::TreeInfo(const char* aMsg, uint32_t aExtraFlags,
-                       Accessible* aParent) {
+                       LocalAccessible* aParent) {
   if (IsEnabledAll(logging::eTree | aExtraFlags)) {
     MsgBegin("TREE", "%s; doc: %p", aMsg, aParent->Document());
     AccessibleInfo("container", aParent);
     for (uint32_t idx = 0; idx < aParent->ChildCount(); idx++) {
-      AccessibleInfo("child", aParent->GetChildAt(idx));
+      AccessibleInfo("child", aParent->LocalChildAt(idx));
     }
     MsgEnd();
   }
 }
 
-void logging::Tree(const char* aTitle, const char* aMsgText, Accessible* aRoot,
-                   GetTreePrefix aPrefixFunc, void* aGetTreePrefixData) {
+void logging::Tree(const char* aTitle, const char* aMsgText,
+                   LocalAccessible* aRoot, GetTreePrefix aPrefixFunc,
+                   void* aGetTreePrefixData) {
   logging::MsgBegin(aTitle, "%s", aMsgText);
 
   nsAutoString level;
-  Accessible* root = aRoot;
+  LocalAccessible* root = aRoot;
   do {
     const char* prefix =
         aPrefixFunc ? aPrefixFunc(aGetTreePrefixData, root) : "";
     printf("%s", NS_ConvertUTF16toUTF8(level).get());
     logging::AccessibleInfo(prefix, root);
-    if (root->FirstChild() && !root->FirstChild()->IsDoc()) {
+    if (root->LocalFirstChild() && !root->LocalFirstChild()->IsDoc()) {
       level.AppendLiteral(u"  ");
-      root = root->FirstChild();
+      root = root->LocalFirstChild();
       continue;
     }
     int32_t idxInParent = root != aRoot && root->mParent
@@ -690,7 +701,7 @@ void logging::Tree(const char* aTitle, const char* aMsgText, Accessible* aRoot,
       root = root->mParent->mChildren.ElementAt(idxInParent + 1);
       continue;
     }
-    while (root != aRoot && (root = root->Parent())) {
+    while (root != aRoot && (root = root->LocalParent())) {
       level.Cut(0, 2);
       int32_t idxInParent = !root->IsDoc() && root->mParent
                                 ? root->mParent->mChildren.IndexOf(root)
@@ -735,6 +746,49 @@ void logging::DOMTree(const char* aTitle, const char* aMsgText,
   logging::MsgEnd();
 }
 
+void logging::TreeSize(const char* aTitle, const char* aMsgText,
+                       LocalAccessible* aRoot) {
+  logging::MsgBegin(aTitle, "%s", aMsgText);
+  logging::AccessibleInfo("Logging tree size from: ", aRoot);
+  size_t b = 0;
+  size_t n = 0;
+  LocalAccessible* root = aRoot;
+  do {
+    // Process the current acc
+    b += AccessibleLoggingMallocSizeOf(root);
+    n++;
+
+    // Get next acc
+    if (root->LocalFirstChild() && !root->LocalFirstChild()->IsDoc()) {
+      root = root->LocalFirstChild();
+      continue;
+    }
+    int32_t idxInParent = root != aRoot && root->mParent
+                              ? root->mParent->mChildren.IndexOf(root)
+                              : -1;
+    if (idxInParent != -1 &&
+        idxInParent <
+            static_cast<int32_t>(root->mParent->mChildren.Length() - 1)) {
+      root = root->mParent->mChildren.ElementAt(idxInParent + 1);
+      continue;
+    }
+    while (root != aRoot && (root = root->LocalParent())) {
+      int32_t idxInParent = !root->IsDoc() && root->mParent
+                                ? root->mParent->mChildren.IndexOf(root)
+                                : -1;
+      if (idxInParent != -1 &&
+          idxInParent <
+              static_cast<int32_t>(root->mParent->mChildren.Length() - 1)) {
+        root = root->mParent->mChildren.ElementAt(idxInParent + 1);
+        break;
+      }
+    }
+  } while (root && root != aRoot);
+
+  printf("\nTree contains %zu accessibles and is %zu bytes\n", n, b);
+  logging::MsgEnd();
+}
+
 void logging::MsgBegin(const char* aTitle, const char* aMsgText, ...) {
   printf("\nA11Y %s: ", aTitle);
 
@@ -771,7 +825,7 @@ void logging::MsgEntry(const char* aEntryText, ...) {
 
 void logging::Text(const char* aText) { printf("  %s\n", aText); }
 
-void logging::Address(const char* aDescr, Accessible* aAcc) {
+void logging::Address(const char* aDescr, LocalAccessible* aAcc) {
   if (!aAcc->IsDoc()) {
     printf("    %s accessible: %p, node: %p\n", aDescr,
            static_cast<void*>(aAcc), static_cast<void*>(aAcc->GetNode()));
@@ -806,7 +860,7 @@ void logging::Document(DocAccessible* aDocument) {
   printf("\n");
 }
 
-void logging::AccessibleInfo(const char* aDescr, Accessible* aAccessible) {
+void logging::AccessibleInfo(const char* aDescr, LocalAccessible* aAccessible) {
   printf("    %s: %p; ", aDescr, static_cast<void*>(aAccessible));
   if (!aAccessible) {
     printf("\n");
@@ -838,7 +892,8 @@ void logging::AccessibleInfo(const char* aDescr, Accessible* aAccessible) {
   printf(", node: %s\n", NS_ConvertUTF16toUTF8(nodeDesc).get());
 }
 
-void logging::AccessibleNNode(const char* aDescr, Accessible* aAccessible) {
+void logging::AccessibleNNode(const char* aDescr,
+                              LocalAccessible* aAccessible) {
   printf("    %s: %p; ", aDescr, static_cast<void*>(aAccessible));
   if (!aAccessible) return;
 
@@ -862,7 +917,7 @@ void logging::AccessibleNNode(const char* aDescr, nsINode* aNode) {
       GetAccService()->GetDocAccessible(aNode->OwnerDoc());
 
   if (document) {
-    Accessible* accessible = document->GetAccessible(aNode);
+    LocalAccessible* accessible = document->GetAccessible(aNode);
     if (accessible) {
       AccessibleNNode(aDescr, accessible);
       return;
@@ -894,7 +949,7 @@ void logging::DOMEvent(const char* aDescr, nsINode* aOrigTarget,
 void logging::Stack() {
   if (IsEnabled(eStack)) {
     printf("  stack: \n");
-    nsTraceRefcnt::WalkTheStack(stdout);
+    MozWalkTheStack(stdout);
   }
 }
 
@@ -909,8 +964,9 @@ bool logging::IsEnabledAll(uint32_t aModules) {
 
 bool logging::IsEnabled(const nsAString& aModuleStr) {
   for (unsigned int idx = 0; idx < ArrayLength(sModuleMap); idx++) {
-    if (aModuleStr.EqualsASCII(sModuleMap[idx].mStr))
+    if (aModuleStr.EqualsASCII(sModuleMap[idx].mStr)) {
       return sModules & sModuleMap[idx].mModule;
+    }
   }
 
   return false;
