@@ -1,19 +1,24 @@
 /* exported MANAGE_ADDRESSES_DIALOG_URL, MANAGE_CREDIT_CARDS_DIALOG_URL, EDIT_ADDRESS_DIALOG_URL, EDIT_CREDIT_CARD_DIALOG_URL,
             BASE_URL, TEST_ADDRESS_1, TEST_ADDRESS_2, TEST_ADDRESS_3, TEST_ADDRESS_4, TEST_ADDRESS_5, TEST_ADDRESS_CA_1, TEST_ADDRESS_DE_1,
             TEST_ADDRESS_IE_1,
-            TEST_CREDIT_CARD_1, TEST_CREDIT_CARD_2, TEST_CREDIT_CARD_3, FORM_URL, CREDITCARD_FORM_URL, CREDITCARD_FORM_IFRAME_URL
-            FTU_PREF, ENABLED_AUTOFILL_ADDRESSES_PREF, AUTOFILL_CREDITCARDS_AVAILABLE_PREF, ENABLED_AUTOFILL_CREDITCARDS_PREF,
+            TEST_CREDIT_CARD_1, TEST_CREDIT_CARD_2, TEST_CREDIT_CARD_3, TEST_CREDIT_CARD_4, TEST_CREDIT_CARD_5,
+            FORM_URL, CREDITCARD_FORM_URL, CREDITCARD_FORM_IFRAME_URL
+            FTU_PREF, ENABLED_AUTOFILL_ADDRESSES_PREF, ENABLED_AUTOFILL_ADDRESSES_CAPTURE_PREF, AUTOFILL_CREDITCARDS_AVAILABLE_PREF, ENABLED_AUTOFILL_CREDITCARDS_PREF,
             SUPPORTED_COUNTRIES_PREF,
             SYNC_USERNAME_PREF, SYNC_ADDRESSES_PREF, SYNC_CREDITCARDS_PREF, SYNC_CREDITCARDS_AVAILABLE_PREF, CREDITCARDS_USED_STATUS_PREF,
-            sleep, expectPopupOpen, openPopupOn, openPopupForSubframe, expectPopupClose, closePopup, closePopupForSubframe,
+            sleep, expectPopupOpen, openPopupOn, openPopupForSubframe, closePopup, closePopupForSubframe,
             clickDoorhangerButton, getAddresses, saveAddress, removeAddresses, saveCreditCard,
-            getDisplayedPopupItems, getDoorhangerCheckbox,
+            getDisplayedPopupItems, getDoorhangerCheckbox, waitForPopupEnabled,
             getNotification, getDoorhangerButton, removeAllRecords, expectWarningText, testDialog */
 
 "use strict";
 
-ChromeUtils.import("resource://gre/modules/OSKeyStore.jsm", this);
-ChromeUtils.import("resource://testing-common/OSKeyStoreTestUtils.jsm", this);
+const { OSKeyStore } = ChromeUtils.import(
+  "resource://gre/modules/OSKeyStore.jsm"
+);
+const { OSKeyStoreTestUtils } = ChromeUtils.import(
+  "resource://testing-common/OSKeyStoreTestUtils.jsm"
+);
 
 const MANAGE_ADDRESSES_DIALOG_URL =
   "chrome://formautofill/content/manageAddresses.xhtml";
@@ -40,6 +45,8 @@ const FTU_PREF = "extensions.formautofill.firstTimeUse";
 const CREDITCARDS_USED_STATUS_PREF = "extensions.formautofill.creditCards.used";
 const ENABLED_AUTOFILL_ADDRESSES_PREF =
   "extensions.formautofill.addresses.enabled";
+const ENABLED_AUTOFILL_ADDRESSES_CAPTURE_PREF =
+  "extensions.formautofill.addresses.capture.enabled";
 const AUTOFILL_CREDITCARDS_AVAILABLE_PREF =
   "extensions.formautofill.creditCards.available";
 const ENABLED_AUTOFILL_CREDITCARDS_PREF =
@@ -154,6 +161,17 @@ const TEST_CREDIT_CARD_3 = {
   "cc-type": "mastercard",
 };
 
+const TEST_CREDIT_CARD_4 = {
+  "cc-number": "5105105105105100",
+  "cc-type": "mastercard",
+};
+
+const TEST_CREDIT_CARD_5 = {
+  "cc-name": "Chris P. Bacon",
+  "cc-number": "4012888888881881",
+  "cc-type": "visa",
+};
+
 const MAIN_BUTTON = "button";
 const SECONDARY_BUTTON = "secondaryButton";
 const MENU_BUTTON = "menubutton";
@@ -182,7 +200,7 @@ async function focusAndWaitForFieldsIdentified(browserOrContext, selector) {
   /* eslint no-shadow: ["error", { "allow": ["selector", "previouslyFocused", "previouslyIdentified"] }] */
 
   const { FormAutofillParent } = ChromeUtils.import(
-    "resource://formautofill/FormAutofillParent.jsm"
+    "resource://autofill/FormAutofillParent.jsm"
   );
 
   // If the input is previously focused, no more notifications will be
@@ -265,64 +283,134 @@ async function focusAndWaitForFieldsIdentified(browserOrContext, selector) {
 
 async function expectPopupOpen(browser) {
   info("expectPopupOpen");
-  const { autoCompletePopup } = browser;
-  await BrowserTestUtils.waitForCondition(
-    () => autoCompletePopup.popupOpen,
-    "popup should be open"
+  await BrowserTestUtils.waitForPopupEvent(browser.autoCompletePopup, "shown");
+  await BrowserTestUtils.waitForMutationCondition(
+    browser.autoCompletePopup.richlistbox,
+    { childList: true, subtree: true, attributes: true },
+    () => {
+      const listItemElems = getDisplayedPopupItems(browser);
+      return (
+        !![...listItemElems].length &&
+        [...listItemElems].every(item => {
+          return (
+            (item.getAttribute("originaltype") == "autofill-profile" ||
+              item.getAttribute("originaltype") == "autofill-insecureWarning" ||
+              item.getAttribute("originaltype") == "autofill-footer") &&
+            item.hasAttribute("formautofillattached")
+          );
+        })
+      );
+    }
   );
-  await BrowserTestUtils.waitForCondition(() => {
-    const listItemElems = getDisplayedPopupItems(browser);
-    return (
-      !![...listItemElems].length &&
-      [...listItemElems].every(item => {
-        return (
-          (item.getAttribute("originaltype") == "autofill-profile" ||
-            item.getAttribute("originaltype") == "autofill-insecureWarning" ||
-            item.getAttribute("originaltype") == "autofill-footer") &&
-          item.hasAttribute("formautofillattached")
-        );
-      })
-    );
-  }, "The popup should be a form autofill one");
+}
+
+async function waitForPopupEnabled(browser) {
+  const {
+    autoCompletePopup: { richlistbox: itemsBox },
+  } = browser;
+  info("Wait for list elements to become enabled");
+  await BrowserTestUtils.waitForMutationCondition(
+    itemsBox,
+    { subtree: true, attributes: true, attributeFilter: ["disabled"] },
+    () => !itemsBox.querySelectorAll(".autocomplete-richlistitem")[0].disabled
+  );
+}
+
+// Wait for the popup state change notification to happen in a child process.
+function waitPopupStateInChild(bc, messageName) {
+  return SpecialPowers.spawn(bc, [messageName], expectedMessage => {
+    return new Promise(resolve => {
+      const { AutoCompleteChild } = ChromeUtils.import(
+        "resource://gre/actors/AutoCompleteChild.jsm"
+      );
+
+      let listener = {
+        popupStateChanged: name => {
+          if (name != expectedMessage) {
+            info("Expected " + expectedMessage + " but received " + name);
+            return;
+          }
+
+          AutoCompleteChild.removePopupStateListener(listener);
+          resolve();
+        },
+      };
+      AutoCompleteChild.addPopupStateListener(listener);
+    });
+  });
 }
 
 async function openPopupOn(browser, selector) {
+  let childNotifiedPromise = waitPopupStateInChild(
+    browser,
+    "FormAutoComplete:PopupOpened"
+  );
   await SimpleTest.promiseFocus(browser);
   await focusAndWaitForFieldsIdentified(browser, selector);
-  info("openPopupOn: before VK_DOWN");
-  await BrowserTestUtils.synthesizeKey("VK_DOWN", {}, browser);
+  if (!selector.includes("cc-")) {
+    info(`openPopupOn: before VK_DOWN on ${selector}`);
+    await BrowserTestUtils.synthesizeKey("VK_DOWN", {}, browser);
+  }
   await expectPopupOpen(browser);
+  await childNotifiedPromise;
 }
 
 async function openPopupForSubframe(browser, frameBrowsingContext, selector) {
+  let childNotifiedPromise = waitPopupStateInChild(
+    frameBrowsingContext,
+    "FormAutoComplete:PopupOpened"
+  );
+
   await SimpleTest.promiseFocus(browser);
   await focusAndWaitForFieldsIdentified(frameBrowsingContext, selector);
-  info("openPopupOn: before VK_DOWN");
-  await BrowserTestUtils.synthesizeKey("VK_DOWN", {}, frameBrowsingContext);
+  if (!selector.includes("cc-")) {
+    info(`openPopupForSubframe: before VK_DOWN on ${selector}`);
+    await BrowserTestUtils.synthesizeKey("VK_DOWN", {}, frameBrowsingContext);
+  }
   await expectPopupOpen(browser);
-}
-
-async function expectPopupClose(browser) {
-  await BrowserTestUtils.waitForCondition(
-    () => !browser.autoCompletePopup.popupOpen,
-    "popup should have closed"
-  );
+  await childNotifiedPromise;
 }
 
 async function closePopup(browser) {
+  // Return if the popup isn't open.
+  if (!browser.autoCompletePopup.popupOpen) {
+    return;
+  }
+
+  let childNotifiedPromise = waitPopupStateInChild(
+    browser,
+    "FormAutoComplete:PopupClosed"
+  );
+  let popupClosePromise = BrowserTestUtils.waitForPopupEvent(
+    browser.autoCompletePopup,
+    "hidden"
+  );
+
   await SpecialPowers.spawn(browser, [], async function() {
     content.document.activeElement.blur();
   });
 
-  await expectPopupClose(browser);
+  await popupClosePromise;
+  await childNotifiedPromise;
 }
 
 async function closePopupForSubframe(browser, frameBrowsingContext) {
+  let childNotifiedPromise = waitPopupStateInChild(
+    browser,
+    "FormAutoComplete:PopupClosed"
+  );
+
+  let popupClosePromise = BrowserTestUtils.waitForPopupEvent(
+    browser.autoCompletePopup,
+    "hidden"
+  );
+
   await SpecialPowers.spawn(frameBrowsingContext, [], async function() {
     content.document.activeElement.blur();
   });
 
-  await expectPopupClose(browser);
+  await popupClosePromise;
+  await childNotifiedPromise;
 }
 
 function emulateMessageToBrowser(name, data) {
@@ -462,9 +550,11 @@ async function expectWarningText(browser, expectedText) {
   }
   warningBox = warningBox._warningTextBox;
 
-  await BrowserTestUtils.waitForCondition(() => {
-    return warningBox.textContent == expectedText;
-  }, `Waiting for expected warning text: ${expectedText}, Got ${warningBox.textContent}`);
+  await BrowserTestUtils.waitForMutationCondition(
+    warningBox,
+    { childList: true, characterData: true },
+    () => warningBox.textContent == expectedText
+  );
   ok(true, `Got expected warning text: ${expectedText}`);
 }
 
