@@ -2,8 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/* import-globals-from item-editing/calendar-item-editing.js */
 /* import-globals-from calendar-command-controller.js */
-/* import-globals-from calendar-item-editing.js */
 /* import-globals-from calendar-management.js */
 /* import-globals-from calendar-ui-utils.js */
 /* import-globals-from calendar-views-utils.js */
@@ -222,7 +222,7 @@ agendaListbox.onKeyPress = function(aEvent) {
 agendaListbox.editSelectedItem = function() {
   let listItem = document.getElementById("agenda-listbox").selectedItem;
   if (listItem) {
-    modifyEventWithDialog(listItem.occurrence, null, true);
+    openEventDialogForViewing(listItem.occurrence);
   }
 };
 
@@ -309,7 +309,7 @@ agendaListbox.addItemBefore = function(aNewItem, aAgendaItem, aPeriod, visible)
  * @return              The newly created XUL element.
  */
 agendaListbox.addItem = function(aItem) {
-  if (!cal.item.isEvent(aItem)) {
+  if (!aItem.isEvent()) {
     return null;
   }
   aItem.QueryInterface(Ci.calIEvent);
@@ -322,35 +322,29 @@ agendaListbox.addItem = function(aItem) {
     let period = periods[i];
     let complistItem = period.listItem;
     let visible = complistItem.getCheckbox().checked;
-    if (aItem.startDate.isDate && period.duration == 1 && aItem.duration.days == 1) {
-      if (this.getListItems(aItem, period).length == 0) {
-        this.addItemBefore(aItem, period.listItem.nextElementSibling, period, visible);
-      }
-    } else {
-      do {
-        complistItem = complistItem.nextElementSibling;
-        if (this.isEventListItem(complistItem)) {
-          let compitem = complistItem.occurrence;
-          if (this.isSameEvent(aItem, compitem)) {
-            // The same event occurs on several calendars but we only
-            // display the first one.
-            // TODO: find a way to display this special circumstance
-            break;
-          } else if (this.isBefore(aItem, compitem, period)) {
-            if (this.isSameEvent(aItem, compitem)) {
-              newlistItem = this.addItemBefore(aItem, complistItem, period, visible);
-              break;
-            } else {
-              newlistItem = this.addItemBefore(aItem, complistItem, period, visible);
-              break;
-            }
-          }
-        } else {
-          newlistItem = this.addItemBefore(aItem, complistItem, period, visible);
+    do {
+      complistItem = complistItem.nextElementSibling;
+      if (this.isEventListItem(complistItem)) {
+        let compitem = complistItem.occurrence;
+        if (this.isSameEvent(aItem, compitem)) {
+          // The same event occurs on several calendars but we only
+          // display the first one.
+          // TODO: find a way to display this special circumstance
           break;
+        } else if (this.isBefore(aItem, compitem, period)) {
+          if (this.isSameEvent(aItem, compitem)) {
+            newlistItem = this.addItemBefore(aItem, complistItem, period, visible);
+            break;
+          } else {
+            newlistItem = this.addItemBefore(aItem, complistItem, period, visible);
+            break;
+          }
         }
-      } while (complistItem);
-    }
+      } else {
+        newlistItem = this.addItemBefore(aItem, complistItem, period, visible);
+        break;
+      }
+    } while (complistItem);
   }
   return newlistItem;
 };
@@ -366,8 +360,10 @@ agendaListbox.addItem = function(aItem) {
 agendaListbox.isBefore = function(aItem, aCompItem, aPeriod) {
   let itemDate = this.comparisonDate(aItem, aPeriod);
   let compItemDate = this.comparisonDate(aCompItem, aPeriod);
-  let itemDateEndDate = itemDate.clone();
-  itemDateEndDate.day++;
+
+  let calendarSortOrder = cal.view.calendarSortOrder;
+  let itemCalendarIndex = calendarSortOrder.indexOf(aItem.calendar.id);
+  let compItemCalendarIndex = calendarSortOrder.indexOf(aCompItem.calendar.id);
 
   if (compItemDate.day == itemDate.day) {
     // In the same day the order is:
@@ -377,19 +373,26 @@ agendaListbox.isBefore = function(aItem, aCompItem, aPeriod) {
     //   time) and intermediate.
     if (itemDate.isDate && aItem.duration.days == 1) {
       // all-day events with duration one day
+      if (compItemDate.isDate && aCompItem.duration.days == 1) {
+        return itemCalendarIndex < compItemCalendarIndex;
+      }
       return true;
     } else if (itemDate.isDate) {
       if (aItem.startDate.compare(itemDate) == 0) {
-        // starting day of an all-day events spannig multiple days
+        // starting day of an all-day events spanning multiple days
         return !compItemDate.isDate || aCompItem.duration.days != 1;
-      } else if (aItem.endDate.compare(itemDateEndDate) == 0) {
-        // ending day of an all-day events spannig multiple days
+      }
+
+      let itemDateEndDate = itemDate.clone();
+      itemDateEndDate.day++;
+      if (aItem.endDate.compare(itemDateEndDate) == 0) {
+        // ending day of an all-day events spanning multiple days
         return (
           !compItemDate.isDate ||
           (aCompItem.duration.days != 1 && aCompItem.startDate.compare(compItemDate) != 0)
         );
       }
-      // intermediate day of an all-day events spannig multiple days
+      // intermediate day of an all-day events spanning multiple days
       return !compItemDate.isDate;
     } else if (aCompItem.startDate.isDate) {
       return false;
@@ -402,6 +405,9 @@ agendaListbox.isBefore = function(aItem, aCompItem, aPeriod) {
     comp = aItem.startDate.compare(aCompItem.startDate);
     if (comp == 0) {
       comp = aItem.endDate.compare(aCompItem.endDate);
+      if (comp == 0) {
+        comp = itemCalendarIndex - compItemCalendarIndex;
+      }
     }
   }
   return comp <= 0;
@@ -442,7 +448,7 @@ agendaListbox.comparisonDate = function(aItem, aPeriod) {
     // it starts before the given period
     return endDateToReturn;
   }
-  return aItem.startDate.clone();
+  return aItem.startDate.getInTimezone(cal.dtz.defaultTimezone);
 };
 
 /**
@@ -486,7 +492,7 @@ agendaListbox.getListItems = function(aItem, aPeriod) {
  *                            sibling that is not an period item.
  * @return                  Returns true if the removed item was selected.
  */
-agendaListbox.deleteItem = function(aItem, aMoveSelection) {
+agendaListbox.removeItem = function(aItem, aMoveSelection) {
   let isSelected = false;
   let listItems = this.getListItems(aItem);
   if (listItems.length > 0) {
@@ -508,9 +514,9 @@ agendaListbox.deleteItem = function(aItem, aMoveSelection) {
 /**
  * Remove all items belonging to the specified calendar.
  *
- * @param aCalendar         The item to compare.
+ * @param aCalendar         The calendar to remove items from.
  */
-agendaListbox.deleteItemsFromCalendar = function(aCalendar) {
+agendaListbox.removeItemsFromCalendar = function(aCalendar) {
   let children = Array.from(this.agendaListboxControl.children);
   for (let childNode of children) {
     if (childNode && childNode.occurrence && childNode.occurrence.calendar.id == aCalendar.id) {
@@ -568,7 +574,7 @@ agendaListbox.deleteSelectedItem = function(aDoNotConfirm) {
  * @param aEvent            The DOM event that targets the period.
  */
 agendaListbox.createNewEvent = function(aEvent) {
-  if (!this.isEventListItem(aEvent.explicitOriginalTarget)) {
+  if (!this.isEventListItem(aEvent.target)) {
     // Create new event for the date currently displayed in the agenda. Setting
     // isDate = true automatically makes the start time be the next full hour.
     let eventStart = agendaListbox.today.start.clone();
@@ -589,7 +595,7 @@ agendaListbox.setupContextMenu = function(popup) {
   let enabled = this.isEventListItem(listItem);
   let menuitems = popup.children;
   for (let i = 0; i < menuitems.length; i++) {
-    setBooleanAttribute(menuitems[i], "disabled", !enabled);
+    menuitems[i].disabled = !enabled;
   }
 
   let menu = document.getElementById("calendar-today-pane-menu-attendance-menu");
@@ -607,7 +613,7 @@ agendaListbox.setupContextMenu = function(popup) {
  */
 agendaListbox.refreshCalendarQuery = function(aStart, aEnd, aCalendar) {
   let refreshJob = {
-    QueryInterface: ChromeUtils.generateQI([Ci.calIOperationListener]),
+    QueryInterface: ChromeUtils.generateQI(["calIOperationListener"]),
     agendaListbox: this,
     calendar: null,
     calId: null,
@@ -635,7 +641,7 @@ agendaListbox.refreshCalendarQuery = function(aStart, aEnd, aCalendar) {
 
     cancel() {
       this.cancelled = true;
-      let operation = cal.wrapInstance(this.operation, Ci.calIOperation);
+      let { operation } = this;
       if (operation && operation.isPending) {
         operation.cancel();
         this.operation = null;
@@ -643,20 +649,16 @@ agendaListbox.refreshCalendarQuery = function(aStart, aEnd, aCalendar) {
     },
 
     execute() {
-      if (!(aStart || aEnd || aCalendar)) {
+      if (!(aStart || aEnd || (aCalendar && aCalendar.type != "composite"))) {
         this.agendaListbox.removeListItems();
       }
 
       if (!aCalendar) {
         aCalendar = this.agendaListbox.calendar;
       }
-      if (!aStart) {
-        aStart = this.agendaListbox.getStart();
-      }
-      if (!aEnd) {
-        aEnd = this.agendaListbox.getEnd();
-      }
-      if (!(aStart || aEnd || aCalendar)) {
+      let start = aStart || this.agendaListbox.getStart();
+      let end = aEnd || this.agendaListbox.getEnd();
+      if (!(start || end || aCalendar)) {
         return;
       }
 
@@ -669,6 +671,8 @@ agendaListbox.refreshCalendarQuery = function(aStart, aEnd, aCalendar) {
         }
         this.agendaListbox.mPendingRefreshJobs.clear();
       } else {
+        cal.ASSERT(!aStart && !aEnd, "refreshCalendarQuery called with date(s) and a calendar set");
+        this.agendaListbox.removeItemsFromCalendar(aCalendar);
         this.calId = aCalendar.id;
         if (this.agendaListbox.mPendingRefreshJobs.has(this.calId)) {
           this.agendaListbox.mPendingRefreshJobs.get(this.calId).cancel();
@@ -679,8 +683,7 @@ agendaListbox.refreshCalendarQuery = function(aStart, aEnd, aCalendar) {
 
       let filter =
         this.calendar.ITEM_FILTER_CLASS_OCCURRENCES | this.calendar.ITEM_FILTER_TYPE_EVENT;
-      let operation = this.calendar.getItems(filter, 0, aStart, aEnd, this);
-      operation = cal.wrapInstance(operation, Ci.calIOperation);
+      let operation = this.calendar.getItems(filter, 0, start, end, this);
       if (operation && operation.isPending) {
         this.operation = operation;
         this.agendaListbox.mPendingRefreshJobs.set(this.calId, this);
@@ -861,27 +864,41 @@ agendaListbox.calendarOpListener = { agendaListbox };
  * @see calIObserver
  * @see calICompositeObserver
  */
-agendaListbox.calendarObserver = { agendaListbox };
+agendaListbox.calendarObserver = {
+  agendaListbox,
+  calendarsInBatch: new Set(),
+};
 
-agendaListbox.calendarObserver.QueryInterface = cal.generateQI([
-  Ci.calIObserver,
-  Ci.calICompositeObserver,
+agendaListbox.calendarObserver.QueryInterface = ChromeUtils.generateQI([
+  "calIObserver",
+  "calICompositeObserver",
 ]);
 
 // calIObserver:
-agendaListbox.calendarObserver.onStartBatch = function() {
-  this.needsRefresh = true;
+agendaListbox.calendarObserver.onStartBatch = function(calendar) {
+  if (calendar.type != "composite") {
+    this.calendarsInBatch.add(calendar);
+  }
 };
 
-agendaListbox.calendarObserver.onEndBatch = function() {
-  this.needsRefresh = false;
-  this.agendaListbox.refreshCalendarQuery();
+agendaListbox.calendarObserver.onEndBatch = function(calendar) {
+  if (calendar.type != "composite") {
+    this.calendarsInBatch.delete(calendar);
+  }
 };
 
-agendaListbox.calendarObserver.onLoad = function() {};
+agendaListbox.calendarObserver.onLoad = function(calendar) {
+  if (calendar.type != "composite") {
+    this.agendaListbox.refreshCalendarQuery(null, null, calendar);
+  }
+};
 
 agendaListbox.calendarObserver.onAddItem = function(item) {
-  if (this.needsRefresh || !cal.item.isEvent(item)) {
+  if (
+    item.calendar.type == "composite" ||
+    this.calendarsInBatch.has(item.calendar) ||
+    !item.isEvent()
+  ) {
     return;
   }
   // get all sub items if it is a recurring item
@@ -905,14 +922,14 @@ agendaListbox.calendarObserver.onDeleteItem = function(item, rebuildFlag) {
 };
 
 agendaListbox.calendarObserver.onLocalDeleteItem = function(item, moveSelection) {
-  if (!cal.item.isEvent(item)) {
+  if (!item.isEvent()) {
     return false;
   }
   let selectedItemHashId = -1;
   // get all sub items if it is a recurring item
   let occs = this.getOccurrencesBetween(item);
   for (let i = 0; i < occs.length; i++) {
-    let isSelected = this.agendaListbox.deleteItem(occs[i], moveSelection);
+    let isSelected = this.agendaListbox.removeItem(occs[i], moveSelection);
     if (isSelected) {
       selectedItemHashId = occs[i].hashId;
     }
@@ -921,8 +938,12 @@ agendaListbox.calendarObserver.onLocalDeleteItem = function(item, moveSelection)
 };
 
 agendaListbox.calendarObserver.onModifyItem = function(newItem, oldItem) {
+  if (newItem.calendar.type == "composite" || this.calendarsInBatch.has(newItem.calendar)) {
+    return;
+  }
+
   let selectedItemHashId = this.onLocalDeleteItem(oldItem, false);
-  if (!cal.item.isEvent(newItem)) {
+  if (!newItem.isEvent()) {
     return;
   }
   this.onAddItem(newItem);
@@ -942,20 +963,10 @@ agendaListbox.calendarObserver.onError = function(_cal, errno, msg) {};
 agendaListbox.calendarObserver.onPropertyChanged = function(aCalendar, aName, aValue, aOldValue) {
   switch (aName) {
     case "disabled":
-      this.agendaListbox.refreshCalendarQuery();
-      break;
-    case "color":
-      for (
-        let node = agendaListbox.agendaListboxControl.firstElementChild;
-        node;
-        node = node.nextElementSibling
-      ) {
-        // Change color on all nodes that don't do so themselves, which
-        // is currently only he agenda-richlistitem
-        if (node.localName != "agenda-richlistitem") {
-          continue;
-        }
-        node.refreshColor();
+      if (aValue) {
+        this.agendaListbox.removeItemsFromCalendar(aCalendar);
+      } else {
+        this.agendaListbox.refreshCalendarQuery(null, null, aCalendar);
       }
       break;
   }
@@ -967,7 +978,7 @@ agendaListbox.calendarObserver.onPropertyDeleting = function(aCalendar, aName) {
 
 agendaListbox.calendarObserver.onCalendarRemoved = function(aCalendar) {
   if (!aCalendar.getProperty("disabled")) {
-    this.agendaListbox.deleteItemsFromCalendar(aCalendar);
+    this.agendaListbox.removeItemsFromCalendar(aCalendar);
   }
 };
 
@@ -1103,7 +1114,7 @@ function scheduleNextCurrentEventUpdate(aRefreshCallback, aMsUntil) {
 
 /**
  * Gets a right value for calendar.agendaListbox.soondays preference, avoid
- * erroneous values edited in the lightning.js preference file
+ * erroneous values edited in the calendar.js preference file
  **/
 function getSoondaysPreference() {
   let prefName = "calendar.agendaListbox.soondays";
