@@ -2,8 +2,8 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+/* import-globals-from item-editing/calendar-item-editing.js */
 /* import-globals-from calendar-command-controller.js */
-/* import-globals-from calendar-item-editing.js */
 /* import-globals-from calendar-management.js */
 /* import-globals-from calendar-views-utils.js */
 
@@ -218,9 +218,9 @@ function pasteFromClipboard() {
       } else {
         let pasteText = "paste";
         if (withAttendees.length) {
-          if (withAttendees.every(cal.item.isEvent)) {
+          if (withAttendees.every(item => item.isEvent())) {
             pasteText += "Event";
-          } else if (withAttendees.every(cal.item.isToDo)) {
+          } else if (withAttendees.every(item => item.isTodo())) {
             pasteText += "Task";
           } else {
             pasteText += "Item";
