@@ -13,13 +13,15 @@ var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
 
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalAlarm: "resource:///modules/CalAlarm.jsm",
+});
+
 var allowedActionsMap = {};
 var suppressListUpdate = false;
 
-const gNotification = {};
-XPCOMUtils.defineLazyGetter(gNotification, "notificationbox", () => {
+XPCOMUtils.defineLazyGetter(this, "gReminderNotification", () => {
   return new MozElements.NotificationBox(element => {
-    element.setAttribute("flex", "1");
     document.getElementById("reminder-notifications").append(element);
   });
 });
@@ -36,17 +38,21 @@ function onLoad() {
     return cal.l10n.getString("calendar-alarms", getItemBundleStringName(x));
   }
 
-  setElementValue(
-    "reminder-before-start-menuitem",
-    _sn("reminderCustomOriginBeginBefore"),
-    "label"
+  document.getElementById("reminder-before-start-menuitem").label = _sn(
+    "reminderCustomOriginBeginBefore"
   );
 
-  setElementValue("reminder-after-start-menuitem", _sn("reminderCustomOriginBeginAfter"), "label");
+  document.getElementById("reminder-after-start-menuitem").label = _sn(
+    "reminderCustomOriginBeginAfter"
+  );
 
-  setElementValue("reminder-before-end-menuitem", _sn("reminderCustomOriginEndBefore"), "label");
+  document.getElementById("reminder-before-end-menuitem").label = _sn(
+    "reminderCustomOriginEndBefore"
+  );
 
-  setElementValue("reminder-after-end-menuitem", _sn("reminderCustomOriginEndAfter"), "label");
+  document.getElementById("reminder-after-end-menuitem").label = _sn(
+    "reminderCustomOriginEndAfter"
+  );
 
   // Set up the action map
   let supportedActions = calendar.getProperty("capabilities.alarms.actionValues") || ["DISPLAY"]; // TODO email support, "EMAIL"
@@ -61,7 +67,7 @@ function onLoad() {
     let shouldHide =
       !(actionNode.value in allowedActionsMap) ||
       (actionNode.hasAttribute("provider") && actionNode.getAttribute("provider") != calendar.type);
-    setElementValue(actionNode, shouldHide && "true", "hidden");
+    actionNode.hidden = shouldHide;
     if (!firstAvailableItem && !shouldHide) {
       firstAvailableItem = actionNode;
     }
@@ -95,7 +101,10 @@ function loadReminders() {
     if (reminders[i].action in allowedActionsMap) {
       // Set up the listitem and add it to the listbox, but only if the
       // action is actually supported by the calendar.
-      listbox.appendChild(setupListItem(null, reminders[i].clone(), args.item));
+      let listitem = setupListItem(null, reminders[i].clone(), args.item);
+      if (listitem) {
+        listbox.appendChild(listitem);
+      }
     }
   }
 
@@ -130,33 +139,28 @@ function setupRadioEnabledState(aDisableAll) {
   let relationItem = document.getElementById("reminder-relation-radiogroup").selectedItem;
   let relativeDisabled, absoluteDisabled;
 
-  // Note that the mix of string/boolean here is not a mistake.
-  // setElementValue removes the attribute from the node if the second
-  // parameter is === false, otherwise sets the attribute value to the given
-  // string (i.e "true").
   if (aDisableAll) {
-    relativeDisabled = "true";
-    absoluteDisabled = "true";
+    relativeDisabled = true;
+    absoluteDisabled = true;
   } else if (relationItem) {
     // This is not a mistake, when this function is called from onselect,
     // the value has not been set.
-    relativeDisabled = relationItem.value == "absolute" && "true";
-    absoluteDisabled = relationItem.value == "relative" && "true";
+    relativeDisabled = relationItem.value == "absolute";
+    absoluteDisabled = relationItem.value == "relative";
   } else {
     relativeDisabled = false;
     absoluteDisabled = false;
   }
 
-  setElementValue("reminder-length", relativeDisabled, "disabled");
-  setElementValue("reminder-unit", relativeDisabled, "disabled");
-  setElementValue("reminder-relation-origin", relativeDisabled, "disabled");
+  document.getElementById("reminder-length").disabled = relativeDisabled;
+  document.getElementById("reminder-unit").disabled = relativeDisabled;
+  document.getElementById("reminder-relation-origin").disabled = relativeDisabled;
 
-  setElementValue("reminder-absolute-date", absoluteDisabled, "disabled");
+  document.getElementById("reminder-absolute-date").setAttribute("disabled", !!absoluteDisabled);
 
-  let disableAll = aDisableAll ? "true" : false;
-  setElementValue("reminder-relative-radio", disableAll, "disabled");
-  setElementValue("reminder-absolute-radio", disableAll, "disabled");
-  setElementValue("reminder-actions-menulist", disableAll, "disabled");
+  document.getElementById("reminder-relative-radio").disabled = aDisableAll;
+  document.getElementById("reminder-absolute-radio").disabled = aDisableAll;
+  document.getElementById("reminder-actions-menulist").disabled = aDisableAll;
 }
 
 /**
@@ -168,13 +172,11 @@ function setupMaxReminders() {
   let listbox = document.getElementById("reminder-listbox");
   let maxReminders = args.calendar.getProperty("capabilities.alarms.maxCount");
 
-  // != null is needed here to ensure cond to be true/false, instead of
-  // true/null. The former is needed for setElementValue.
-  let cond = maxReminders != null && listbox.children.length >= maxReminders;
+  let hitMaxReminders = maxReminders && listbox.children.length >= maxReminders;
 
   // If we hit the maximum number of reminders, show the error box and
   // disable the new button.
-  setElementValue("reminder-new-button", cond && "true", "disabled");
+  document.getElementById("reminder-new-button").disabled = hitMaxReminders;
 
   let localeErrorString = cal.l10n.getString(
     "calendar-alarms",
@@ -186,18 +188,16 @@ function setupMaxReminders() {
     maxReminders
   );
 
-  if (cond) {
-    let notification = gNotification.notificationbox.appendNotification(
+  if (hitMaxReminders) {
+    let notification = gReminderNotification.appendNotification(
       pluralErrorLabel,
       "reminderNotification",
       null,
-      gNotification.notificationbox.PRIORITY_WARNING_MEDIUM
+      gReminderNotification.PRIORITY_WARNING_MEDIUM
     );
-
-    let closeButton = notification.messageDetails.nextElementSibling;
-    closeButton.setAttribute("hidden", "true");
+    notification.closeButton.hidden = true;
   } else {
-    gNotification.notificationbox.removeAllNotifications();
+    gReminderNotification.removeAllNotifications();
   }
 }
 
@@ -208,9 +208,29 @@ function setupMaxReminders() {
  *                                   passed, a new listitem will be created.
  * @param aReminder     The calIAlarm to display in this listitem
  * @param aItem         The item the alarm is set up on.
- * @return              The  XUL listitem node showing the passed reminder.
+ * @return              The  XUL listitem node showing the passed reminder, or
+ *   null if no list item should be shown.
  */
 function setupListItem(aListItem, aReminder, aItem) {
+  let src;
+  let l10nId;
+  switch (aReminder.action) {
+    case "DISPLAY":
+      src = "chrome://calendar/skin/shared/icons/alarm.svg";
+      l10nId = "calendar-event-reminder-icon-display";
+      break;
+    case "EMAIL":
+      src = "chrome://calendar/skin/shared/icons/email.svg";
+      l10nId = "calendar-event-reminder-icon-email";
+      break;
+    case "AUDIO":
+      src = "chrome://global/skin/media/audio.svg";
+      l10nId = "calendar-event-reminder-icon-audio";
+      break;
+    default:
+      return null;
+  }
+
   let listitem = aListItem || document.createXULElement("richlistitem");
 
   // Create a random id to be used for accessibility
@@ -223,12 +243,15 @@ function setupListItem(aListItem, aReminder, aItem) {
   listitem.setAttribute("aria-labelledby", ariaLabel);
   listitem.setAttribute("value", aReminder.action);
 
-  let image = listitem.querySelector("image");
+  let image = listitem.querySelector("img");
   if (!image) {
-    image = document.createXULElement("image");
+    image = document.createElement("img");
     image.setAttribute("class", "reminder-icon");
     listitem.appendChild(image);
   }
+  image.setAttribute("src", src);
+  // Sets alt.
+  document.l10n.setAttributes(image, l10nId);
   image.setAttribute("value", aReminder.action);
 
   let label = listitem.querySelector("label");
@@ -319,9 +342,9 @@ function onReminderSelected() {
 function updateReminder(event) {
   if (
     suppressListUpdate ||
-    event.explicitOriginalTarget.localName == "richlistitem" ||
-    event.explicitOriginalTarget.parentNode.localName == "richlistitem" ||
-    event.explicitOriginalTarget.id == "reminder-remove-button" ||
+    event.target.localName == "richlistitem" ||
+    event.target.parentNode.localName == "richlistitem" ||
+    event.target.id == "reminder-remove-button" ||
     !document.commandDispatcher.focusedElement
   ) {
     // Do not set things if the select came from selecting or removing an
@@ -370,7 +393,10 @@ function updateReminder(event) {
     }
   }
 
-  setupListItem(listitem, reminder, window.arguments[0].item);
+  if (!setupListItem(listitem, reminder, window.arguments[0].item)) {
+    // Unexpected since this would mean switching to an unsupported type.
+    listitem.remove();
+  }
 }
 
 /**
@@ -381,7 +407,7 @@ function updateReminder(event) {
  * @return              The full string name.
  */
 function getItemBundleStringName(aPrefix) {
-  if (cal.item.isEvent(window.arguments[0].item)) {
+  if (window.arguments[0].item.isEvent()) {
     return aPrefix + "Event";
   }
   return aPrefix + "Task";
@@ -392,10 +418,10 @@ function getItemBundleStringName(aPrefix) {
  * new reminder item.
  */
 function onNewReminder() {
-  let itemType = cal.item.isEvent(window.arguments[0].item) ? "event" : "todo";
+  let itemType = window.arguments[0].item.isEvent() ? "event" : "todo";
   let listbox = document.getElementById("reminder-listbox");
 
-  let reminder = cal.createAlarm();
+  let reminder = new CalAlarm();
   let alarmlen = Services.prefs.getIntPref("calendar.alarms." + itemType + "alarmlen", 15);
   let alarmunit = Services.prefs.getStringPref(
     "calendar.alarms." + itemType + "alarmunit",
@@ -415,7 +441,7 @@ function onNewReminder() {
   }
   offset.normalize();
   offset.isNegative = true;
-  reminder.related = reminder.ALARM_RELATED_START;
+  reminder.related = Ci.calIAlarm.ALARM_RELATED_START;
   reminder.offset = offset;
   if ("DISPLAY" in allowedActionsMap) {
     reminder.action = "DISPLAY";
@@ -427,6 +453,9 @@ function onNewReminder() {
 
   // Set up the listbox
   let listitem = setupListItem(null, reminder, window.arguments[0].item);
+  if (!listitem) {
+    return;
+  }
   listbox.appendChild(listitem);
   listbox.selectItem(listitem);
 
@@ -453,7 +482,7 @@ function onRemoveReminder() {
   listitem.remove();
   listbox.selectItem(newSelection);
 
-  setElementValue("reminder-remove-button", listbox.children.length < 1 && "true", "disabled");
+  document.getElementById("reminder-remove-button").disabled = listbox.children.length < 1;
   setupMaxReminders();
 }
 
