@@ -446,7 +446,7 @@ calFilter.prototype = {
     }
 
     // test the status property. Only applies to tasks.
-    if (result && props.status != null && cal.item.isToDo(aItem)) {
+    if (result && props.status != null && aItem.isTodo()) {
       let completed = aItem.isCompleted;
       let current = !aItem.completedDate || today.compare(aItem.completedDate) <= 0;
       let percent = aItem.percentComplete || 0;
@@ -459,7 +459,7 @@ calFilter.prototype = {
     }
 
     // test the due property. Only applies to tasks.
-    if (result && props.due != null && cal.item.isToDo(aItem)) {
+    if (result && props.due != null && aItem.isTodo()) {
       let due = aItem.dueDate;
       let now = cal.dtz.now();
 
@@ -591,7 +591,7 @@ calFilter.prototype = {
    * calculated from the filter properties by the getDatesForFilter function.
    */
   set startDate(aStartDate) {
-    return (this.mStartDate = aStartDate);
+    this.mStartDate = aStartDate;
   },
 
   /**
@@ -609,7 +609,7 @@ calFilter.prototype = {
    * calculated from the filter properties by the getDatesForFilter function.
    */
   set endDate(aEndDate) {
-    return (this.mEndDate = aEndDate);
+    this.mEndDate = aEndDate;
   },
 
   /**
@@ -625,7 +625,7 @@ calFilter.prototype = {
    * @param aValue              The string value to use for the text filter.
    */
   set filterText(aValue) {
-    return (this.mFilterText = aValue);
+    this.mFilterText = aValue;
   },
 
   /**
@@ -641,7 +641,7 @@ calFilter.prototype = {
    * that are relative to the selected date.
    */
   set selectedDate(aSelectedDate) {
-    return (this.mSelectedDate = aSelectedDate);
+    this.mSelectedDate = aSelectedDate;
   },
 
   /**
@@ -884,7 +884,7 @@ calFilter.prototype = {
     // the listener passed in the aListener argument.
     let self = this;
     let listener = {
-      QueryInterface: ChromeUtils.generateQI([Ci.calIOperationListener]),
+      QueryInterface: ChromeUtils.generateQI(["calIOperationListener"]),
       onOperationComplete: aListener.onOperationComplete.bind(aListener),
 
       onGetResult(aOpCalendar, aStatus, aOpItemType, aDetail, aItems) {
