@@ -4,30 +4,29 @@
 
 requestLongerTimeout(3);
 
-var mozmill = ChromeUtils.import("resource://testing-common/mozmill/mozmill.jsm");
-
 var {
   CALENDARNAME,
-  CANVAS_BOX,
   createCalendar,
-  DAY_VIEW,
+  controller,
   deleteCalendars,
-  helpersForController,
-  invokeEventDialog,
+  invokeNewEventDialog,
   switchToView,
   goToDate,
   findEventsInNode,
   viewForward,
   viewBack,
-} = ChromeUtils.import("resource://testing-common/mozmill/CalendarUtils.jsm");
-var { setData } = ChromeUtils.import("resource://testing-common/mozmill/ItemEditingHelpers.jsm");
+} = ChromeUtils.import("resource://testing-common/calendar/CalendarUtils.jsm");
+var { saveAndCloseItemDialog, setData } = ChromeUtils.import(
+  "resource://testing-common/calendar/ItemEditingHelpers.jsm"
+);
+
+var { dayView } = ChromeUtils.import(
+  "resource://testing-common/calendar/CalendarTestUtils.jsm"
+).CalendarTestUtils;
 
 var { cal } = ChromeUtils.import("resource:///modules/calendar/calUtils.jsm");
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
-var controller = mozmill.getMail3PaneController();
-var { eid, lookupEventBox } = helpersForController(controller);
-
 var DATES = [
   [2009, 1, 1],
   [2009, 4, 2],
@@ -67,13 +66,13 @@ add_task(async function testTimezones2_CreateEvents() {
   ];
   let time = cal.createDateTime();
   for (let i = 0; i < TIMEZONES.length; i++) {
-    let eventBox = lookupEventBox("day", CANVAS_BOX, null, 1, i + 11);
-    await invokeEventDialog(controller, eventBox, async (event, iframe) => {
+    let eventBox = dayView.getHourBoxAt(controller.window, i + 11);
+    await invokeNewEventDialog(window, eventBox, async (eventWindow, iframeWindow) => {
       time.hour = times[i][0];
       time.minute = times[i][1];
 
       // Set event data.
-      await setData(event, iframe, {
+      await setData(eventWindow, iframeWindow, {
         title: TIMEZONES[i],
         repeat: "weekly",
         repeatuntil: cal.createDateTime("20091231T000000Z"),
@@ -81,9 +80,7 @@ add_task(async function testTimezones2_CreateEvents() {
         timezone: TIMEZONES[i],
       });
 
-      // save
-      let { eid: eventid } = helpersForController(event);
-      event.click(eventid("button-saveandclose"));
+      await saveAndCloseItemDialog(eventWindow);
     });
   }
 });
@@ -172,7 +169,7 @@ add_task(function testTimezones3_checkStJohns() {
       [11, 30],
     ],
   ];
-  controller.click(eid("calendar-tab-button"));
+  controller.click(controller.window.document.getElementById("calendar-tab-button"));
   switchToView(controller, "day");
   goToDate(controller, 2009, 1, 1);
 
@@ -265,7 +262,7 @@ add_task(function testTimezones4_checkCaracas() {
       [11, 0],
     ],
   ];
-  controller.click(eid("calendar-tab-button"));
+  controller.click(controller.window.document.getElementById("calendar-tab-button"));
   switchToView(controller, "day");
   goToDate(controller, 2009, 1, 1);
 
@@ -356,7 +353,7 @@ add_task(function testTimezones5_checkPhoenix() {
       [8, 0],
     ],
   ];
-  controller.click(eid("calendar-tab-button"));
+  controller.click(controller.window.document.getElementById("calendar-tab-button"));
   switchToView(controller, "day");
   goToDate(controller, 2009, 1, 1);
 
@@ -447,7 +444,7 @@ add_task(function testTimezones6_checkLosAngeles() {
       [7, 0],
     ],
   ];
-  controller.click(eid("calendar-tab-button"));
+  controller.click(controller.window.document.getElementById("calendar-tab-button"));
   switchToView(controller, "day");
   goToDate(controller, 2009, 1, 1);
 
@@ -538,7 +535,7 @@ add_task(function testTimezones7_checkBuenosAires() {
       [12, 0],
     ],
   ];
-  controller.click(eid("calendar-tab-button"));
+  controller.click(controller.window.document.getElementById("calendar-tab-button"));
   switchToView(controller, "day");
   goToDate(controller, 2009, 1, 1);
 
@@ -629,7 +626,7 @@ add_task(function testTimezones8_checkParis() {
       [16, 0],
     ],
   ];
-  controller.click(eid("calendar-tab-button"));
+  controller.click(controller.window.document.getElementById("calendar-tab-button"));
   switchToView(controller, "day");
   goToDate(controller, 2009, 1, 1);
 
@@ -720,7 +717,7 @@ add_task(function testTimezones9_checkKathmandu() {
       [20, 45],
     ],
   ];
-  controller.click(eid("calendar-tab-button"));
+  controller.click(controller.window.document.getElementById("calendar-tab-button"));
   switchToView(controller, "day");
   goToDate(controller, 2009, 1, 1);
 
@@ -811,7 +808,7 @@ add_task(function testTimezones10_checkAdelaide() {
       [1, 30, +1],
     ],
   ];
-  controller.click(eid("calendar-tab-button"));
+  controller.click(controller.window.document.getElementById("calendar-tab-button"));
   switchToView(controller, "day");
   goToDate(controller, 2009, 1, 1);
 
@@ -824,18 +821,6 @@ function verify(dates, timezones, times) {
       yield [dates[idx][0], dates[idx][1], dates[idx][2], times[idx]];
     }
   }
-
-  let { lookup } = helpersForController(controller);
-
-  let dayStack = `
-        ${DAY_VIEW}/{"class":"mainbox"}/{"class":"scrollbox"}/
-        {"class":"daybox"}/[0]/{"class":"multiday-column-box-stack"}/
-        {"class":"multiday-column-top-box"}/{"flex":"1"}
-    `;
-  let timeLine = `
-        ${DAY_VIEW}/{"class":"mainbox"}/{"class":"scrollbox"}/
-        {"class":"timebar"}/{"class":"timebarboxstack"}/{"class":"topbox"}
-    `;
   let allowedDifference = 3;
 
   /* Event box' time can't be deduced from it's position in                    ----------------
@@ -852,7 +837,10 @@ function verify(dates, timezones, times) {
     for (let tzIdx = 0; tzIdx < timezones.length; tzIdx++) {
       let [correctHour, minutes, day] = selectedTime[tzIdx];
 
-      let timeNode = lookup(`${timeLine}/[${correctHour}]`).getNode();
+      let timeNode = window.document.querySelector(
+        `#day-view .timebarboxstack > .topbox > box:nth-of-type(${correctHour + 1})`
+      );
+      Assert.ok(timeNode);
       let boundingRect = timeNode.getBoundingClientRect();
       let timeY = boundingRect.y + boundingRect.height * (minutes / 60);
 
@@ -865,11 +853,7 @@ function verify(dates, timezones, times) {
         viewBack(controller, 1);
       }
 
-      let stackNode = lookup(dayStack);
-      controller.waitForElement(stackNode);
-      stackNode = stackNode.getNode();
-
-      findEventsInNode(stackNode, eventNodes);
+      eventNodes = Array.from(dayView.getEventBoxes(controller.window));
       eventNodes = eventNodes
         .filter(node => node.mOccurrence.title == timezones[tzIdx])
         .map(node => node.getBoundingClientRect().y);
@@ -885,7 +869,7 @@ function verify(dates, timezones, times) {
         viewForward(controller, 1);
       }
 
-      controller.assert(() => eventNodes.some(node => Math.abs(timeY - node) < allowedDifference));
+      Assert.ok(eventNodes.some(node => Math.abs(timeY - node) < allowedDifference));
     }
   }
 }
