@@ -3,6 +3,14 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 var { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+var { TestUtils } = ChromeUtils.import("resource://testing-common/TestUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalAlarm: "resource:///modules/CalAlarm.jsm",
+  CalEvent: "resource:///modules/CalEvent.jsm",
+  CalRecurrenceInfo: "resource:///modules/CalRecurrenceInfo.jsm",
+});
 
 var EXPECT_NONE = 0;
 var EXPECT_FIRED = 1;
@@ -13,7 +21,7 @@ function do_check_xor(a, b, aMessage) {
 }
 
 var alarmObserver = {
-  QueryInterface: ChromeUtils.generateQI([Ci.calIAlarmServiceObserver]),
+  QueryInterface: ChromeUtils.generateQI(["calIAlarmServiceObserver"]),
 
   service: null,
   firedMap: {},
@@ -25,6 +33,8 @@ var alarmObserver = {
     this.firedMap[aItem.hashId][aAlarm.icalString] = true;
   },
 
+  onNotification(item) {},
+
   onRemoveAlarmsByItem(aItem) {
     if (aItem.hashId in this.firedMap) {
       delete this.firedMap[aItem.hashId];
@@ -142,6 +152,8 @@ function run_test() {
   add_test(test_addItems);
   add_test(test_loadCalendar);
   add_test(test_modifyItems);
+  add_test(test_notificationTimers);
+  add_test(test_calendarLevelNotificationTimers);
 
   run_next_test();
 }
@@ -162,7 +174,7 @@ function initializeAlarmService() {
 }
 
 function createAlarmFromDuration(aOffset) {
-  let alarm = cal.createAlarm();
+  let alarm = new CalAlarm();
 
   alarm.related = Ci.calIAlarm.ALARM_RELATED_START;
   alarm.offset = cal.createDuration(aOffset);
@@ -172,7 +184,7 @@ function createAlarmFromDuration(aOffset) {
 
 function createEventWithAlarm(aCalendar, aStart, aEnd, aOffset, aRRule) {
   let alarm = null;
-  let item = cal.createEvent();
+  let item = new CalEvent();
 
   item.id = cal.getUUID();
   item.calendar = aCalendar;
@@ -183,7 +195,7 @@ function createEventWithAlarm(aCalendar, aStart, aEnd, aOffset, aRRule) {
     item.addAlarm(alarm);
   }
   if (aRRule) {
-    item.recurrenceInfo = cal.createRecurrenceInfo(item);
+    item.recurrenceInfo = new CalRecurrenceInfo(item);
     item.recurrenceInfo.appendRecurrenceItem(cal.createRecurrenceRule(aRRule));
   }
   return [item, alarm];
@@ -455,9 +467,9 @@ function doRunTest(aOnCalendarCreated, aOnAlarmsLoaded) {
 
   calmgr.registerCalendar(memory);
 
-  alarmObserver.doOnAlarmsLoaded(memory, () => {
+  alarmObserver.doOnAlarmsLoaded(memory, async () => {
     if (aOnAlarmsLoaded) {
-      aOnAlarmsLoaded.call(aOnAlarmsLoaded, memory);
+      await aOnAlarmsLoaded(memory);
     }
 
     run_next_test();
@@ -485,3 +497,128 @@ function test_modifyItems() {
     doAcknowledgeTest(memory);
   });
 }
+
+/**
+ * Test an array of timers has expected delay values.
+ * @param {nsITimer[]} timers - An array of nsITimer.
+ * @param {number[]} expected - Expected delays in seconds.
+ */
+function matchTimers(timers, expected) {
+  let delays = timers.map(timer => timer.delay / 1000);
+  let matched = true;
+  for (let i = 0; i < delays.length; i++) {
+    if (Math.abs(delays[i] - expected[i]) > 1) {
+      matched = false;
+      break;
+    }
+  }
+  ok(matched, `Delays=${delays} should match Expected=${expected}`);
+}
+
+/**
+ * Test notification timers are set up correctly when add/modify/remove a
+ * calendar item.
+ */
+function test_notificationTimers() {
+  doRunTest(null, memory => {
+    // Add an item.
+    let date = cal.dtz.now();
+    date.hour += 1;
+    let item, oldItem;
+    [item] = createEventWithAlarm(memory, date, date, null);
+    memory.addItem(item, null);
+    equal(
+      alarmObserver.service.mNotificationTimerMap[item.calendar.id],
+      undefined,
+      "should have no notification timer"
+    );
+
+    // Set the pref to have one notifiaction.
+    Services.prefs.setCharPref("calendar.notifications.times", "-PT1H");
+    oldItem = item.clone();
+    date.hour += 1;
+    item.startDate = date.clone();
+    item.generation++;
+    memory.modifyItem(item, oldItem, null);
+    // Should have one notification timer
+    matchTimers(alarmObserver.service.mNotificationTimerMap[item.calendar.id][item.hashId], [3600]);
+
+    // Set the pref to have three notifiactions.
+    Services.prefs.setCharPref("calendar.notifications.times", "END:PT2M,PT0M,END:-PT30M,-PT5M");
+    oldItem = item.clone();
+    date.hour -= 1;
+    item.startDate = date.clone();
+    date.hour += 1;
+    item.endDate = date.clone();
+    item.generation++;
+    memory.modifyItem(item, oldItem, null);
+    // Should have four notification timers.
+    matchTimers(alarmObserver.service.mNotificationTimerMap[item.calendar.id][item.hashId], [
+      3300, // 55 minutes
+      3600, // 60 minutes
+      5400, // 90 minutes, which is 30 minutes before the end (END:-PT30M)
+      7320, // 122 minutes, which is 2 minutes after the end (END:PT2M)
+    ]);
+
+    alarmObserver.service.removeFiredNotificationTimer(item);
+    // Should have three notification timers.
+    matchTimers(alarmObserver.service.mNotificationTimerMap[item.calendar.id][item.hashId], [
+      3600,
+      5400,
+      7320,
+    ]);
+
+    memory.deleteItem(item, null);
+    equal(
+      alarmObserver.service.mNotificationTimerMap[item.calendar.id],
+      undefined,
+      "notification timers should be removed"
+    );
+
+    Services.prefs.clearUserPref("calendar.notifications.times");
+  });
+}
+
+/**
+ * Test notification timers are set up correctly according to the calendar level
+ * notifications.times config.
+ */
+function test_calendarLevelNotificationTimers() {
+  let loaded = false;
+  let item;
+  doRunTest(null, async memory => {
+    if (!loaded) {
+      loaded = true;
+      // Set the global pref to have one notifiaction.
+      Services.prefs.setCharPref("calendar.notifications.times", "-PT1H");
+
+      // Add an item.
+      let date = cal.dtz.now();
+      date.hour += 2;
+      [item] = createEventWithAlarm(memory, date, date, null);
+      memory.addItem(item, null);
+
+      // Should have one notification timer.
+      matchTimers(alarmObserver.service.mNotificationTimerMap[item.calendar.id][item.hashId], [
+        3600,
+      ]);
+      // Set the calendar level pref to have two notification timers.
+      memory.setProperty("notifications.times", "-PT5M,PT0M");
+    }
+
+    await TestUtils.waitForCondition(
+      () => alarmObserver.service.mNotificationTimerMap[item.calendar.id]?.[item.hashId].length == 2
+    );
+    // Should have two notification timers
+    matchTimers(alarmObserver.service.mNotificationTimerMap[item.calendar.id][item.hashId], [
+      6900, // 105 minutes
+      7200, // 120 minutes
+    ]);
+
+    Services.prefs.clearUserPref("calendar.notifications.times");
+  });
+}
+
+registerCleanupFunction(() => {
+  Services.prefs.clearUserPref("calendar.notifications.times");
+});
