@@ -2,13 +2,20 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+var { XPCOMUtils } = ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+
+XPCOMUtils.defineLazyModuleGetters(this, {
+  CalEvent: "resource:///modules/CalEvent.jsm",
+  CalTodo: "resource:///modules/CalTodo.jsm",
+});
+
 function run_test() {
   test_get_item_sort_key();
   test_sort_items();
 }
 
 function test_get_item_sort_key() {
-  let event = cal.createEvent(dedent`
+  let event = new CalEvent(dedent`
         BEGIN:VEVENT
         PRIORITY:8
         SUMMARY:summary
@@ -29,7 +36,7 @@ function test_get_item_sort_key() {
   equal(cal.unifinder.getItemSortKey(event, "location"), "location");
   equal(cal.unifinder.getItemSortKey(event, "status"), 1);
 
-  let task = cal.createTodo(dedent`
+  let task = new CalTodo(dedent`
         BEGIN:VTODO
         DTSTART:20180102T030405Z
         DUE:20180607T080910Z
@@ -48,20 +55,24 @@ function test_get_item_sort_key() {
   strictEqual(cal.unifinder.getItemSortKey(task, "location"), "");
   equal(cal.unifinder.getItemSortKey(task, "status"), 2);
 
-  let task2 = cal.createTodo(dedent`
+  let task2 = new CalTodo(dedent`
         BEGIN:VTODO
         STATUS:GETTIN' THERE
         END:VTODO
     `);
   equal(cal.unifinder.getItemSortKey(task2, "percentComplete"), 0);
   equal(cal.unifinder.getItemSortKey(task2, "status"), -1);
+
+  // Default CalTodo objects have the default percentComplete.
+  let task3 = new CalTodo();
+  equal(cal.unifinder.getItemSortKey(task3, "percentComplete"), 0);
 }
 
 function test_sort_items() {
   // string comparison
   let summaries = ["", "a", "b"];
   let items = summaries.map(summary => {
-    return cal.createEvent(dedent`
+    return new CalEvent(dedent`
             BEGIN:VEVENT
             SUMMARY:${summary}
             END:VEVENT
@@ -83,7 +94,7 @@ function test_sort_items() {
   // date comparison
   let dates = ["20180101T000002Z", "20180101T000000Z", "20180101T000001Z"];
   items = dates.map(date => {
-    return cal.createEvent(dedent`
+    return new CalEvent(dedent`
             BEGIN:VEVENT
             DTSTART:${date}
             END:VEVENT
@@ -105,7 +116,7 @@ function test_sort_items() {
   // number comparison
   let percents = [3, 1, 2];
   items = percents.map(percent => {
-    return cal.createTodo(dedent`
+    return new CalTodo(dedent`
             BEGIN:VTODO
             PERCENT-COMPLETE:${percent}
             END:VTODO
