@@ -11,21 +11,23 @@
   const { Services } = ChromeUtils.import(
     "resource://gre/modules/Services.jsm"
   );
-  const {
-    initHTMLDocument,
-    serializeSelection,
-    getCurrentTheme,
-    isNextMessage,
-    getHTMLForMessage,
-    insertHTMLForMessage,
-  } = ChromeUtils.import("resource:///modules/imThemes.jsm");
-  const { smileTextNode } = ChromeUtils.import(
-    "resource:///modules/imSmileys.jsm"
-  );
-  const { cleanupImMarkup } = ChromeUtils.import(
-    "resource:///modules/imContentSink.jsm"
+  const { XPCOMUtils } = ChromeUtils.import(
+    "resource://gre/modules/XPCOMUtils.jsm"
   );
 
+  const LazyModules = {};
+  XPCOMUtils.defineLazyModuleGetters(LazyModules, {
+    cleanupImMarkup: "resource:///modules/imContentSink.jsm",
+    getCurrentTheme: "resource:///modules/imThemes.jsm",
+    getDocumentFragmentFromHTML: "resource:///modules/imThemes.jsm",
+    getHTMLForMessage: "resource:///modules/imThemes.jsm",
+    initHTMLDocument: "resource:///modules/imThemes.jsm",
+    insertHTMLForMessage: "resource:///modules/imThemes.jsm",
+    isNextMessage: "resource:///modules/imThemes.jsm",
+    serializeSelection: "resource:///modules/imThemes.jsm",
+    smileTextNode: "resource:///modules/imSmileys.jsm",
+  });
+
   (function() {
     // <browser> is lazily set up through setElementCreationCallback,
     // i.e. put into customElements the first time it's really seen.
@@ -46,74 +48,6 @@
 
       this._conv = null;
 
-      // @implements {nsIWebProgressListener}
-      this.progressListener = {
-        onStateChange: (progress, request, stateFlags, status) => {
-          if (
-            !(stateFlags & Ci.nsIWebProgressListener.STATE_STOP) ||
-            !(
-              stateFlags & Ci.nsIWebProgressListener.STATE_IS_DOCUMENT ||
-              stateFlags & Ci.nsIWebProgressListener.STATE_IS_WINDOW
-            )
-          ) {
-            return;
-          }
-          if (!this._loadState) {
-            initHTMLDocument(this._conv, this.theme, this.contentDocument);
-            this._loadState = 1;
-            this._exposeMethodsToContent();
-            return;
-          }
-          this.removeProgressListener(this.progressListener);
-
-          this.initMagicCopy();
-
-          // We need to reset these variables here to avoid a race
-          // condition if we are starting to display a new conversation
-          // but the display of the previous conversation wasn't finished.
-          // This can happen if the user quickly changes the selected
-          // conversation in the log viewer.
-          this._lastMessage = null;
-          this._lastMessageIsContext = true;
-          this._firstNonContextElt = null;
-          this._messageDisplayPending = false;
-          this._pendingMessages = [];
-          this._nextPendingMessageIndex = 0;
-          this._pendingMessagesDisplayed = 0;
-          this._displayPendingMessagesCalls = 0;
-          this._sessions = [];
-          if (this.progressBar) {
-            this.progressBar.hidden = true;
-          }
-
-          this.onChatNodeContentLoad = this.onContentElementLoad.bind(this);
-          this.contentChatNode.addEventListener(
-            "load",
-            this.onChatNodeContentLoad,
-            true
-          );
-
-          // Notify observers to get the conversation shown.
-          Services.obs.notifyObservers(this, "conversation-loaded");
-        },
-        onProgressChange(
-          progress,
-          request,
-          curSelf,
-          maxSelf,
-          curTotal,
-          maxTotal
-        ) {},
-        onLocationChange(aprogress, request, location) {},
-        onStatusChange(progress, request, status, message) {},
-        onSecurityChange(progress, request, state) {},
-        onContentBlockingEvent(progress, request, event) {},
-        QueryInterface: ChromeUtils.generateQI([
-          Ci.nsIWebProgressListener,
-          Ci.nsISupportsWeakReference,
-        ]),
-      };
-
       // Make sure to load URLs externally.
       this.addEventListener("click", event => {
         // Right click should open the context menu.
@@ -188,12 +122,15 @@
     }
 
     connectedCallback() {
+      if (this.hasConnected) {
+        return;
+      }
+      this.hasConnected = true;
+
       super.connectedCallback();
 
       this._theme = null;
 
-      this._loadState = 0;
-
       this.autoCopyEnabled = false;
 
       this.magicCopyPref =
@@ -209,7 +146,7 @@
       // (within 10px) of the bottom.
       this._convScrollEnabled = true;
 
-      this._textModifiers = [smileTextNode];
+      this._textModifiers = [LazyModules.smileTextNode];
 
       // These variables are reset in onStateChange:
       this._lastMessage = null;
@@ -255,10 +192,10 @@
 
           Cc["@mozilla.org/widget/clipboardhelper;1"]
             .getService(Ci.nsIClipboardHelper)
-            .copyString(serializeSelection(selection));
+            .copyString(LazyModules.serializeSelection(selection));
         },
         onEvent(command) {},
-        QueryInterface: ChromeUtils.generateQI([Ci.nsIController]),
+        QueryInterface: ChromeUtils.generateQI(["nsIController"]),
       };
 
       // @implements {nsISelectionListener}
@@ -278,11 +215,11 @@
           Cc["@mozilla.org/widget/clipboardhelper;1"]
             .getService(Ci.nsIClipboardHelper)
             .copyStringToClipboard(
-              serializeSelection(selection),
+              LazyModules.serializeSelection(selection),
               Ci.nsIClipboard.kSelectionClipboard
             );
         },
-        QueryInterface: ChromeUtils.generateQI([Ci.nsISelectionListener]),
+        QueryInterface: ChromeUtils.generateQI(["nsISelectionListener"]),
       };
     }
 
@@ -300,21 +237,58 @@
       // Prevent ongoing asynchronous message display from continuing.
       this._messageDisplayPending = false;
 
-      // _loadState is 0 while loading conv.html and 1 while
-      // loading the real conversation HTML.
-      this._loadState = 0;
+      this.addEventListener(
+        "load",
+        () => {
+          LazyModules.initHTMLDocument(
+            this._conv,
+            this.theme,
+            this.contentDocument
+          );
 
-      this.docShell.charset = "UTF-8";
-      const URI = "chrome://chat/content/conv.html";
-      const loadURIOptions = {
+          this._exposeMethodsToContent();
+          this.initMagicCopy();
+
+          // We need to reset these variables here to avoid a race
+          // condition if we are starting to display a new conversation
+          // but the display of the previous conversation wasn't finished.
+          // This can happen if the user quickly changes the selected
+          // conversation in the log viewer.
+          this._lastMessage = null;
+          this._lastMessageIsContext = true;
+          this._firstNonContextElt = null;
+          this._messageDisplayPending = false;
+          this._pendingMessages = [];
+          this._nextPendingMessageIndex = 0;
+          this._pendingMessagesDisplayed = 0;
+          this._displayPendingMessagesCalls = 0;
+          this._sessions = [];
+          if (this.progressBar) {
+            this.progressBar.hidden = true;
+          }
+
+          this.onChatNodeContentLoad = this.onContentElementLoad.bind(this);
+          this.contentChatNode.addEventListener(
+            "load",
+            this.onChatNodeContentLoad,
+            true
+          );
+
+          // Notify observers to get the conversation shown.
+          Services.obs.notifyObservers(this, "conversation-loaded");
+        },
+        {
+          once: true,
+          capture: true,
+        }
+      );
+      this.loadURI("chrome://chat/content/conv.html", {
         triggeringPrincipal: Services.scriptSecurityManager.getSystemPrincipal(),
-      };
-      this.webNavigation.loadURI(URI, loadURIOptions);
-      this.addProgressListener(this.progressListener);
+      });
     }
 
     get theme() {
-      return this._theme || (this._theme = getCurrentTheme());
+      return this._theme || (this._theme = LazyModules.getCurrentTheme());
     }
 
     get contentDocument() {
@@ -424,7 +398,10 @@
     }
 
     set isActive(value) {
-      this.docShell.isActive = value;
+      if (!value && !this.browsingContext) {
+        return;
+      }
+      this.browsingContext.isActive = value;
       if (value && this._pendingMessages.length) {
         this.startDisplayingPendingMessages(false);
       }
@@ -436,7 +413,7 @@
         context: aContext,
         firstUnread: aFirstUnread,
       });
-      if (this.docShell.isActive) {
+      if (this.browsingContext.isActive) {
         this.startDisplayingPendingMessages(true);
       }
     }
@@ -577,7 +554,7 @@
 
       // Right trim before displaying. This removes any OTR related
       // whitespace when the extension isn't enabled.
-      let msg = aMsg.displayMessage.trimRight();
+      let msg = aMsg.displayMessage?.trimRight() ?? "";
 
       // The slash of a leading '/me' should not be used to
       // format as italic, so we remove the '/me' text before
@@ -597,7 +574,7 @@
         msg = msg.replace(/^((<[^>]+>)*)/, "$1/me ");
       }
 
-      aMsg.message = cleanupImMarkup(
+      aMsg.message = LazyModules.cleanupImMarkup(
         msg.replace(/\r?\n/g, "<br/>"),
         null,
         this._textModifiers
@@ -605,12 +582,17 @@
 
       let next =
         (aContext == this._lastMessageIsContext || aMsg.system) &&
-        isNextMessage(this.theme, aMsg, this._lastMessage);
+        LazyModules.isNextMessage(this.theme, aMsg, this._lastMessage);
       let newElt;
       if (next && aFirstUnread) {
         // If there wasn't an unread ruler, this would be a Next message.
         // Therefore, save that version for later.
-        let html = getHTMLForMessage(aMsg, this.theme, next, aContext);
+        let html = LazyModules.getHTMLForMessage(
+          aMsg,
+          this.theme,
+          next,
+          aContext
+        );
         let ruler = doc.getElementById("unread-ruler");
         ruler.nextMsgHtml = html;
         ruler._originalMsg = aMsg;
@@ -624,8 +606,8 @@
         insert.id = "insert-before";
 
         next = false;
-        html = getHTMLForMessage(aMsg, this.theme, next, aContext);
-        newElt = insertHTMLForMessage(aMsg, html, doc, next);
+        html = LazyModules.getHTMLForMessage(aMsg, this.theme, next, aContext);
+        newElt = LazyModules.insertHTMLForMessage(aMsg, html, doc, next);
         let marker = doc.createElement("div");
         marker.id = "end-of-split-block";
         newElt.parentNode.appendChild(marker);
@@ -642,8 +624,13 @@
           insert.parentNode.insertBefore(marker, insert.nextElementSibling);
         }
       } else {
-        let html = getHTMLForMessage(aMsg, this.theme, next, aContext);
-        newElt = insertHTMLForMessage(aMsg, html, doc, next);
+        let html = LazyModules.getHTMLForMessage(
+          aMsg,
+          this.theme,
+          next,
+          aContext
+        );
+        newElt = LazyModules.insertHTMLForMessage(aMsg, html, doc, next);
       }
 
       if (!aNoAutoScroll) {
@@ -670,6 +657,10 @@
     }
 
     removeUnreadRuler() {
+      if (this._lastMessage) {
+        this._lastMessage.whenRead();
+      }
+
       let doc = this.contentDocument;
       let ruler = doc.getElementById("unread-ruler");
       if (!ruler) {
@@ -691,7 +682,8 @@
         let moveToParent = moveTo.parentNode;
         range.selectNode(moveToParent);
         // eslint-disable-next-line no-unsanitized/method
-        let documentFragment = range.createContextualFragment(
+        let documentFragment = LazyModules.getDocumentFragmentFromHTML(
+          doc,
           ruler.nextMsgHtml
         );
         for (
