@@ -2,16 +2,12 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
-// @flow
-
 import {
   validateNavigateContext,
   validateContext,
 } from "../../../utils/context";
 
-import type { ThunkArgs } from "../../types";
-
-function validateActionContext(getState, action): void {
+function validateActionContext(getState, action) {
   if (action.type == "COMMAND" && action.status == "done") {
     // The thread will have resumed execution since the action was initiated,
     // so just make sure we haven't navigated.
@@ -25,8 +21,8 @@ function validateActionContext(getState, action): void {
 
 // Middleware which looks for actions that have a cx property and ignores
 // them if the context is no longer valid.
-function context({ dispatch, getState }: ThunkArgs) {
-  return (next: Function) => (action: Object) => {
+function context({ dispatch, getState }) {
+  return next => action => {
     if ("cx" in action) {
       validateActionContext(getState, action);
     }
