@@ -2,8 +2,6 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
-// @flow
-
 import React from "react";
 import { shallow } from "enzyme";
 import Group from "../Group.js";
@@ -31,6 +29,7 @@ function render(overrides = {}) {
     disableContextMenu: false,
     displayFullUrl: false,
     panel: "webconsole",
+    restart: jest.fn(),
   };
 
   const props = { ...defaultProps, ...overrides };
