@@ -1,5 +1,3 @@
-/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -13,54 +11,82 @@ Cc["@mozilla.org/psm;1"].getService(Ci.nsISupports);
 
 var Services = require("Services");
 var promise = require("promise");
-var defer = require("devtools/shared/defer");
 var DevToolsUtils = require("devtools/shared/DevToolsUtils");
 var { dumpn, dumpv } = DevToolsUtils;
-loader.lazyRequireGetter(this, "WebSocketServer",
-  "devtools/server/websocket-server");
-loader.lazyRequireGetter(this, "DebuggerTransport",
-  "devtools/shared/transport/transport", true);
-loader.lazyRequireGetter(this, "WebSocketDebuggerTransport",
-  "devtools/shared/transport/websocket-transport");
-loader.lazyRequireGetter(this, "DebuggerServer",
-  "devtools/server/main", true);
-loader.lazyRequireGetter(this, "discovery",
-  "devtools/shared/discovery/discovery");
-loader.lazyRequireGetter(this, "cert",
-  "devtools/shared/security/cert");
-loader.lazyRequireGetter(this, "Authenticators",
-  "devtools/shared/security/auth", true);
-loader.lazyRequireGetter(this, "AuthenticationResult",
-  "devtools/shared/security/auth", true);
+loader.lazyRequireGetter(
+  this,
+  "WebSocketServer",
+  "devtools/server/socket/websocket-server"
+);
+loader.lazyRequireGetter(
+  this,
+  "DebuggerTransport",
+  "devtools/shared/transport/transport",
+  true
+);
+loader.lazyRequireGetter(
+  this,
+  "WebSocketDebuggerTransport",
+  "devtools/shared/transport/websocket-transport"
+);
+loader.lazyRequireGetter(
+  this,
+  "discovery",
+  "devtools/shared/discovery/discovery"
+);
+loader.lazyRequireGetter(this, "cert", "devtools/shared/security/cert");
+loader.lazyRequireGetter(
+  this,
+  "Authenticators",
+  "devtools/shared/security/auth",
+  true
+);
+loader.lazyRequireGetter(
+  this,
+  "AuthenticationResult",
+  "devtools/shared/security/auth",
+  true
+);
+loader.lazyRequireGetter(
+  this,
+  "DevToolsSocketStatus",
+  "resource://devtools/shared/security/DevToolsSocketStatus.jsm",
+  true
+);
+
+loader.lazyRequireGetter(this, "EventEmitter", "devtools/shared/event-emitter");
 
 DevToolsUtils.defineLazyGetter(this, "nsFile", () => {
   return CC("@mozilla.org/file/local;1", "nsIFile", "initWithPath");
 });
 
 DevToolsUtils.defineLazyGetter(this, "socketTransportService", () => {
-  return Cc["@mozilla.org/network/socket-transport-service;1"]
-         .getService(Ci.nsISocketTransportService);
+  return Cc["@mozilla.org/network/socket-transport-service;1"].getService(
+    Ci.nsISocketTransportService
+  );
 });
 
 DevToolsUtils.defineLazyGetter(this, "certOverrideService", () => {
-  return Cc["@mozilla.org/security/certoverride;1"]
-         .getService(Ci.nsICertOverrideService);
+  return Cc["@mozilla.org/security/certoverride;1"].getService(
+    Ci.nsICertOverrideService
+  );
 });
 
 DevToolsUtils.defineLazyGetter(this, "nssErrorsService", () => {
-  return Cc["@mozilla.org/nss_errors_service;1"]
-         .getService(Ci.nsINSSErrorsService);
+  return Cc["@mozilla.org/nss_errors_service;1"].getService(
+    Ci.nsINSSErrorsService
+  );
 });
 
 var DebuggerSocket = {};
 
 /**
- * Connects to a debugger server socket.
+ * Connects to a devtools server socket.
  *
  * @param host string
- *        The host name or IP address of the debugger server.
+ *        The host name or IP address of the devtools server.
  * @param port number
- *        The port number of the debugger server.
+ *        The port number of the devtools server.
  * @param encryption boolean (optional)
  *        Whether the server requires encryption.  Defaults to false.
  * @param webSocket boolean (optional)
@@ -73,20 +99,21 @@ var DebuggerSocket = {};
  * @return promise
  *         Resolved to a DebuggerTransport instance.
  */
-DebuggerSocket.connect = async function (settings) {
+DebuggerSocket.connect = async function(settings) {
   // Default to PROMPT |Authenticator| instance if not supplied
   if (!settings.authenticator) {
     settings.authenticator = new (Authenticators.get().Client)();
   }
   _validateSettings(settings);
-  let { host, port, encryption, authenticator, cert } = settings;
-  let transport = await _getTransport(settings);
+  // eslint-disable-next-line no-shadow
+  const { host, port, encryption, authenticator, cert } = settings;
+  const transport = await _getTransport(settings);
   await authenticator.authenticate({
     host,
     port,
     encryption,
     cert,
-    transport
+    transport,
   });
   transport.connectionSettings = settings;
   return transport;
@@ -96,7 +123,7 @@ DebuggerSocket.connect = async function (settings) {
  * Validate that the connection settings have been set to a supported configuration.
  */
 function _validateSettings(settings) {
-  let { encryption, webSocket, authenticator } = settings;
+  const { encryption, webSocket, authenticator } = settings;
 
   if (webSocket && encryption) {
     throw new Error("Encryption not supported on WebSocket transport");
@@ -109,9 +136,9 @@ function _validateSettings(settings) {
  * connect attempts in the process.
  *
  * @param host string
- *        The host name or IP address of the debugger server.
+ *        The host name or IP address of the devtools server.
  * @param port number
- *        The port number of the debugger server.
+ *        The port number of the devtools server.
  * @param encryption boolean (optional)
  *        Whether the server requires encryption.  Defaults to false.
  * @param webSocket boolean (optional)
@@ -125,13 +152,13 @@ function _validateSettings(settings) {
  *         A possible DevTools transport (if connection succeeded and streams
  *         are actually alive and working)
  */
-var _getTransport = async function (settings) {
-  let { host, port, encryption, webSocket } = settings;
+var _getTransport = async function(settings) {
+  const { host, port, encryption, webSocket } = settings;
 
   if (webSocket) {
     // Establish a connection and wait until the WebSocket is ready to send and receive
-    let socket = await new Promise((resolve, reject) => {
-      let s = new WebSocket(`ws://${host}:${port}`);
+    const socket = await new Promise((resolve, reject) => {
+      const s = new WebSocket(`ws://${host}:${port}`);
       s.onopen = () => resolve(s);
       s.onerror = err => reject(err);
     });
@@ -169,9 +196,9 @@ var _getTransport = async function (settings) {
  * after storing a cert override.
  *
  * @param host string
- *        The host name or IP address of the debugger server.
+ *        The host name or IP address of the devtools server.
  * @param port number
- *        The port number of the debugger server.
+ *        The port number of the devtools server.
  * @param encryption boolean (optional)
  *        Whether the server requires encryption.  Defaults to false.
  * @param authenticator Authenticator
@@ -187,17 +214,17 @@ var _getTransport = async function (settings) {
  * @return s nsISocketTransport
  *         Underlying socket transport, in case more details are needed.
  */
-var _attemptTransport = async function (settings) {
-  let { authenticator } = settings;
+var _attemptTransport = async function(settings) {
+  const { authenticator } = settings;
   // _attemptConnect only opens the streams.  Any failures at that stage
   // aborts the connection process immedidately.
-  let { s, input, output } = await _attemptConnect(settings);
+  const { s, input, output } = await _attemptConnect(settings);
 
   // Check if the input stream is alive.  If encryption is enabled, we need to
   // watch out for cert errors by testing the input stream.
   let alive, certError;
   try {
-    let results = await _isInputAlive(input);
+    const results = await _isInputAlive(input);
     alive = results.alive;
     certError = results.certError;
   } catch (e) {
@@ -211,13 +238,15 @@ var _attemptTransport = async function (settings) {
 
   // The |Authenticator| examines the connection as well and may determine it
   // should be dropped.
-  alive = alive && authenticator.validateConnection({
-    host: settings.host,
-    port: settings.port,
-    encryption: settings.encryption,
-    cert: settings.cert,
-    socket: s
-  });
+  alive =
+    alive &&
+    authenticator.validateConnection({
+      host: settings.host,
+      port: settings.port,
+      encryption: settings.encryption,
+      cert: settings.cert,
+      socket: s,
+    });
 
   let transport;
   if (alive) {
@@ -245,13 +274,20 @@ var _attemptTransport = async function (settings) {
  * @return output nsIAsyncOutputStream
  *         The socket's output stream.
  */
-var _attemptConnect = async function ({ host, port, encryption }) {
+var _attemptConnect = async function({ host, port, encryption }) {
   let s;
   if (encryption) {
-    s = socketTransportService.createTransport(["ssl"], 1, host, port, null);
+    s = socketTransportService.createTransport(["ssl"], host, port, null, null);
   } else {
-    s = socketTransportService.createTransport(null, 0, host, port, null);
+    s = socketTransportService.createTransport([], host, port, null, null);
   }
+
+  // Force disabling IPV6 if we aren't explicitely connecting to an IPv6 address
+  // It fails intermitently on MacOS when opening the Browser Toolbox (bug 1615412)
+  if (!host.includes(":")) {
+    s.connectionFlags |= Ci.nsISocketTransport.DISABLE_IPV6;
+  }
+
   // By default the CONNECT socket timeout is very long, 65535 seconds,
   // so that if we race to be in CONNECT state while the server socket is still
   // initializing, the connection is stuck in connecting state for 18.20 hours!
@@ -264,45 +300,48 @@ var _attemptConnect = async function ({ host, port, encryption }) {
     clientCert = await cert.local.getOrCreate();
   }
 
-  let deferred = defer();
   let input;
   let output;
-  // Delay opening the input stream until the transport has fully connected.
-  // The goal is to avoid showing the user a client cert UI prompt when
-  // encryption is used.  This prompt is shown when the client opens the input
-  // stream and does not know which client cert to present to the server.  To
-  // specify a client cert programmatically, we need to access the transport's
-  // nsISSLSocketControl interface, which is not accessible until the transport
-  // has connected.
-  s.setEventSink({
-    onTransportStatus(transport, status) {
-      if (status != Ci.nsISocketTransport.STATUS_CONNECTING_TO) {
-        return;
-      }
-      if (encryption) {
-        let sslSocketControl =
-          transport.securityInfo.QueryInterface(Ci.nsISSLSocketControl);
-        sslSocketControl.clientCert = clientCert;
-      }
-      try {
-        input = s.openInputStream(0, 0, 0);
-      } catch (e) {
-        deferred.reject(e);
-      }
-      deferred.resolve({ s, input, output });
+  return new Promise((resolve, reject) => {
+    // Delay opening the input stream until the transport has fully connected.
+    // The goal is to avoid showing the user a client cert UI prompt when
+    // encryption is used.  This prompt is shown when the client opens the input
+    // stream and does not know which client cert to present to the server.  To
+    // specify a client cert programmatically, we need to access the transport's
+    // nsISSLSocketControl interface, which is not accessible until the transport
+    // has connected.
+    s.setEventSink(
+      {
+        onTransportStatus(transport, status) {
+          if (status != Ci.nsISocketTransport.STATUS_CONNECTING_TO) {
+            return;
+          }
+          if (encryption) {
+            const sslSocketControl = transport.securityInfo.QueryInterface(
+              Ci.nsISSLSocketControl
+            );
+            sslSocketControl.clientCert = clientCert;
+          }
+          try {
+            input = s.openInputStream(0, 0, 0);
+          } catch (e) {
+            reject(e);
+          }
+          resolve({ s, input, output });
+        },
+      },
+      Services.tm.currentThread
+    );
+
+    // openOutputStream may throw NS_ERROR_NOT_INITIALIZED if we hit some race
+    // where the nsISocketTransport gets shutdown in between its instantiation and
+    // the call to this method.
+    try {
+      output = s.openOutputStream(0, 0, 0);
+    } catch (e) {
+      reject(e);
     }
-  }, Services.tm.currentThread);
-
-  // openOutputStream may throw NS_ERROR_NOT_INITIALIZED if we hit some race
-  // where the nsISocketTransport gets shutdown in between its instantiation and
-  // the call to this method.
-  try {
-    output = s.openOutputStream(0, 0, 0);
-  } catch (e) {
-    deferred.reject(e);
-  }
-
-  deferred.promise.catch(e => {
+  }).catch(e => {
     if (input) {
       input.close();
     }
@@ -311,8 +350,6 @@ var _attemptConnect = async function ({ host, port, encryption }) {
     }
     DevToolsUtils.reportException("_attemptConnect", e);
   });
-
-  return deferred.promise;
 };
 
 /**
@@ -321,28 +358,33 @@ var _attemptConnect = async function ({ host, port, encryption }) {
  * first connection to a new host because the cert is self-signed.
  */
 function _isInputAlive(input) {
-  let deferred = defer();
-  input.asyncWait({
-    onInputStreamReady(stream) {
-      try {
-        stream.available();
-        deferred.resolve({ alive: true });
-      } catch (e) {
-        try {
-          // getErrorClass may throw if you pass a non-NSS error
-          let errorClass = nssErrorsService.getErrorClass(e.result);
-          if (errorClass === Ci.nsINSSErrorsService.ERROR_CLASS_BAD_CERT) {
-            deferred.resolve({ certError: true });
-          } else {
-            deferred.reject(e);
+  return new Promise((resolve, reject) => {
+    input.asyncWait(
+      {
+        onInputStreamReady(stream) {
+          try {
+            stream.available();
+            resolve({ alive: true });
+          } catch (e) {
+            try {
+              // getErrorClass may throw if you pass a non-NSS error
+              const errorClass = nssErrorsService.getErrorClass(e.result);
+              if (errorClass === Ci.nsINSSErrorsService.ERROR_CLASS_BAD_CERT) {
+                resolve({ certError: true });
+              } else {
+                reject(e);
+              }
+            } catch (nssErr) {
+              reject(e);
+            }
           }
-        } catch (nssErr) {
-          deferred.reject(e);
-        }
-      }
-    }
-  }, 0, 0, Services.tm.currentThread);
-  return deferred.promise;
+        },
+      },
+      0,
+      0,
+      Services.tm.currentThread
+    );
+  });
 }
 
 /**
@@ -351,57 +393,104 @@ function _isInputAlive(input) {
  * these connections.
  */
 function _storeCertOverride(s, host, port) {
-  let cert = s.securityInfo.QueryInterface(Ci.nsISSLStatusProvider)
-              .SSLStatus.serverCert;
-  let overrideBits = Ci.nsICertOverrideService.ERROR_UNTRUSTED |
-                     Ci.nsICertOverrideService.ERROR_MISMATCH;
-  certOverrideService.rememberValidityOverride(host, port, cert, overrideBits,
-                                               true /* temporary */);
+  // eslint-disable-next-line no-shadow
+  const cert = s.securityInfo.QueryInterface(Ci.nsITransportSecurityInfo)
+    .serverCert;
+  const overrideBits =
+    Ci.nsICertOverrideService.ERROR_UNTRUSTED |
+    Ci.nsICertOverrideService.ERROR_MISMATCH;
+  certOverrideService.rememberValidityOverride(
+    host,
+    port,
+    {},
+    cert,
+    overrideBits,
+    true /* temporary */
+  );
 }
 
 /**
- * Creates a new socket listener for remote connections to the DebuggerServer.
+ * Creates a new socket listener for remote connections to the DevToolsServer.
  * This helps contain and organize the parts of the server that may differ or
  * are particular to one given listener mechanism vs. another.
+ * This can be closed at any later time by calling |close|.
+ * If remote connections are disabled, an error is thrown.
+ *
+ * @param {DevToolsServer} devToolsServer
+ * @param {Object} socketOptions
+ *        options of socket as follows
+ *        {
+ *          authenticator:
+ *            Controls the |Authenticator| used, which hooks various socket steps to
+ *            implement an authentication policy.  It is expected that different use
+ *            cases may override pieces of the |Authenticator|.  See auth.js.
+ *            We set the default |Authenticator|, which is |Prompt|.
+ *          discoverable:
+ *            Controls whether this listener is announced via the service discovery
+ *            mechanism. Defaults is false.
+ *          encryption:
+ *            Controls whether this listener's transport uses encryption.
+ *            Defaults is false.
+ *          fromBrowserToolbox:
+ *            Should only be passed when opening a socket for a Browser Toolbox
+ *            session. This will skip notifying DevToolsSocketStatus
+ *            about the opened socket, to avoid triggering the visual cue in the
+ *            URL bar.
+ *          portOrPath:
+ *            The port or path to listen on.
+ *            If given an integer, the port to listen on.  Use -1 to choose any available
+ *            port. Otherwise, the path to the unix socket domain file to listen on.
+ *            Defaults is null.
+ *          webSocket:
+ *            Whether to use WebSocket protocol. Defaults is false.
+ *        }
  */
-function SocketListener() {}
+function SocketListener(devToolsServer, socketOptions) {
+  this._devToolsServer = devToolsServer;
+
+  // Set socket options with default value
+  this._socketOptions = {
+    authenticator:
+      socketOptions.authenticator || new (Authenticators.get().Server)(),
+    discoverable: !!socketOptions.discoverable,
+    encryption: !!socketOptions.encryption,
+    fromBrowserToolbox: !!socketOptions.fromBrowserToolbox,
+    portOrPath: socketOptions.portOrPath || null,
+    webSocket: !!socketOptions.webSocket,
+  };
+
+  EventEmitter.decorate(this);
+}
 
 SocketListener.prototype = {
+  get authenticator() {
+    return this._socketOptions.authenticator;
+  },
 
-  /* Socket Options */
+  get discoverable() {
+    return this._socketOptions.discoverable;
+  },
 
-  /**
-   * The port or path to listen on.
-   *
-   * If given an integer, the port to listen on.  Use -1 to choose any available
-   * port. Otherwise, the path to the unix socket domain file to listen on.
-   */
-  portOrPath: null,
+  get encryption() {
+    return this._socketOptions.encryption;
+  },
 
-  /**
-   * Controls whether this listener is announced via the service discovery
-   * mechanism.
-   */
-  discoverable: false,
+  get fromBrowserToolbox() {
+    return this._socketOptions.fromBrowserToolbox;
+  },
 
-  /**
-   * Controls whether this listener's transport uses encryption.
-   */
-  encryption: false,
+  get portOrPath() {
+    return this._socketOptions.portOrPath;
+  },
 
-  /**
-   * Controls the |Authenticator| used, which hooks various socket steps to
-   * implement an authentication policy.  It is expected that different use
-   * cases may override pieces of the |Authenticator|.  See auth.js.
-   *
-   * Here we set the default |Authenticator|, which is |Prompt|.
-   */
-  authenticator: new (Authenticators.get().Server)(),
+  get webSocket() {
+    return this._socketOptions.webSocket;
+  },
 
   /**
    * Validate that all options have been set to a supported configuration.
    */
-  _validateOptions: function () {
+  _validateOptions: function() {
     if (this.portOrPath === null) {
       throw new Error("Must set a port / path to listen on.");
     }
@@ -417,9 +506,9 @@ SocketListener.prototype = {
   /**
    * Listens on the given port or socket file for remote debugger connections.
    */
-  open: function () {
+  open: function() {
     this._validateOptions();
-    DebuggerServer._addListener(this);
+    this._devToolsServer.addSocketListener(this);
 
     let flags = Ci.nsIServerSocket.KeepWhenOffline;
     // A preference setting can force binding on the loopback interface.
@@ -427,38 +516,50 @@ SocketListener.prototype = {
       flags |= Ci.nsIServerSocket.LoopbackOnly;
     }
 
-    let self = this;
-    return (async function () {
-      let backlog = 4;
+    const self = this;
+    return (async function() {
+      const backlog = 4;
       self._socket = self._createSocketInstance();
       if (self.isPortBased) {
-        let port = Number(self.portOrPath);
+        const port = Number(self.portOrPath);
         self._socket.initSpecialConnection(port, flags, backlog);
-      } else {
-        let file = nsFile(self.portOrPath);
+      } else if (self.portOrPath.startsWith("/")) {
+        const file = nsFile(self.portOrPath);
         if (file.exists()) {
           file.remove(false);
         }
         self._socket.initWithFilename(file, parseInt("666", 8), backlog);
+      } else {
+        // Path isn't absolute path, so we use abstract socket address
+        self._socket.initWithAbstractAddress(self.portOrPath, backlog);
       }
       await self._setAdditionalSocketOptions();
       self._socket.asyncListen(self);
       dumpn("Socket listening on: " + (self.port || self.portOrPath));
-    })().then(() => {
-      this._advertise();
-    }).catch(e => {
-      dumpn("Could not start debugging listener on '" + this.portOrPath +
-            "': " + e);
-      this.close();
-    });
+    })()
+      .then(() => {
+        if (!self.fromBrowserToolbox) {
+          DevToolsSocketStatus.notifySocketOpened();
+        }
+        this._advertise();
+      })
+      .catch(e => {
+        dumpn(
+          "Could not start debugging listener on '" +
+            this.portOrPath +
+            "': " +
+            e
+        );
+        this.close();
+      });
   },
 
-  _advertise: function () {
+  _advertise: function() {
     if (!this.discoverable || !this.port) {
       return;
     }
 
-    let advertisement = {
+    const advertisement = {
       port: this.port,
       encryption: this.encryption,
     };
@@ -468,21 +569,22 @@ SocketListener.prototype = {
     discovery.addService("devtools", advertisement);
   },
 
-  _createSocketInstance: function () {
+  _createSocketInstance: function() {
     if (this.encryption) {
-      return Cc["@mozilla.org/network/tls-server-socket;1"]
-             .createInstance(Ci.nsITLSServerSocket);
+      return Cc["@mozilla.org/network/tls-server-socket;1"].createInstance(
+        Ci.nsITLSServerSocket
+      );
     }
-    return Cc["@mozilla.org/network/server-socket;1"]
-           .createInstance(Ci.nsIServerSocket);
+    return Cc["@mozilla.org/network/server-socket;1"].createInstance(
+      Ci.nsIServerSocket
+    );
   },
 
   async _setAdditionalSocketOptions() {
     if (this.encryption) {
       this._socket.serverCert = await cert.local.getOrCreate();
-      this._socket.setSessionCache(false);
       this._socket.setSessionTickets(false);
-      let requestCert = Ci.nsITLSServerSocket.REQUEST_NEVER;
+      const requestCert = Ci.nsITLSServerSocket.REQUEST_NEVER;
       this._socket.setRequestClientCertificate(requestCert);
     }
     this.authenticator.augmentSocketOptions(this, this._socket);
@@ -492,15 +594,19 @@ SocketListener.prototype = {
    * Closes the SocketListener.  Notifies the server to remove the listener from
    * the set of active SocketListeners.
    */
-  close: function () {
+  close: function() {
     if (this.discoverable && this.port) {
       discovery.removeService("devtools");
     }
     if (this._socket) {
       this._socket.close();
       this._socket = null;
+
+      if (!this.fromBrowserToolbox) {
+        DevToolsSocketStatus.notifySocketClosed();
+      }
     }
-    DebuggerServer._removeListener(this);
+    this._devToolsServer.removeSocketListener(this);
   },
 
   get host() {
@@ -536,21 +642,29 @@ SocketListener.prototype = {
       return null;
     }
     return {
-      sha256: this._socket.serverCert.sha256Fingerprint
+      sha256: this._socket.serverCert.sha256Fingerprint,
     };
   },
 
+  onAllowedConnection(transport) {
+    dumpn("onAllowedConnection, transport: " + transport);
+    this.emit("accepted", transport, this);
+  },
+
   // nsIServerSocketListener implementation
 
-  onSocketAccepted:
-  DevToolsUtils.makeInfallible(function (socket, socketTransport) {
-    new ServerSocketConnection(this, socketTransport);
-  }, "SocketListener.onSocketAccepted"),
+  onSocketAccepted: DevToolsUtils.makeInfallible(function(
+    socket,
+    socketTransport
+  ) {
+    const connection = new ServerSocketConnection(this, socketTransport);
+    connection.once("allowed", this.onAllowedConnection.bind(this));
+  },
+  "SocketListener.onSocketAccepted"),
 
-  onStopListening: function (socket, status) {
+  onStopListening: function(socket, status) {
     dumpn("onStopListening, status: " + status);
-  }
-
+  },
 };
 
 // Client must complete TLS handshake within this window (ms)
@@ -562,16 +676,16 @@ loader.lazyGetter(this, "HANDSHAKE_TIMEOUT", () => {
  * A |ServerSocketConnection| is created by a |SocketListener| for each accepted
  * incoming socket.  This is a short-lived object used to implement
  * authentication and verify encryption prior to handing off the connection to
- * the |DebuggerServer|.
+ * the |DevToolsServer|.
  */
 function ServerSocketConnection(listener, socketTransport) {
   this._listener = listener;
   this._socketTransport = socketTransport;
   this._handle();
+  EventEmitter.decorate(this);
 }
 
 ServerSocketConnection.prototype = {
-
   get authentication() {
     return this._listener.authenticator.mode;
   },
@@ -589,7 +703,7 @@ ServerSocketConnection.prototype = {
       return null;
     }
     return {
-      sha256: this._clientCert.sha256Fingerprint
+      sha256: this._clientCert.sha256Fingerprint,
     };
   },
 
@@ -598,9 +712,9 @@ ServerSocketConnection.prototype = {
   },
 
   get client() {
-    let client = {
+    const client = {
       host: this.host,
-      port: this.port
+      port: this.port,
     };
     if (this.cert) {
       client.cert = this.cert;
@@ -609,9 +723,9 @@ ServerSocketConnection.prototype = {
   },
 
   get server() {
-    let server = {
+    const server = {
       host: this._listener.host,
-      port: this._listener.port
+      port: this._listener.port,
     };
     if (this._listener.cert) {
       server.cert = this._listener.cert;
@@ -622,17 +736,19 @@ ServerSocketConnection.prototype = {
   /**
    * This is the main authentication workflow.  If any pieces reject a promise,
    * the connection is denied.  If the entire process resolves successfully,
-   * the connection is finally handed off to the |DebuggerServer|.
+   * the connection is finally handed off to the |DevToolsServer|.
    */
-  _handle() {
+  async _handle() {
     dumpn("Debugging connection starting authentication on " + this.address);
-    let self = this;
-    (async function () {
-      self._listenForTLSHandshake();
-      await self._createTransport();
-      await self._awaitTLSHandshake();
-      await self._authenticate();
-    })().then(() => this.allow()).catch(e => this.deny(e));
+    try {
+      this._listenForTLSHandshake();
+      await this._createTransport();
+      await this._awaitTLSHandshake();
+      await this._authenticate();
+      this.allow();
+    } catch (e) {
+      this.deny(e);
+    }
   },
 
   /**
@@ -640,11 +756,15 @@ ServerSocketConnection.prototype = {
    * TLS sockets to keep the handshake moving.
    */
   async _createTransport() {
-    let input = this._socketTransport.openInputStream(0, 0, 0);
-    let output = this._socketTransport.openOutputStream(0, 0, 0);
+    const input = this._socketTransport.openInputStream(0, 0, 0);
+    const output = this._socketTransport.openOutputStream(0, 0, 0);
 
     if (this._listener.webSocket) {
-      let socket = await WebSocketServer.accept(this._socketTransport, input, output);
+      const socket = await WebSocketServer.accept(
+        this._socketTransport,
+        input,
+        output
+      );
       this._transport = new WebSocketDebuggerTransport(socket);
     } else {
       this._transport = new DebuggerTransport(input, output);
@@ -653,9 +773,9 @@ ServerSocketConnection.prototype = {
     // Start up the transport to observe the streams in case they are closed
     // early.  This allows us to clean up our state as well.
     this._transport.hooks = {
-      onClosed: reason => {
+      onTransportClosed: reason => {
         this.deny(reason);
-      }
+      },
     };
     this._transport.ready();
   },
@@ -668,8 +788,9 @@ ServerSocketConnection.prototype = {
     if (!this._socketTransport || !this._socketTransport.securityInfo) {
       return;
     }
-    let connectionInfo = this._socketTransport.securityInfo
-                         .QueryInterface(Ci.nsITLSServerConnectionInfo);
+    const connectionInfo = this._socketTransport.securityInfo.QueryInterface(
+      Ci.nsITLSServerConnectionInfo
+    );
     connectionInfo.setSecurityObserver(observer);
   },
 
@@ -679,59 +800,61 @@ ServerSocketConnection.prototype = {
    * |onHandshakeDone|.
    */
   _listenForTLSHandshake() {
-    this._handshakeDeferred = defer();
     if (!this._listener.encryption) {
-      this._handshakeDeferred.resolve();
+      this._handshakePromise = Promise.resolve();
       return;
     }
-    this._setSecurityObserver(this);
-    this._handshakeTimeout = setTimeout(this._onHandshakeTimeout.bind(this),
-                                        HANDSHAKE_TIMEOUT);
+
+    this._handshakePromise = new Promise((resolve, reject) => {
+      this._observer = {
+        // nsITLSServerSecurityObserver implementation
+        onHandshakeDone: (socket, clientStatus) => {
+          clearTimeout(this._handshakeTimeout);
+          this._setSecurityObserver(null);
+          dumpv("TLS version:    " + clientStatus.tlsVersionUsed.toString(16));
+          dumpv("TLS cipher:     " + clientStatus.cipherName);
+          dumpv("TLS key length: " + clientStatus.keyLength);
+          dumpv("TLS MAC length: " + clientStatus.macLength);
+          this._clientCert = clientStatus.peerCert;
+          /*
+           * TODO: These rules should be really be set on the TLS socket directly, but
+           * this would need more platform work to expose it via XPCOM.
+           *
+           * Enforcing cipher suites here would be a bad idea, as we want TLS
+           * cipher negotiation to work correctly.  The server already allows only
+           * Gecko's normal set of cipher suites.
+           */
+          if (
+            clientStatus.tlsVersionUsed < Ci.nsITLSClientStatus.TLS_VERSION_1_2
+          ) {
+            reject(Cr.NS_ERROR_CONNECTION_REFUSED);
+            return;
+          }
+
+          resolve();
+        },
+      };
+      this._setSecurityObserver(this._observer);
+      this._handshakeTimeout = setTimeout(() => {
+        dumpv("Client failed to complete TLS handshake");
+        reject(Cr.NS_ERROR_NET_TIMEOUT);
+      }, HANDSHAKE_TIMEOUT);
+    });
   },
 
   _awaitTLSHandshake() {
-    return this._handshakeDeferred.promise;
-  },
-
-  _onHandshakeTimeout() {
-    dumpv("Client failed to complete TLS handshake");
-    this._handshakeDeferred.reject(Cr.NS_ERROR_NET_TIMEOUT);
-  },
-
-  // nsITLSServerSecurityObserver implementation
-  onHandshakeDone(socket, clientStatus) {
-    clearTimeout(this._handshakeTimeout);
-    this._setSecurityObserver(null);
-    dumpv("TLS version:    " + clientStatus.tlsVersionUsed.toString(16));
-    dumpv("TLS cipher:     " + clientStatus.cipherName);
-    dumpv("TLS key length: " + clientStatus.keyLength);
-    dumpv("TLS MAC length: " + clientStatus.macLength);
-    this._clientCert = clientStatus.peerCert;
-    /*
-     * TODO: These rules should be really be set on the TLS socket directly, but
-     * this would need more platform work to expose it via XPCOM.
-     *
-     * Enforcing cipher suites here would be a bad idea, as we want TLS
-     * cipher negotiation to work correctly.  The server already allows only
-     * Gecko's normal set of cipher suites.
-     */
-    if (clientStatus.tlsVersionUsed < Ci.nsITLSClientStatus.TLS_VERSION_1_2) {
-      this._handshakeDeferred.reject(Cr.NS_ERROR_CONNECTION_REFUSED);
-      return;
-    }
-
-    this._handshakeDeferred.resolve();
+    return this._handshakePromise;
   },
 
   async _authenticate() {
-    let result = await this._listener.authenticator.authenticate({
+    const result = await this._listener.authenticator.authenticate({
       client: this.client,
       server: this.server,
-      transport: this._transport
+      transport: this._transport,
     });
     switch (result) {
       case AuthenticationResult.DISABLE_ALL:
-        DebuggerServer.closeAllListeners();
+        this._listener._devToolsServer.closeAllSocketListeners();
         Services.prefs.setBoolPref("devtools.debugger.remote-enabled", false);
         return promise.reject(Cr.NS_ERROR_CONNECTION_REFUSED);
       case AuthenticationResult.DENY:
@@ -749,14 +872,15 @@ ServerSocketConnection.prototype = {
       return;
     }
     let errorName = result;
-    for (let name in Cr) {
+    for (const name in Cr) {
       if (Cr[name] === result) {
         errorName = name;
         break;
       }
     }
-    dumpn("Debugging connection denied on " + this.address +
-          " (" + errorName + ")");
+    dumpn(
+      "Debugging connection denied on " + this.address + " (" + errorName + ")"
+    );
     if (this._transport) {
       this._transport.hooks = null;
       this._transport.close(result);
@@ -770,7 +894,7 @@ ServerSocketConnection.prototype = {
       return;
     }
     dumpn("Debugging connection allowed on " + this.address);
-    DebuggerServer._onConnection(this._transport);
+    this.emit("allowed", this._transport);
     this.destroy();
   },
 
@@ -782,12 +906,8 @@ ServerSocketConnection.prototype = {
     this._socketTransport = null;
     this._transport = null;
     this._clientCert = null;
-  }
-
-};
-
-DebuggerSocket.createListener = function () {
-  return new SocketListener();
+  },
 };
 
 exports.DebuggerSocket = DebuggerSocket;
+exports.SocketListener = SocketListener;
