@@ -16,11 +16,11 @@
 #include "nsWindowsDllInterceptor.h"
 #include "mozilla/CmdLineAndEnvUtils.h"
 #include "mozilla/DebugOnly.h"
-#include "mozilla/ScopeExit.h"
 #include "mozilla/StackWalk_windows.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Vector.h"
+#include "mozilla/WindowsProcessMitigations.h"
 #include "mozilla/WindowsVersion.h"
 #include "mozilla/WinHeaderOnlyUtils.h"
 #include "nsWindowsHelpers.h"
@@ -70,8 +70,8 @@ typedef NTSTATUS(NTAPI* LdrLoadDll_func)(PWCHAR filePath, PULONG flags,
 static WindowsDllInterceptor::FuncHookType<LdrLoadDll_func> stub_LdrLoadDll;
 
 #ifdef _M_AMD64
-typedef decltype(
-    RtlInstallFunctionTableCallback)* RtlInstallFunctionTableCallback_func;
+typedef decltype(RtlInstallFunctionTableCallback)*
+    RtlInstallFunctionTableCallback_func;
 static WindowsDllInterceptor::FuncHookType<RtlInstallFunctionTableCallback_func>
     stub_RtlInstallFunctionTableCallback;
 
@@ -468,6 +468,11 @@ static NTSTATUS NTAPI patched_LdrLoadDll(PWCHAR filePath, PULONG flags,
         goto continue_loading;
       }
 
+      if ((info->mFlags & DllBlockInfo::BLOCK_WIN7_AND_OLDER) &&
+          IsWin8OrLater()) {
+        goto continue_loading;
+      }
+
       if ((info->mFlags & DllBlockInfo::CHILD_PROCESSES_ONLY) &&
           !(sInitFlags & eDllBlocklistInitFlagIsChildProcess)) {
         goto continue_loading;
@@ -593,7 +598,7 @@ static WindowsDllInterceptor Kernel32Intercept;
 static void GetNativeNtBlockSetWriter();
 
 static glue::LoaderObserver gMozglueLoaderObserver;
-static nt::LoaderAPI::InitDllBlocklistOOPFnPtr gInitDllBlocklistOOPFnPtr;
+static nt::WinLauncherFunctions gWinLauncherFunctions;
 
 MFBT_API void DllBlocklist_Initialize(uint32_t aInitFlags) {
   if (sBlocklistInitAttempted) {
@@ -603,8 +608,8 @@ MFBT_API void DllBlocklist_Initialize(uint32_t aInitFlags) {
 
   sInitFlags = aInitFlags;
 
-  gInitDllBlocklistOOPFnPtr =
-      glue::ModuleLoadFrame::StaticInit(&gMozglueLoaderObserver);
+  glue::ModuleLoadFrame::StaticInit(&gMozglueLoaderObserver,
+                                    &gWinLauncherFunctions);
 
 #ifdef _M_AMD64
   if (!IsWin8OrLater()) {
@@ -668,7 +673,7 @@ MFBT_API void DllBlocklist_Initialize(uint32_t aInitFlags) {
   // If someone injects a thread early that causes user32.dll to load off the
   // main thread this causes issues, so load it as soon as we've initialized
   // the block-list. (See bug 1400637)
-  if (!sUser32BeforeBlocklist) {
+  if (!sUser32BeforeBlocklist && !IsWin32kLockedDown()) {
     ::LoadLibraryW(L"user32.dll");
   }
 
@@ -765,7 +770,7 @@ MFBT_API void DllBlocklist_SetFullDllServices(
   glue::AutoExclusiveLock lock(gDllServicesLock);
   if (aSvc) {
     aSvc->SetAuthenticodeImpl(GetAuthenticode());
-    aSvc->SetInitDllBlocklistOOPFnPtr(gInitDllBlocklistOOPFnPtr);
+    aSvc->SetWinLauncherFunctions(gWinLauncherFunctions);
     gMozglueLoaderObserver.Forward(aSvc);
   }
 
