@@ -5,12 +5,13 @@
 
 #include "SocketProcessHost.h"
 
-#include "nsAppRunner.h"
-#include "nsIObserverService.h"
-#include "nsIOService.h"
 #include "SocketProcessParent.h"
-#include "ProcessUtils.h"
 #include "mozilla/ipc/FileDescriptor.h"
+#include "mozilla/ipc/ProcessUtils.h"
+#include "nsAppRunner.h"
+#include "nsIOService.h"
+#include "nsIObserverService.h"
+#include "ProfilerParent.h"
 
 #if defined(XP_LINUX) && defined(MOZ_SANDBOX)
 #  include "mozilla/SandboxBroker.h"
@@ -18,10 +19,6 @@
 #  include "mozilla/SandboxSettings.h"
 #endif
 
-#ifdef MOZ_GECKO_PROFILER
-#  include "ProfilerParent.h"
-#endif
-
 #if defined(XP_MACOSX) && defined(MOZ_SANDBOX)
 #  include "mozilla/Sandbox.h"
 #endif
@@ -31,65 +28,6 @@ using namespace mozilla::ipc;
 namespace mozilla {
 namespace net {
 
-#define NS_IPC_IOSERVICE_SET_OFFLINE_TOPIC "ipc:network:set-offline"
-
-class OfflineObserver final : public nsIObserver {
- public:
-  NS_DECL_THREADSAFE_ISUPPORTS
-  explicit OfflineObserver(SocketProcessHost* aProcessHost)
-      : mProcessHost(aProcessHost) {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
-    if (obs) {
-      obs->AddObserver(this, NS_IPC_IOSERVICE_SET_OFFLINE_TOPIC, false);
-      obs->AddObserver(this, NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID, false);
-    }
-  }
-
-  void Destroy() {
-    MOZ_ASSERT(NS_IsMainThread());
-
-    nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
-    if (obs) {
-      obs->RemoveObserver(this, NS_IPC_IOSERVICE_SET_OFFLINE_TOPIC);
-    }
-    mProcessHost = nullptr;
-  }
-
- private:
-  // nsIObserver implementation.
-  NS_IMETHOD
-  Observe(nsISupports* aSubject, const char* aTopic,
-          const char16_t* aData) override {
-    if (!mProcessHost) {
-      return NS_OK;
-    }
-
-    if (!strcmp(aTopic, NS_IPC_IOSERVICE_SET_OFFLINE_TOPIC)) {
-      NS_ConvertUTF16toUTF8 dataStr(aData);
-      const char* offline = dataStr.get();
-      if (!mProcessHost->IsConnected() ||
-          mProcessHost->GetActor()->SendSetOffline(
-              !strcmp(offline, "true") ? true : false)) {
-        return NS_ERROR_NOT_AVAILABLE;
-      }
-    } else if (!strcmp(aTopic, NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID)) {
-      nsCOMPtr<nsIObserverService> obs =
-          mozilla::services::GetObserverService();
-      obs->RemoveObserver(this, NS_IPC_IOSERVICE_SET_OFFLINE_TOPIC);
-      obs->RemoveObserver(this, NS_XPCOM_WILL_SHUTDOWN_OBSERVER_ID);
-    }
-
-    return NS_OK;
-  }
-  virtual ~OfflineObserver() = default;
-
-  SocketProcessHost* mProcessHost;
-};
-
-NS_IMPL_ISUPPORTS(OfflineObserver, nsIObserver)
-
 #if defined(XP_MACOSX) && defined(MOZ_SANDBOX)
 bool SocketProcessHost::sLaunchWithMacSandbox = false;
 #endif
@@ -112,15 +50,7 @@ SocketProcessHost::SocketProcessHost(Listener* aListener)
 #endif
 }
 
-SocketProcessHost::~SocketProcessHost() {
-  MOZ_COUNT_DTOR(SocketProcessHost);
-  if (mOfflineObserver) {
-    RefPtr<OfflineObserver> observer = mOfflineObserver;
-    NS_DispatchToMainThread(
-        NS_NewRunnableFunction("SocketProcessHost::DestroyOfflineObserver",
-                               [observer]() { observer->Destroy(); }));
-  }
-}
+SocketProcessHost::~SocketProcessHost() { MOZ_COUNT_DTOR(SocketProcessHost); }
 
 bool SocketProcessHost::Launch() {
   MOZ_ASSERT(mLaunchPhase == LaunchPhase::Unlaunched);
@@ -201,50 +131,53 @@ void SocketProcessHost::InitAfterConnect(bool aSucceeded) {
   MOZ_ASSERT(NS_IsMainThread());
 
   mLaunchPhase = LaunchPhase::Complete;
-
-  if (aSucceeded) {
-    mSocketProcessParent = MakeUnique<SocketProcessParent>(this);
-    DebugOnly<bool> rv = mSocketProcessParent->Open(
-        TakeChannel(), base::GetProcId(GetChildProcessHandle()));
-    MOZ_ASSERT(rv);
-
-    nsCOMPtr<nsIIOService> ioService(do_GetIOService());
-    MOZ_ASSERT(ioService, "No IO service?");
-    bool offline = false;
-    DebugOnly<nsresult> result = ioService->GetOffline(&offline);
-    MOZ_ASSERT(NS_SUCCEEDED(result), "Failed getting offline?");
-
-    Maybe<FileDescriptor> brokerFd;
-
-#if defined(XP_LINUX) && defined(MOZ_SANDBOX)
-    if (GetEffectiveSocketProcessSandboxLevel() > 0) {
-      auto policy = SandboxBrokerPolicyFactory::GetSocketProcessPolicy(
-          GetActor()->OtherPid());
-      if (policy != nullptr) {
-        brokerFd = Some(FileDescriptor());
-        mSandboxBroker = SandboxBroker::Create(
-            std::move(policy), GetActor()->OtherPid(), brokerFd.ref());
-        // This is unlikely to fail and probably indicates OS resource
-        // exhaustion.
-        Unused << NS_WARN_IF(mSandboxBroker == nullptr);
-        MOZ_ASSERT(brokerFd.ref().IsValid());
-      }
-      Unused << GetActor()->SendInitLinuxSandbox(brokerFd);
+  if (!aSucceeded) {
+    if (mListener) {
+      mListener->OnProcessLaunchComplete(this, false);
     }
-#endif  // XP_LINUX && MOZ_SANDBOX
-
-#ifdef MOZ_GECKO_PROFILER
-    Unused << GetActor()->SendInitProfiler(
-        ProfilerParent::CreateForProcess(GetActor()->OtherPid()));
-#endif
-
-    Unused << GetActor()->SendSetOffline(offline);
-
-    mOfflineObserver = new OfflineObserver(this);
+    return;
   }
 
+  mSocketProcessParent = MakeUnique<SocketProcessParent>(this);
+  DebugOnly<bool> rv = mSocketProcessParent->Open(
+      TakeInitialPort(), base::GetProcId(GetChildProcessHandle()));
+  MOZ_ASSERT(rv);
+
+  SocketPorcessInitAttributes attributes;
+  nsCOMPtr<nsIIOService> ioService(do_GetIOService());
+  MOZ_ASSERT(ioService, "No IO service?");
+  DebugOnly<nsresult> result = ioService->GetOffline(&attributes.mOffline());
+  MOZ_ASSERT(NS_SUCCEEDED(result), "Failed getting offline?");
+  result = ioService->GetConnectivity(&attributes.mConnectivity());
+  MOZ_ASSERT(NS_SUCCEEDED(result), "Failed getting connectivity?");
+
+  attributes.mInitSandbox() = false;
+
+#if defined(XP_LINUX) && defined(MOZ_SANDBOX)
+  if (GetEffectiveSocketProcessSandboxLevel() > 0) {
+    auto policy = SandboxBrokerPolicyFactory::GetSocketProcessPolicy(
+        GetActor()->OtherPid());
+    if (policy != nullptr) {
+      attributes.mSandboxBroker() = Some(FileDescriptor());
+      mSandboxBroker =
+          SandboxBroker::Create(std::move(policy), GetActor()->OtherPid(),
+                                attributes.mSandboxBroker().ref());
+      // This is unlikely to fail and probably indicates OS resource
+      // exhaustion.
+      Unused << NS_WARN_IF(mSandboxBroker == nullptr);
+      MOZ_ASSERT(attributes.mSandboxBroker().ref().IsValid());
+    }
+    attributes.mInitSandbox() = true;
+  }
+#endif  // XP_LINUX && MOZ_SANDBOX
+
+  Unused << GetActor()->SendInit(attributes);
+
+  Unused << GetActor()->SendInitProfiler(
+      ProfilerParent::CreateForProcess(GetActor()->OtherPid()));
+
   if (mListener) {
-    mListener->OnProcessLaunchComplete(this, aSucceeded);
+    mListener->OnProcessLaunchComplete(this, true);
   }
 }
 
@@ -253,10 +186,6 @@ void SocketProcessHost::Shutdown() {
   MOZ_ASSERT(NS_IsMainThread());
 
   mListener = nullptr;
-  if (mOfflineObserver) {
-    mOfflineObserver->Destroy();
-    mOfflineObserver = nullptr;
-  }
 
   if (mSocketProcessParent) {
     // OnChannelClosed uses this to check if the shutdown was expected or
@@ -297,7 +226,7 @@ void SocketProcessHost::DestroyProcess() {
     mTaskFactory.RevokeAll();
   }
 
-  MessageLoop::current()->PostTask(NS_NewRunnableFunction(
+  GetCurrentSerialEventTarget()->Dispatch(NS_NewRunnableFunction(
       "DestroySocketProcessRunnable", [this] { Destroy(); }));
 }
 
