@@ -158,7 +158,7 @@ class TLSFilterTransaction final : public nsAHttpTransaction,
   int32_t FilterInput(char* aBuf, int32_t aAmount);
 
   static PRStatus GetPeerName(PRFileDesc* fd, PRNetAddr* addr);
-  static PRStatus GetSocketOption(PRFileDesc* fd, PRSocketOptionData* data);
+  static PRStatus GetSocketOption(PRFileDesc* fd, PRSocketOptionData* aOpt);
   static PRStatus SetSocketOption(PRFileDesc* fd,
                                   const PRSocketOptionData* data);
   static int32_t FilterWrite(PRFileDesc* fd, const void* buf, int32_t amount);
@@ -221,7 +221,9 @@ class SpdyConnectTransaction final : public NullHttpTransaction {
   // CONNECT tunnel but the setup fails. The plaintext only carries the CONNECT
   // error.
   void ForcePlainText();
-  void MapStreamToHttpConnection(nsISocketTransport* aTransport,
+  // True if we successfully map stream to a nsHttpConnection. Currently we skip
+  // 1xx response only.
+  bool MapStreamToHttpConnection(nsISocketTransport* aTransport,
                                  nsHttpConnectionInfo* aConnInfo,
                                  const nsACString& aFlat407Headers,
                                  int32_t aHttpResponseCode);
@@ -233,7 +235,7 @@ class SpdyConnectTransaction final : public NullHttpTransaction {
                                        uint32_t count,
                                        uint32_t* countWritten) final;
   nsHttpRequestHead* RequestHead() final;
-  void Close(nsresult reason) final;
+  void Close(nsresult code) final;
 
   // ConnectedReadyForInput() tests whether the spdy connect transaction is
   // attached to an nsHttpConnection that can properly deal with flow control,
@@ -269,7 +271,6 @@ class SpdyConnectTransaction final : public NullHttpTransaction {
 
   bool mForcePlainText;
   TimeStamp mTimestampSyn;
-  RefPtr<nsHttpConnectionInfo> mConnInfo;
 
   // mTunneledConn, mTunnelTransport, mTunnelStreamIn, mTunnelStreamOut
   // are the connectors to the "real" http connection. They are created
