@@ -17,7 +17,6 @@
 //-----------------------------------------------------------------------------
 
 #include "nsIPrefBranch.h"
-#include "nsIPrefService.h"
 #include "nsIHttpAuthenticableChannel.h"
 #include "nsIURI.h"
 #ifdef XP_WIN
@@ -28,9 +27,11 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/Base64.h"
 #include "mozilla/CheckedInt.h"
+#include "mozilla/Maybe.h"
 #include "mozilla/Tokenizer.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Unused.h"
+#include "nsCRT.h"
 #include "nsNetUtil.h"
 #include "nsIChannel.h"
 #include "nsUnicharUtils.h"
@@ -51,13 +52,12 @@ StaticRefPtr<nsHttpNTLMAuth> nsHttpNTLMAuth::gSingleton;
 
 static bool IsNonFqdn(nsIURI* uri) {
   nsAutoCString host;
-  PRNetAddr addr;
-
-  if (NS_FAILED(uri->GetAsciiHost(host))) return false;
+  if (NS_FAILED(uri->GetAsciiHost(host))) {
+    return false;
+  }
 
   // return true if host does not contain a dot and is not an ip address
-  return !host.IsEmpty() && !host.Contains('.') &&
-         PR_StringToNetAddr(host.BeginReading(), &addr) != PR_SUCCESS;
+  return !host.IsEmpty() && !host.Contains('.') && !HostIsIPLiteral(host);
 }
 
 // Check to see if we should use our generic (internal) NTLM auth module.
@@ -113,8 +113,9 @@ static bool CanUseDefaultCredentials(nsIHttpAuthenticableChannel* channel,
   Unused << channel->GetURI(getter_AddRefs(uri));
 
   bool allowNonFqdn;
-  if (NS_FAILED(prefs->GetBoolPref(kAllowNonFqdn, &allowNonFqdn)))
+  if (NS_FAILED(prefs->GetBoolPref(kAllowNonFqdn, &allowNonFqdn))) {
     allowNonFqdn = false;
+  }
   if (allowNonFqdn && uri && IsNonFqdn(uri)) {
     LOG(("Host is non-fqdn, default credentials are allowed\n"));
     return true;
@@ -171,7 +172,7 @@ nsHttpNTLMAuth::ChallengeReceived(nsIHttpAuthenticableChannel* channel,
   // Start a new auth sequence if the challenge is exactly "NTLM".
   // If native NTLM auth apis are available and enabled through prefs,
   // try to use them.
-  if (PL_strcasecmp(challenge, "NTLM") == 0) {
+  if (nsCRT::strcasecmp(challenge, "NTLM") == 0) {
     nsCOMPtr<nsIAuthModule> module;
 
     // Check to see if we should default to our generic NTLM auth module
@@ -217,9 +218,8 @@ nsHttpNTLMAuth::ChallengeReceived(nsIHttpAuthenticableChannel* channel,
       if (!*sessionState) {
         // Remember the fact that we cannot use the "sys-ntlm" module,
         // so we don't ever bother trying again for this auth domain.
-        *sessionState = new nsNTLMSessionState();
-        if (!*sessionState) return NS_ERROR_OUT_OF_MEMORY;
-        NS_ADDREF(*sessionState);
+        RefPtr<nsNTLMSessionState> state = new nsNTLMSessionState();
+        state.forget(sessionState);
       }
 
       // Use our internal NTLM implementation. Note, this is less secure,
@@ -283,9 +283,10 @@ nsHttpNTLMAuth::GenerateCredentials(nsIHttpAuthenticableChannel* authChannel,
 
   void *inBuf, *outBuf;
   uint32_t inBufLen, outBufLen;
+  Maybe<nsTArray<uint8_t>> certArray;
 
   // initial challenge
-  if (PL_strcasecmp(challenge, "NTLM") == 0) {
+  if (nsCRT::strcasecmp(challenge, "NTLM") == 0) {
     // NTLM service name format is 'HTTP@host' for both http and https
     nsCOMPtr<nsIURI> uri;
     rv = authChannel->GetURI(getter_AddRefs(uri));
@@ -331,15 +332,14 @@ nsHttpNTLMAuth::GenerateCredentials(nsIHttpAuthenticableChannel* authChannel,
       rv = secInfo->GetServerCert(getter_AddRefs(cert));
       if (NS_FAILED(rv)) return rv;
 
-      uint32_t length;
-      uint8_t* certArray;
-      rv = cert->GetRawDER(&length, &certArray);
+      certArray.emplace();
+      rv = cert->GetRawDER(*certArray);
       if (NS_FAILED(rv)) return rv;
 
       // If there is a server certificate, we pass it along the
       // first time we call GetNextToken().
-      inBufLen = length;
-      inBuf = certArray;
+      inBufLen = certArray->Length();
+      inBuf = certArray->Elements();
     } else {
       // If there is no server certificate, we don't pass anything.
       inBufLen = 0;
@@ -358,7 +358,7 @@ nsHttpNTLMAuth::GenerateCredentials(nsIHttpAuthenticableChannel* authChannel,
     len -= 5;
 
     // strip off any padding (see bug 230351)
-    while (challenge[len - 1] == '=') len--;
+    while (len && challenge[len - 1] == '=') len--;
 
     // decode into the input secbuffer
     rv = Base64Decode(challenge, len, (char**)&inBuf, &inBufLen);
@@ -387,7 +387,10 @@ nsHttpNTLMAuth::GenerateCredentials(nsIHttpAuthenticableChannel* authChannel,
     free(outBuf);
   }
 
-  if (inBuf) free(inBuf);
+  // inBuf needs to be freed if it's not pointing into certArray
+  if (inBuf && !certArray) {
+    free(inBuf);
+  }
 
   return rv;
 }
