@@ -195,8 +195,9 @@ bool mozTXTToHTMLConv::FindURLStart(const char16_t* aInString,
               IsAsciiDigit(aInString[uint32_t(i)]) ||
               aInString[uint32_t(i)] == '+' || aInString[uint32_t(i)] == '-' ||
               aInString[uint32_t(i)] == '.');
-           i--)
+           i--) {
         ;
+      }
       if (++i >= 0 && uint32_t(i) < pos &&
           IsAsciiAlpha(aInString[uint32_t(i)])) {
         start = uint32_t(i);
@@ -219,8 +220,9 @@ bool mozTXTToHTMLConv::FindURLStart(const char16_t* aInString,
              !IsSpace(aInString[uint32_t(i)]) &&
              (!isEmail || IsAscii(aInString[uint32_t(i)])) &&
              (!isEmail || aInString[uint32_t(i)] != ')');
-           i--)
+           i--) {
         ;
+      }
       if (++i >= 0 && uint32_t(i) < pos &&
           (IsAsciiAlpha(aInString[uint32_t(i)]) ||
            IsAsciiDigit(aInString[uint32_t(i)]))) {
@@ -268,13 +270,15 @@ bool mozTXTToHTMLConv::FindURLEnd(const char16_t* aInString,
             // Allow IPv6 adresses like http://[1080::8:800:200C:417A]/foo.
             (aInString[i] == '[' && i > 2 &&
              (aInString[i - 1] != '/' || aInString[i - 2] != '/')) ||
-            IsSpace(aInString[i]))
+            IsSpace(aInString[i])) {
           break;
+        }
         // Disallow non-ascii-characters for email.
         // Currently correct, but revisit later after standards changed.
         if (isEmail && (aInString[i] == '(' || aInString[i] == '\'' ||
-                        !IsAscii(aInString[i])))
+                        !IsAscii(aInString[i]))) {
           break;
+        }
         if (aInString[i] == '(') seenOpeningParenthesis = true;
         if (aInString[i] == '[') seenOpeningSquareBracket = true;
       }
@@ -283,8 +287,9 @@ bool mozTXTToHTMLConv::FindURLEnd(const char16_t* aInString,
       while (--i > pos && (aInString[i] == '.' || aInString[i] == ',' ||
                            aInString[i] == ';' || aInString[i] == '!' ||
                            aInString[i] == '?' || aInString[i] == '-' ||
-                           aInString[i] == ':' || aInString[i] == '\''))
+                           aInString[i] == ':' || aInString[i] == '\'')) {
         ;
+      }
       if (i > pos) {
         end = i;
         return true;
@@ -452,8 +457,9 @@ bool mozTXTToHTMLConv::FindURL(const char16_t* aInString, int32_t aInLength,
   /* all modes but abbreviated are checked for text[pos] == ':',
      only abbreviated for '.', RFC2396E and abbreviated for '@' */
   for (modetype iState = unknown; iState <= mozTXTToHTMLConv_lastMode;
-       iState = modetype(iState + 1))
+       iState = modetype(iState + 1)) {
     state[iState] = aInString[pos] == ':' ? unchecked : invalid;
+  }
   switch (aInString[pos]) {
     case '@':
       state[RFC2396E] = unchecked;
@@ -480,13 +486,17 @@ bool mozTXTToHTMLConv::FindURL(const char16_t* aInString, int32_t aInLength,
 
     uint32_t start, end;
 
-    if (state[check] == unchecked)
-      if (FindURLStart(aInString, aInLength, pos, check, start))
+    if (state[check] == unchecked) {
+      if (FindURLStart(aInString, aInLength, pos, check, start)) {
         state[check] = startok;
+      }
+    }
 
-    if (state[check] == startok)
-      if (FindURLEnd(aInString, aInLength, pos, check, start, end))
+    if (state[check] == startok) {
+      if (FindURLEnd(aInString, aInLength, pos, check, start, end)) {
         state[check] = endok;
+      }
+    }
 
     if (state[check] == endok) {
       nsAutoString txtURL, desc;
@@ -536,8 +546,9 @@ bool mozTXTToHTMLConv::ItMatchesDelimited(const char16_t* aInString,
       ((before != LT_IGNORE || (after != LT_IGNORE && after != LT_DELIMITER)) &&
        textLen < aRepLen + 1) ||
       (before != LT_IGNORE && after != LT_IGNORE && after != LT_DELIMITER &&
-       textLen < aRepLen + 2))
+       textLen < aRepLen + 2)) {
     return false;
+  }
 
   uint32_t text0 = aInString[0];
   if (aInLength > 1 && NS_IS_SURROGATE_PAIR(text0, aInString[1])) {
@@ -558,22 +569,19 @@ bool mozTXTToHTMLConv::ItMatchesDelimited(const char16_t* aInString,
     textAfterPos = SURROGATE_TO_UCS4(textAfterPos, aInString[afterIndex + 1]);
   }
 
-  if ((before == LT_ALPHA && !IsAlpha(text0)) ||
-      (before == LT_DIGIT && !IsDigit(text0)) ||
-      (before == LT_DELIMITER &&
-       (IsAlpha(text0) || IsDigit(text0) || text0 == *rep)) ||
-      (after == LT_ALPHA && !IsAlpha(textAfterPos)) ||
-      (after == LT_DIGIT && !IsDigit(textAfterPos)) ||
-      (after == LT_DELIMITER &&
-       (IsAlpha(textAfterPos) || IsDigit(textAfterPos) ||
-        textAfterPos == *rep)) ||
-      !Substring(Substring(aInString, aInString + aInLength), ignoreLen,
-                 aRepLen)
-           .Equals(Substring(rep, rep + aRepLen),
-                   nsCaseInsensitiveStringComparator))
-    return false;
-
-  return true;
+  return !((before == LT_ALPHA && !IsAlpha(text0)) ||
+           (before == LT_DIGIT && !IsDigit(text0)) ||
+           (before == LT_DELIMITER &&
+            (IsAlpha(text0) || IsDigit(text0) || text0 == *rep)) ||
+           (after == LT_ALPHA && !IsAlpha(textAfterPos)) ||
+           (after == LT_DIGIT && !IsDigit(textAfterPos)) ||
+           (after == LT_DELIMITER &&
+            (IsAlpha(textAfterPos) || IsDigit(textAfterPos) ||
+             textAfterPos == *rep)) ||
+           !Substring(Substring(aInString, aInString + aInLength), ignoreLen,
+                      aRepLen)
+                .Equals(Substring(rep, rep + aRepLen),
+                        nsCaseInsensitiveStringComparator));
 }
 
 uint32_t mozTXTToHTMLConv::NumberOfMatches(const char16_t* aInString,
@@ -631,9 +639,8 @@ bool mozTXTToHTMLConv::StructPhraseHit(
   }
 
   // closing tag
-  else if (openTags > 0 &&
-           ItMatchesDelimited(aInString, aInStringLength, tagTXT, aTagTXTLen,
-                              LT_ALPHA, LT_DELIMITER)) {
+  if (openTags > 0 && ItMatchesDelimited(aInString, aInStringLength, tagTXT,
+                                         aTagTXTLen, LT_ALPHA, LT_DELIMITER)) {
     openTags--;
     aOutString.AppendLiteral("<span class=\"moz-txt-tag\">");
     aOutString.Append(tagTXT);
@@ -845,8 +852,9 @@ bool mozTXTToHTMLConv::GlyphHit(const char16_t* aInString, int32_t aInLength,
            (IsAsciiDigit(aInString[delimPos]) ||
             (aInString[delimPos] == '.' && delimPos + 1 < aInLength &&
              IsAsciiDigit(aInString[delimPos + 1])));
-         delimPos++)
+         delimPos++) {
       ;
+    }
 
     if (delimPos < aInLength && IsAsciiAlpha(aInString[delimPos])) {
       return false;
@@ -932,16 +940,17 @@ int32_t mozTXTToHTMLConv::CiteLevelTXT(const char16_t* line,
       // here, |logLineStart < lineLength| is always true
       uint32_t minlength = std::min(uint32_t(6), NS_strlen(indexString));
       if (Substring(indexString, indexString + minlength)
-              .Equals(Substring(NS_LITERAL_STRING(">From "), 0, minlength),
-                      nsCaseInsensitiveStringComparator))
+              .Equals(Substring(u">From "_ns, 0, minlength),
+                      nsCaseInsensitiveStringComparator)) {
         // XXX RFC2646
         moreCites = false;
-      else {
+      } else {
         result++;
         logLineStart = i;
       }
-    } else
+    } else {
       moreCites = false;
+    }
   }
 
   return result;
@@ -1129,28 +1138,31 @@ mozTXTToHTMLConv::ScanHTML(const nsAString& input, uint32_t whattodo,
       // Make sure there's a white-space character after, not to match "abbr".
       {
         i = aInString.Find("</a>", true, i);
-        if (i == kNotFound)
+        if (i == kNotFound) {
           i = lengthOfInString;
-        else
+        } else {
           i += 4;
+        }
       } else if (Substring(aInString, i + 1, 3).LowerCaseEqualsASCII("!--"))
       // if out-commended code, skip until -->
       {
         i = aInString.Find("-->", false, i);
-        if (i == kNotFound)
+        if (i == kNotFound) {
           i = lengthOfInString;
-        else
+        } else {
           i += 3;
+        }
       } else if (i + 6 < lengthOfInString &&
                  Substring(aInString, i + 1, 5).LowerCaseEqualsASCII("style") &&
                  canFollow.FindChar(aInString[i + 6]) != kNotFound)
       // if style tag, skip until </style>
       {
         i = aInString.Find("</style>", true, i);
-        if (i == kNotFound)
+        if (i == kNotFound) {
           i = lengthOfInString;
-        else
+        } else {
           i += 8;
+        }
       } else if (i + 7 < lengthOfInString &&
                  Substring(aInString, i + 1, 6)
                      .LowerCaseEqualsASCII("script") &&
@@ -1158,10 +1170,11 @@ mozTXTToHTMLConv::ScanHTML(const nsAString& input, uint32_t whattodo,
       // if script tag, skip until </script>
       {
         i = aInString.Find("</script>", true, i);
-        if (i == kNotFound)
+        if (i == kNotFound) {
           i = lengthOfInString;
-        else
+        } else {
           i += 9;
+        }
       } else if (i + 5 < lengthOfInString &&
                  Substring(aInString, i + 1, 4).LowerCaseEqualsASCII("head") &&
                  canFollow.FindChar(aInString[i + 5]) != kNotFound)
@@ -1169,17 +1182,19 @@ mozTXTToHTMLConv::ScanHTML(const nsAString& input, uint32_t whattodo,
       // Make sure not to match <header>.
       {
         i = aInString.Find("</head>", true, i);
-        if (i == kNotFound)
+        if (i == kNotFound) {
           i = lengthOfInString;
-        else
+        } else {
           i += 7;
+        }
       } else  // just skip tag (attributes etc.)
       {
         i = aInString.FindChar('>', i);
-        if (i == kNotFound)
+        if (i == kNotFound) {
           i = lengthOfInString;
-        else
+        } else {
           i++;
+        }
       }
       aOutString.Append(&uniBuffer[start], i - start);
     } else {
