@@ -18,7 +18,9 @@
 #include "nsServiceManagerUtils.h"
 #include "nsNetCID.h"
 #include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 #include "mozilla/dom/Document.h"
+#include "mozilla/Preferences.h"
 #include "mozilla/Unused.h"
 #include "mozilla/net/CookieJarSettings.h"
 #include "Cookie.h"
@@ -86,7 +88,7 @@ void SetACookieInternal(nsICookieService* aCookieService, const char* aSpec,
                 nsIContentPolicy::TYPE_OTHER);
 
   nsCOMPtr<nsICookieJarSettings> cookieJarSettings =
-      aAllowed ? CookieJarSettings::Create()
+      aAllowed ? CookieJarSettings::Create(CookieJarSettings::eRegular)
                : CookieJarSettings::GetBlockingAll();
   MOZ_ASSERT(cookieJarSettings);
 
@@ -137,9 +139,9 @@ void GetACookieNoHttp(nsICookieService* aCookieService, const char* aSpec,
 
   nsCOMPtr<mozilla::dom::Document> document;
   nsresult rv = NS_NewDOMDocument(getter_AddRefs(document),
-                                  EmptyString(),  // aNamespaceURI
-                                  EmptyString(),  // aQualifiedName
-                                  nullptr,        // aDoctype
+                                  u""_ns,   // aNamespaceURI
+                                  u""_ns,   // aQualifiedName
+                                  nullptr,  // aDoctype
                                   uri, uri, principal,
                                   false,    // aLoadedAsData
                                   nullptr,  // aEventObject
@@ -193,12 +195,13 @@ void InitPrefs(nsIPrefBranch* aPrefBranch) {
   // Set the base domain limit to 50 so we have a known value.
   aPrefBranch->SetIntPref(kCookiesMaxPerHost, 50);
 
-  // SameSite=none by default. We have other tests for lax-by-default.
-  // XXX: Bug 1617611 - Fix all the tests broken by "cookies sameSite=lax by
+  // SameSite=None by default. We have other tests for lax-by-default.
+  // XXX: Bug 1617611 - Fix all the tests broken by "cookies SameSite=Lax by
   // default"
   Preferences::SetBool("network.cookie.sameSite.laxByDefault", false);
   Preferences::SetBool("network.cookieJarSettings.unblocked_for_testing", true);
   Preferences::SetBool("dom.securecontext.whitelist_onions", false);
+  Preferences::SetBool("network.cookie.sameSite.schemeful", false);
 }
 
 TEST(TestCookie, TestCookieMain)
@@ -725,6 +728,7 @@ TEST(TestCookie, TestCookieMain)
   GetACookieNoHttp(cookieService, "https://www.security.test/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_EQUAL,
                           "test_modify_cookie=non-security-cookie"));
+
   // Test the non-security cookie can set when domain or path not same to secure
   // cookie of same name.
   SetACookie(cookieService, "https://www.security.test/", "test=security3");
@@ -735,12 +739,17 @@ TEST(TestCookie, TestCookieMain)
   GetACookieNoHttp(cookieService, "http://www.security.test/", cookie);
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_CONTAIN, "test=non-security2"));
 
+  Preferences::SetBool("network.cookie.sameSite.schemeful", true);
+  GetACookieNoHttp(cookieService, "http://www.security.test/", cookie);
+  EXPECT_FALSE(CheckResult(cookie.get(), MUST_CONTAIN, "test=security3"));
+  Preferences::SetBool("network.cookie.sameSite.schemeful", false);
+
   // *** nsICookieManager interface tests
   nsCOMPtr<nsICookieManager> cookieMgr =
       do_GetService(NS_COOKIEMANAGER_CONTRACTID, &rv0);
   ASSERT_TRUE(NS_SUCCEEDED(rv0));
 
-  nsCOMPtr<nsICookieManager> cookieMgr2 = cookieMgr;
+  const nsCOMPtr<nsICookieManager>& cookieMgr2 = cookieMgr;
   ASSERT_TRUE(cookieMgr2);
 
   mozilla::OriginAttributes attrs;
@@ -748,39 +757,39 @@ TEST(TestCookie, TestCookieMain)
   // first, ensure a clean slate
   EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->RemoveAll()));
   // add some cookies
-  EXPECT_TRUE(NS_SUCCEEDED(
-      cookieMgr2->AddNative(NS_LITERAL_CSTRING("cookiemgr.test"),  // domain
-                            NS_LITERAL_CSTRING("/foo"),            // path
-                            NS_LITERAL_CSTRING("test1"),           // name
-                            NS_LITERAL_CSTRING("yes"),             // value
-                            false,                                 // is secure
-                            false,      // is httponly
-                            true,       // is session
-                            INT64_MAX,  // expiry time
-                            &attrs,     // originAttributes
-                            nsICookie::SAMESITE_NONE)));
-  EXPECT_TRUE(NS_SUCCEEDED(
-      cookieMgr2->AddNative(NS_LITERAL_CSTRING("cookiemgr.test"),  // domain
-                            NS_LITERAL_CSTRING("/foo"),            // path
-                            NS_LITERAL_CSTRING("test2"),           // name
-                            NS_LITERAL_CSTRING("yes"),             // value
-                            false,                                 // is secure
-                            true,                            // is httponly
-                            true,                            // is session
-                            PR_Now() / PR_USEC_PER_SEC + 2,  // expiry time
-                            &attrs,                          // originAttributes
-                            nsICookie::SAMESITE_NONE)));
-  EXPECT_TRUE(NS_SUCCEEDED(
-      cookieMgr2->AddNative(NS_LITERAL_CSTRING("new.domain"),  // domain
-                            NS_LITERAL_CSTRING("/rabbit"),     // path
-                            NS_LITERAL_CSTRING("test3"),       // name
-                            NS_LITERAL_CSTRING("yes"),         // value
-                            false,                             // is secure
-                            false,                             // is httponly
-                            true,                              // is session
-                            INT64_MAX,                         // expiry time
-                            &attrs,  // originAttributes
-                            nsICookie::SAMESITE_NONE)));
+  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->AddNative("cookiemgr.test"_ns,  // domain
+                                                 "/foo"_ns,            // path
+                                                 "test1"_ns,           // name
+                                                 "yes"_ns,             // value
+                                                 false,      // is secure
+                                                 false,      // is httponly
+                                                 true,       // is session
+                                                 INT64_MAX,  // expiry time
+                                                 &attrs,     // originAttributes
+                                                 nsICookie::SAMESITE_NONE,
+                                                 nsICookie::SCHEME_HTTPS)));
+  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->AddNative(
+      "cookiemgr.test"_ns,             // domain
+      "/foo"_ns,                       // path
+      "test2"_ns,                      // name
+      "yes"_ns,                        // value
+      false,                           // is secure
+      true,                            // is httponly
+      true,                            // is session
+      PR_Now() / PR_USEC_PER_SEC + 2,  // expiry time
+      &attrs,                          // originAttributes
+      nsICookie::SAMESITE_NONE, nsICookie::SCHEME_HTTPS)));
+  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->AddNative("new.domain"_ns,  // domain
+                                                 "/rabbit"_ns,     // path
+                                                 "test3"_ns,       // name
+                                                 "yes"_ns,         // value
+                                                 false,            // is secure
+                                                 false,      // is httponly
+                                                 true,       // is session
+                                                 INT64_MAX,  // expiry time
+                                                 &attrs,     // originAttributes
+                                                 nsICookie::SAMESITE_NONE,
+                                                 nsICookie::SCHEME_HTTPS)));
   // confirm using enumerator
   nsTArray<RefPtr<nsICookie>> cookies;
   EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->GetCookies(cookies)));
@@ -788,10 +797,11 @@ TEST(TestCookie, TestCookieMain)
   for (const auto& cookie : cookies) {
     nsAutoCString name;
     cookie->GetName(name);
-    if (name.EqualsLiteral("test2"))
+    if (name.EqualsLiteral("test2")) {
       expiredCookie = cookie;
-    else if (name.EqualsLiteral("test3"))
+    } else if (name.EqualsLiteral("test3")) {
       newDomainCookie = cookie;
+    }
   }
   EXPECT_EQ(cookies.Length(), 3ul);
   // check the httpOnly attribute of the second cookie is honored
@@ -801,26 +811,24 @@ TEST(TestCookie, TestCookieMain)
   EXPECT_TRUE(CheckResult(cookie.get(), MUST_NOT_CONTAIN, "test2=yes"));
   // check CountCookiesFromHost()
   uint32_t hostCookies = 0;
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->CountCookiesFromHost(
-      NS_LITERAL_CSTRING("cookiemgr.test"), &hostCookies)));
+  EXPECT_TRUE(NS_SUCCEEDED(
+      cookieMgr2->CountCookiesFromHost("cookiemgr.test"_ns, &hostCookies)));
   EXPECT_EQ(hostCookies, 2u);
   // check CookieExistsNative() using the third cookie
   bool found;
   EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->CookieExistsNative(
-      NS_LITERAL_CSTRING("new.domain"), NS_LITERAL_CSTRING("/rabbit"),
-      NS_LITERAL_CSTRING("test3"), &attrs, &found)));
+      "new.domain"_ns, "/rabbit"_ns, "test3"_ns, &attrs, &found)));
   EXPECT_TRUE(found);
 
   // sleep four seconds, to make sure the second cookie has expired
   PR_Sleep(4 * PR_TicksPerSecond());
   // check that both CountCookiesFromHost() and CookieExistsNative() count the
   // expired cookie
-  EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->CountCookiesFromHost(
-      NS_LITERAL_CSTRING("cookiemgr.test"), &hostCookies)));
+  EXPECT_TRUE(NS_SUCCEEDED(
+      cookieMgr2->CountCookiesFromHost("cookiemgr.test"_ns, &hostCookies)));
   EXPECT_EQ(hostCookies, 2u);
   EXPECT_TRUE(NS_SUCCEEDED(cookieMgr2->CookieExistsNative(
-      NS_LITERAL_CSTRING("cookiemgr.test"), NS_LITERAL_CSTRING("/foo"),
-      NS_LITERAL_CSTRING("test2"), &attrs, &found)));
+      "cookiemgr.test"_ns, "/foo"_ns, "test2"_ns, &attrs, &found)));
   EXPECT_TRUE(found);
   // double-check RemoveAll() using the enumerator
   EXPECT_TRUE(NS_SUCCEEDED(cookieMgr->RemoveAll()));
@@ -837,14 +845,14 @@ TEST(TestCookie, TestCookieMain)
   nsAutoCString name;
   nsAutoCString expected;
   for (int32_t i = 0; i < 60; ++i) {
-    name = NS_LITERAL_CSTRING("test");
+    name = "test"_ns;
     name.AppendInt(i);
-    name += NS_LITERAL_CSTRING("=creation");
+    name += "=creation"_ns;
     SetACookie(cookieService, "http://creation.ordering.tests/", name.get());
 
     if (i >= 10) {
       expected += name;
-      if (i < 59) expected += NS_LITERAL_CSTRING("; ");
+      if (i < 59) expected += "; "_ns;
     }
   }
   GetACookie(cookieService, "http://creation.ordering.tests/", cookie);
@@ -853,13 +861,13 @@ TEST(TestCookie, TestCookieMain)
   cookieMgr->RemoveAll();
 
   for (int32_t i = 0; i < 60; ++i) {
-    name = NS_LITERAL_CSTRING("test");
+    name = "test"_ns;
     name.AppendInt(i);
-    name += NS_LITERAL_CSTRING("=delete_non_security");
+    name += "=delete_non_security"_ns;
 
     // Create 50 cookies that include the secure flag.
     if (i < 50) {
-      name += NS_LITERAL_CSTRING("; secure");
+      name += "; secure"_ns;
       SetACookie(cookieService, "https://creation.ordering.tests/", name.get());
     } else {
       // non-security cookies will be removed beside the latest cookie that be
