@@ -100,6 +100,7 @@ function TestFailedStreamCallback(transport, hostname, next) {
   this.hostname = hostname;
   this.next = next;
   this.dummyContent = "G";
+  this.closed = false;
 }
 
 TestFailedStreamCallback.prototype = {
@@ -108,11 +109,15 @@ TestFailedStreamCallback.prototype = {
     "nsIOutputStreamCallback",
   ]),
   processException(e) {
+    if (this.closed) {
+      return;
+    }
     do_check_instanceof(e, Ci.nsIException);
     // A refusal to connect speculatively should throw an error.
     Assert.equal(e.result, Cr.NS_ERROR_CONNECTION_REFUSED);
+    this.closed = true;
     this.transport.close(Cr.NS_BINDING_ABORTED);
-    return true;
+    this.next();
   },
   onOutputStreamReady(outstream) {
     info("outputstream handler.");
@@ -121,7 +126,6 @@ TestFailedStreamCallback.prototype = {
       outstream.write(this.dummyContent, this.dummyContent.length);
     } catch (e) {
       this.processException(e);
-      this.next();
       return;
     }
     info("no exception on write. Wait for read.");
@@ -133,7 +137,6 @@ TestFailedStreamCallback.prototype = {
       instream.available();
     } catch (e) {
       this.processException(e);
-      this.next();
       return;
     }
     do_throw("Speculative Connect should have failed for " + this.hostname);
@@ -184,7 +187,7 @@ function test_hostnames_resolving_to_addresses(host, next) {
     Ci.nsISocketTransportService
   );
   Assert.notEqual(typeof sts, undefined);
-  var transport = sts.createTransport([], host, 80, null);
+  var transport = sts.createTransport([], host, 80, null, null);
   Assert.notEqual(typeof transport, undefined);
 
   transport.connectionFlags = Ci.nsISocketTransport.DISABLE_RFC1918;
@@ -266,7 +269,7 @@ function test_proxies(proxyHost, next) {
   var proxyInfo = pps.newProxyInfo("http", proxyHost, 8080, "", "", 0, 1, null);
   Assert.notEqual(typeof proxyInfo, undefined);
 
-  var transport = sts.createTransport([], "dummyHost", 80, proxyInfo);
+  var transport = sts.createTransport([], "dummyHost", 80, proxyInfo, null);
   Assert.notEqual(typeof transport, undefined);
 
   transport.connectionFlags = Ci.nsISocketTransport.DISABLE_RFC1918;
