@@ -6,8 +6,8 @@
 
 #include "UrlClassifierFeatureFlash.h"
 #include "mozilla/net/HttpBaseChannel.h"
-#include "mozilla/StaticPrefs_fission.h"
 #include "mozilla/StaticPrefs_plugins.h"
+#include "nsIXULRuntime.h"
 #include "nsScriptSecurityManager.h"
 #include "nsQueryObject.h"
 
@@ -16,8 +16,8 @@ namespace net {
 
 struct UrlClassifierFeatureFlash::FlashFeature {
   const char* mName;
-  const char* mBlacklistPrefTables;
-  const char* mWhitelistPrefTables;
+  const char* mBlocklistPrefTables;
+  const char* mEntitylistPrefTables;
   bool mSubdocumentOnly;
   nsIHttpChannel::FlashPluginState mFlashPluginState;
   RefPtr<UrlClassifierFeatureFlash> mFeature;
@@ -44,13 +44,13 @@ UrlClassifierFeatureFlash::UrlClassifierFeatureFlash(
     const UrlClassifierFeatureFlash::FlashFeature& aFlashFeature)
     : UrlClassifierFeatureBase(
           nsDependentCString(aFlashFeature.mName),
-          nsDependentCString(aFlashFeature.mBlacklistPrefTables),
-          nsDependentCString(aFlashFeature.mWhitelistPrefTables),
-          EmptyCString(),  // aPrefBlacklistHosts
-          EmptyCString(),  // aPrefWhitelistHosts
-          EmptyCString(),  // aPrefBlacklistTableName
-          EmptyCString(),  // aPrefWhitelistTableName
-          EmptyCString())  // aPrefSkipHosts
+          nsDependentCString(aFlashFeature.mBlocklistPrefTables),
+          nsDependentCString(aFlashFeature.mEntitylistPrefTables),
+          ""_ns,  // aPrefBlocklistHosts
+          ""_ns,  // aPrefEntitylistHosts
+          ""_ns,  // aPrefBlocklistTableName
+          ""_ns,  // aPrefEntitylistTableName
+          ""_ns)  // aPrefExceptionHosts
       ,
       mFlashPluginState(aFlashFeature.mFlashPluginState) {
   static_assert(nsIHttpChannel::FlashPluginDeniedInSubdocuments ==
@@ -98,8 +98,7 @@ void UrlClassifierFeatureFlash::MaybeCreate(
     nsIChannel* aChannel,
     nsTArray<nsCOMPtr<nsIUrlClassifierFeature>>& aFeatures) {
   const auto fnIsFlashBlockingEnabled = [] {
-    return StaticPrefs::plugins_flashBlock_enabled() &&
-           !StaticPrefs::fission_autostart();
+    return StaticPrefs::plugins_flashBlock_enabled() && !FissionAutostart();
   };
 
   // All disabled.
@@ -109,11 +108,11 @@ void UrlClassifierFeatureFlash::MaybeCreate(
 
   // We use Flash feature just for document loading.
   nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
-  nsContentPolicyType contentPolicyType =
+  ExtContentPolicyType contentPolicyType =
       loadInfo->GetExternalContentPolicyType();
 
-  if (contentPolicyType != nsIContentPolicy::TYPE_DOCUMENT &&
-      contentPolicyType != nsIContentPolicy::TYPE_SUBDOCUMENT) {
+  if (contentPolicyType != ExtContentPolicy::TYPE_DOCUMENT &&
+      contentPolicyType != ExtContentPolicy::TYPE_SUBDOCUMENT) {
     return;
   }
 
@@ -130,7 +129,7 @@ void UrlClassifierFeatureFlash::MaybeCreate(
   for (const FlashFeature& flashFeature : sFlashFeaturesMap) {
     MOZ_ASSERT(flashFeature.mFeature);
     if (!flashFeature.mSubdocumentOnly ||
-        contentPolicyType == nsIContentPolicy::TYPE_SUBDOCUMENT) {
+        contentPolicyType == ExtContentPolicy::TYPE_SUBDOCUMENT) {
       aFeatures.AppendElement(flashFeature.mFeature);
     }
   }
@@ -163,7 +162,7 @@ UrlClassifierFeatureFlash::ProcessChannel(nsIChannel* aChannel,
   // This is not a blocking feature.
   *aShouldContinue = true;
 
-  UC_LOG(("UrlClassifierFeatureFlash::ProcessChannel, annotating channel[%p]",
+  UC_LOG(("UrlClassifierFeatureFlash::ProcessChannel - annotating channel %p",
           aChannel));
 
   nsCOMPtr<nsIParentChannel> parentChannel;
@@ -191,9 +190,9 @@ UrlClassifierFeatureFlash::GetURIByListType(
   NS_ENSURE_ARG_POINTER(aURI);
 
   // Here we return the channel's URI always.
-  *aURIType = aListType == nsIUrlClassifierFeature::blacklist
-                  ? nsIUrlClassifierFeature::URIType::blacklistURI
-                  : nsIUrlClassifierFeature::URIType::whitelistURI;
+  *aURIType = aListType == nsIUrlClassifierFeature::blocklist
+                  ? nsIUrlClassifierFeature::URIType::blocklistURI
+                  : nsIUrlClassifierFeature::URIType::entitylistURI;
   return aChannel->GetURI(aURI);
 }
 
