@@ -14,12 +14,13 @@ nsHtml5OplessBuilder* mBuilder;
 // the fields below aren't in use, either.
 // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 nsHtml5Highlighter* mViewSource;
+mozilla::ImportScanner mImportScanner;
 nsTArray<nsHtml5TreeOperation> mOpQueue;
 nsTArray<nsHtml5SpeculativeLoad> mSpeculativeLoadQueue;
 nsAHtml5TreeOpSink* mOpSink;
-mozilla::UniquePtr<nsIContent* []> mHandles;
+mozilla::UniquePtr<nsIContent*[]> mHandles;
 int32_t mHandlesUsed;
-nsTArray<mozilla::UniquePtr<nsIContent* []>> mOldHandles;
+nsTArray<mozilla::UniquePtr<nsIContent*[]>> mOldHandles;
 nsHtml5TreeOpStage* mSpeculativeLoadStage;
 nsresult mBroken;
 bool mCurrentHtmlScriptIsAsyncOrDefer;
@@ -114,7 +115,7 @@ void NeedsCharsetSwitchTo(NotNull<const Encoding*> aEncoding, int32_t aSource,
 void MaybeComplainAboutCharset(const char* aMsgId, bool aError,
                                int32_t aLineNumber);
 
-void TryToDisableEncodingMenu();
+void TryToEnableEncodingMenu();
 
 void AddSnapshotToScript(nsAHtml5TreeBuilderState* aSnapshot, int32_t aLine);
 
@@ -138,6 +139,8 @@ bool EnsureBufferSpace(int32_t aLength);
 
 void EnableViewSource(nsHtml5Highlighter* aHighlighter);
 
+void errDeepTree();
+
 void errStrayStartTag(nsAtom* aName);
 
 void errStrayEndTag(nsAtom* aName);
@@ -176,7 +179,7 @@ void errStartSelectWhereEndSelectExpected();
 
 void errStartTagWithSelectOpen(nsAtom* aName);
 
-void errBadStartTagInHead(nsAtom* aName);
+void errBadStartTagInNoscriptInHead(nsAtom* aName);
 
 void errImage();
 
@@ -212,8 +215,6 @@ void errNoElementToCloseButEndTagSeen(nsAtom* aName);
 
 void errHtmlStartTagInForeignContext(nsAtom* aName);
 
-void errTableClosedWhileCaptionOpen();
-
 void errNoTableRowToClose();
 
 void errNonSpaceInTable();
