@@ -10,26 +10,30 @@
 #include "BRNameMatchingPolicy.h"
 #include "CTPolicyEnforcer.h"
 #include "CTVerifyResult.h"
+#include "EnterpriseRoots.h"
 #include "OCSPCache.h"
+#include "RootCertificateTelemetryUtils.h"
 #include "ScopedNSSTypes.h"
 #include "mozilla/Telemetry.h"
 #include "mozilla/TimeStamp.h"
 #include "mozilla/UniquePtr.h"
 #include "nsString.h"
-#include "pkix/pkixtypes.h"
+#include "mozpkix/pkixtypes.h"
+#include "sslt.h"
 
 #if defined(_MSC_VER)
-#pragma warning(push)
+#  pragma warning(push)
 // Silence "RootingAPI.h(718): warning C4324: 'js::DispatchWrapper<T>':
 // structure was padded due to alignment specifier with [ T=void * ]"
-#pragma warning(disable:4324)
+#  pragma warning(disable : 4324)
 #endif /* defined(_MSC_VER) */
 #include "mozilla/BasePrincipal.h"
 #if defined(_MSC_VER)
-#pragma warning(pop) /* popping the pragma in this file */
-#endif /* defined(_MSC_VER) */
+#  pragma warning(pop) /* popping the pragma in this file */
+#endif                 /* defined(_MSC_VER) */
 
-namespace mozilla { namespace ct {
+namespace mozilla {
+namespace ct {
 
 // Including the headers of the classes below would bring along all of their
 // dependent headers and force us to export them in moz.build.
@@ -37,12 +41,19 @@ namespace mozilla { namespace ct {
 class MultiLogCTVerifier;
 class CTDiversityPolicy;
 
-} } // namespace mozilla::ct
+}  // namespace ct
+}  // namespace mozilla
 
-namespace mozilla { namespace psm {
+namespace mozilla {
+namespace psm {
 
 typedef mozilla::pkix::Result Result;
 
+enum class EVStatus : uint8_t {
+  NotEV = 0,
+  EV = 1,
+};
+
 // These values correspond to the CERT_CHAIN_KEY_SIZE_STATUS telemetry.
 enum class KeySizeStatus {
   NeverChecked = 0,
@@ -61,35 +72,42 @@ enum class SHA1ModeResult {
   Failed = 5,
 };
 
-// Whether or not we are enforcing one of our CA distrust policies. For context,
-// see Bug 1437754 and Bug 1409257.
-enum class DistrustedCAPolicy : uint32_t {
-  Permit = 0,
-  DistrustSymantecRoots = 1,
+enum class CRLiteMode {
+  Disabled = 0,
+  TelemetryOnly = 1,
+  Enforce = 2,
 };
 
 enum class NetscapeStepUpPolicy : uint32_t;
 
-class PinningTelemetryInfo
-{
-public:
-  PinningTelemetryInfo() { Reset(); }
+class PinningTelemetryInfo {
+ public:
+  PinningTelemetryInfo()
+      : certPinningResultBucket(0), rootBucket(ROOT_CERTIFICATE_UNKNOWN) {
+    Reset();
+  }
 
   // Should we accumulate pinning telemetry for the result?
   bool accumulateResult;
-  Telemetry::HistogramID certPinningResultHistogram;
+  Maybe<Telemetry::HistogramID> certPinningResultHistogram;
   int32_t certPinningResultBucket;
   // Should we accumulate telemetry for the root?
   bool accumulateForRoot;
   int32_t rootBucket;
 
-  void Reset() { accumulateForRoot = false; accumulateResult = false; }
+  void Reset() {
+    accumulateForRoot = false;
+    accumulateResult = false;
+  }
 };
 
-class CertificateTransparencyInfo
-{
-public:
-  CertificateTransparencyInfo() { Reset(); }
+class CertificateTransparencyInfo {
+ public:
+  CertificateTransparencyInfo()
+      : enabled(false),
+        policyCompliance(mozilla::ct::CTPolicyCompliance::Unknown) {
+    Reset();
+  }
 
   // Was CT enabled?
   bool enabled;
@@ -101,11 +119,24 @@ public:
   void Reset();
 };
 
+class DelegatedCredentialInfo {
+ public:
+  DelegatedCredentialInfo() : scheme(ssl_sig_none), authKeyBits(0) {}
+  DelegatedCredentialInfo(SSLSignatureScheme scheme, uint32_t authKeyBits)
+      : scheme(scheme), authKeyBits(authKeyBits) {}
+
+  // The signature scheme to be used in CertVerify. This tells us
+  // whether to interpret |authKeyBits| in an RSA or ECDSA context.
+  SSLSignatureScheme scheme;
+
+  // The size of the key, in bits.
+  uint32_t authKeyBits;
+};
+
 class NSSCertDBTrustDomain;
 
-class CertVerifier
-{
-public:
+class CertVerifier {
+ public:
   typedef unsigned int Flags;
   // XXX: FLAG_LOCAL_ONLY is ignored in the classic verification case
   static const Flags FLAG_LOCAL_ONLY;
@@ -126,49 +157,43 @@ public:
   // *evOidPolicy == SEC_OID_UNKNOWN means the cert is NOT EV
   // Only one usage per verification is supported.
   mozilla::pkix::Result VerifyCert(
-                    CERTCertificate* cert,
-                    SECCertificateUsage usage,
-                    mozilla::pkix::Time time,
-                    void* pinArg,
-                    const char* hostname,
-            /*out*/ UniqueCERTCertList& builtChain,
-                    Flags flags = 0,
-    /*optional in*/ const SECItem* stapledOCSPResponse = nullptr,
-    /*optional in*/ const SECItem* sctsFromTLS = nullptr,
-    /*optional in*/ const OriginAttributes& originAttributes =
-                      OriginAttributes(),
-   /*optional out*/ SECOidTag* evOidPolicy = nullptr,
-   /*optional out*/ OCSPStaplingStatus* ocspStaplingStatus = nullptr,
-   /*optional out*/ KeySizeStatus* keySizeStatus = nullptr,
-   /*optional out*/ SHA1ModeResult* sha1ModeResult = nullptr,
-   /*optional out*/ PinningTelemetryInfo* pinningTelemetryInfo = nullptr,
-   /*optional out*/ CertificateTransparencyInfo* ctInfo = nullptr);
+      CERTCertificate* cert, SECCertificateUsage usage,
+      mozilla::pkix::Time time, void* pinArg, const char* hostname,
+      /*out*/ UniqueCERTCertList& builtChain, Flags flags = 0,
+      /*optional in*/
+      const Maybe<nsTArray<nsTArray<uint8_t>>>& extraCertificates = Nothing(),
+      /*optional in*/ const Maybe<nsTArray<uint8_t>>& stapledOCSPResponseArg =
+          Nothing(),
+      /*optional in*/ const Maybe<nsTArray<uint8_t>>& sctsFromTLS = Nothing(),
+      /*optional in*/ const OriginAttributes& originAttributes =
+          OriginAttributes(),
+      /*optional out*/ EVStatus* evStatus = nullptr,
+      /*optional out*/ OCSPStaplingStatus* ocspStaplingStatus = nullptr,
+      /*optional out*/ KeySizeStatus* keySizeStatus = nullptr,
+      /*optional out*/ SHA1ModeResult* sha1ModeResult = nullptr,
+      /*optional out*/ PinningTelemetryInfo* pinningTelemetryInfo = nullptr,
+      /*optional out*/ CertificateTransparencyInfo* ctInfo = nullptr);
 
   mozilla::pkix::Result VerifySSLServerCert(
-                    const UniqueCERTCertificate& peerCert,
-       /*optional*/ const SECItem* stapledOCSPResponse,
-       /*optional*/ const SECItem* sctsFromTLS,
-                    mozilla::pkix::Time time,
-       /*optional*/ void* pinarg,
-                    const nsACString& hostname,
-            /*out*/ UniqueCERTCertList& builtChain,
-       /*optional*/ bool saveIntermediatesInPermanentDatabase = false,
-       /*optional*/ Flags flags = 0,
-       /*optional*/ const OriginAttributes& originAttributes =
-                      OriginAttributes(),
-   /*optional out*/ SECOidTag* evOidPolicy = nullptr,
-   /*optional out*/ OCSPStaplingStatus* ocspStaplingStatus = nullptr,
-   /*optional out*/ KeySizeStatus* keySizeStatus = nullptr,
-   /*optional out*/ SHA1ModeResult* sha1ModeResult = nullptr,
-   /*optional out*/ PinningTelemetryInfo* pinningTelemetryInfo = nullptr,
-   /*optional out*/ CertificateTransparencyInfo* ctInfo = nullptr);
-
-  enum PinningMode {
-    pinningDisabled = 0,
-    pinningAllowUserCAMITM = 1,
-    pinningStrict = 2,
-    pinningEnforceTestMode = 3
-  };
+      const UniqueCERTCertificate& peerCert, mozilla::pkix::Time time,
+      void* pinarg, const nsACString& hostname,
+      /*out*/ UniqueCERTCertList& builtChain,
+      /*optional*/ Flags flags = 0,
+      /*optional*/ const Maybe<nsTArray<nsTArray<uint8_t>>>& extraCertificates =
+          Nothing(),
+      /*optional*/ const Maybe<nsTArray<uint8_t>>& stapledOCSPResponse =
+          Nothing(),
+      /*optional*/ const Maybe<nsTArray<uint8_t>>& sctsFromTLS = Nothing(),
+      /*optional*/ const Maybe<DelegatedCredentialInfo>& dcInfo = Nothing(),
+      /*optional*/ const OriginAttributes& originAttributes =
+          OriginAttributes(),
+      /*optional out*/ EVStatus* evStatus = nullptr,
+      /*optional out*/ OCSPStaplingStatus* ocspStaplingStatus = nullptr,
+      /*optional out*/ KeySizeStatus* keySizeStatus = nullptr,
+      /*optional out*/ SHA1ModeResult* sha1ModeResult = nullptr,
+      /*optional out*/ PinningTelemetryInfo* pinningTelemetryInfo = nullptr,
+      /*optional out*/ CertificateTransparencyInfo* ctInfo = nullptr,
+      /*optional out*/ bool* isBuiltCertChainRootBuiltInRoot = nullptr);
 
   enum class SHA1Mode {
     Allowed = 0,
@@ -181,47 +206,48 @@ public:
     ImportedRootOrBefore2016 = 4,
   };
 
-  enum OcspDownloadConfig {
-    ocspOff = 0,
-    ocspOn = 1,
-    ocspEVOnly = 2
-  };
+  enum OcspDownloadConfig { ocspOff = 0, ocspOn = 1, ocspEVOnly = 2 };
   enum OcspStrictConfig { ocspRelaxed = 0, ocspStrict };
-  enum OcspGetConfig { ocspGetDisabled = 0, ocspGetEnabled = 1 };
 
   enum class CertificateTransparencyMode {
     Disabled = 0,
     TelemetryOnly = 1,
   };
 
-  CertVerifier(OcspDownloadConfig odc, OcspStrictConfig osc, OcspGetConfig ogc,
+  CertVerifier(OcspDownloadConfig odc, OcspStrictConfig osc,
                mozilla::TimeDuration ocspTimeoutSoft,
                mozilla::TimeDuration ocspTimeoutHard,
-               uint32_t certShortLifetimeInDays,
-               PinningMode pinningMode, SHA1Mode sha1Mode,
+               uint32_t certShortLifetimeInDays, SHA1Mode sha1Mode,
                BRNameMatchingPolicy::Mode nameMatchingMode,
                NetscapeStepUpPolicy netscapeStepUpPolicy,
-               CertificateTransparencyMode ctMode,
-               DistrustedCAPolicy distrustedCAPolicy);
+               CertificateTransparencyMode ctMode, CRLiteMode crliteMode,
+               uint64_t crliteCTMergeDelaySeconds,
+               const Vector<EnterpriseCert>& thirdPartyCerts);
   ~CertVerifier();
 
   void ClearOCSPCache() { mOCSPCache.Clear(); }
 
   const OcspDownloadConfig mOCSPDownloadConfig;
   const bool mOCSPStrict;
-  const bool mOCSPGETEnabled;
   const mozilla::TimeDuration mOCSPTimeoutSoft;
   const mozilla::TimeDuration mOCSPTimeoutHard;
   const uint32_t mCertShortLifetimeInDays;
-  const PinningMode mPinningMode;
   const SHA1Mode mSHA1Mode;
   const BRNameMatchingPolicy::Mode mNameMatchingMode;
   const NetscapeStepUpPolicy mNetscapeStepUpPolicy;
   const CertificateTransparencyMode mCTMode;
-  const DistrustedCAPolicy mDistrustedCAPolicy;
+  const CRLiteMode mCRLiteMode;
+  const uint64_t mCRLiteCTMergeDelaySeconds;
 
-private:
+ private:
   OCSPCache mOCSPCache;
+  // We keep a copy of the bytes of each third party root to own.
+  Vector<EnterpriseCert> mThirdPartyCerts;
+  // This is a reusable, precomputed list of Inputs corresponding to each root
+  // in mThirdPartyCerts that wasn't too long to make an Input out of.
+  Vector<mozilla::pkix::Input> mThirdPartyRootInputs;
+  // Similarly, but with intermediates.
+  Vector<mozilla::pkix::Input> mThirdPartyIntermediateInputs;
 
   // We only have a forward declarations of these classes (see above)
   // so we must allocate dynamically.
@@ -230,11 +256,9 @@ private:
 
   void LoadKnownCTLogs();
   mozilla::pkix::Result VerifyCertificateTransparencyPolicy(
-                     NSSCertDBTrustDomain& trustDomain,
-                     const UniqueCERTCertList& builtChain,
-                     mozilla::pkix::Input sctsFromTLS,
-                     mozilla::pkix::Time time,
-    /*optional out*/ CertificateTransparencyInfo* ctInfo);
+      NSSCertDBTrustDomain& trustDomain, const UniqueCERTCertList& builtChain,
+      mozilla::pkix::Input sctsFromTLS, mozilla::pkix::Time time,
+      /*optional out*/ CertificateTransparencyInfo* ctInfo);
 
   // Returns true if the configured SHA1 mode is more restrictive than the given
   // mode. SHA1Mode::Forbidden is more restrictive than any other mode except
@@ -244,10 +268,11 @@ private:
 };
 
 mozilla::pkix::Result IsCertBuiltInRoot(CERTCertificate* cert, bool& result);
-mozilla::pkix::Result CertListContainsExpectedKeys(
-  const CERTCertList* certList, const char* hostname, mozilla::pkix::Time time,
-  CertVerifier::PinningMode pinningMode);
+mozilla::pkix::Result CertListContainsExpectedKeys(const CERTCertList* certList,
+                                                   const char* hostname,
+                                                   mozilla::pkix::Time time);
 
-} } // namespace mozilla::psm
+}  // namespace psm
+}  // namespace mozilla
 
-#endif // CertVerifier_h
+#endif  // CertVerifier_h
