@@ -36,8 +36,10 @@
 
 #include "mozilla/Array.h"
 #include "mozilla/Atomics.h"
+#include "mozilla/Attributes.h"
 #include "mozilla/Range.h"
 #include "mozilla/SandboxInfo.h"
+#include "mozilla/StackWalk.h"
 #include "mozilla/Span.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/Unused.h"
@@ -81,7 +83,7 @@ mozilla::Atomic<int> gSeccompTsyncBroadcastSignum(0);
 
 namespace mozilla {
 
-static bool gSandboxCrashOnError = false;
+static mozilla::Atomic<bool> gSandboxCrashOnError(false);
 
 // This is initialized by SandboxSetCrashFunc().
 SandboxCrashFunc gSandboxCrashFunc;
@@ -116,7 +118,8 @@ static bool ContextIsError(const ucontext_t* aContext, int aError) {
  * that it could be in async signal context (e.g., intercepting an
  * open() called from an async signal handler).
  */
-static void SigSysHandler(int nr, siginfo_t* info, void* void_context) {
+MOZ_NEVER_INLINE static void SigSysHandler(int nr, siginfo_t* info,
+                                           void* void_context) {
   ucontext_t* ctx = static_cast<ucontext_t*>(void_context);
   // This shouldn't ever be null, but the Chromium handler checks for
   // that and refrains from crashing, so let's not crash release builds:
@@ -149,7 +152,7 @@ static void SigSysHandler(int nr, siginfo_t* info, void* void_context) {
     // Bug 1017393: record syscall number somewhere useful.
     info->si_addr = reinterpret_cast<void*>(report.mSyscall);
 
-    gSandboxCrashFunc(nr, info, &savedCtx);
+    gSandboxCrashFunc(nr, info, &savedCtx, CallerPC());
     _exit(127);
   }
 }
@@ -212,8 +215,8 @@ static void InstallSigSysHandler(void) {
  * @see SandboxInfo
  * @see BroadcastSetThreadSandbox
  */
-static bool MOZ_MUST_USE InstallSyscallFilter(const sock_fprog* aProg,
-                                              bool aUseTSync) {
+[[nodiscard]] static bool InstallSyscallFilter(const sock_fprog* aProg,
+                                               bool aUseTSync) {
   if (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0)) {
     if (!aUseTSync && errno == ETXTBSY) {
       return false;
@@ -651,7 +654,7 @@ void SetMediaPluginSandbox(const char* aFilePath) {
 
   auto files = new SandboxOpenedFiles();
   files->Add(std::move(plugin));
-  files->Add("/dev/urandom", true);
+  files->Add("/dev/urandom", SandboxOpenedFile::Dup::YES);
   files->Add("/etc/ld.so.cache");  // Needed for NSS in clearkey.
   files->Add("/sys/devices/system/cpu/cpu0/tsc_freq_khz");
   files->Add("/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq");
@@ -660,6 +663,12 @@ void SetMediaPluginSandbox(const char* aFilePath) {
 #ifdef __i386__
   files->Add("/proc/self/auxv");  // Info also in process's address space.
 #endif
+  // Bug 1712506: the Widevine CDM will try to access these but
+  // doesn't appear to need them.
+  files->Add("/sys/devices/system/cpu/online", SandboxOpenedFile::Error{});
+  files->Add("/proc/stat", SandboxOpenedFile::Error{});
+  files->Add("/proc/net/unix", SandboxOpenedFile::Error{});
+  files->Add("/proc/self/maps", SandboxOpenedFile::Error{});
 
   // Finally, start the sandbox.
   SetCurrentProcessSandbox(GetMediaSandboxPolicy(files));
@@ -706,4 +715,10 @@ void SetSocketProcessSandbox(int aBroker) {
   SetCurrentProcessSandbox(GetSocketProcessSandboxPolicy(sBroker));
 }
 
+bool SetSandboxCrashOnError(bool aValue) {
+  bool oldValue = gSandboxCrashOnError;
+  gSandboxCrashOnError = aValue;
+  return oldValue;
+}
+
 }  // namespace mozilla
