@@ -6,13 +6,19 @@
 
 #include <string.h>
 #include "mozilla/Telemetry.h"
-#include "mozilla/Preferences.h"
 #include "sqlite3.h"
 #include "nsThreadUtils.h"
 #include "mozilla/dom/quota/PersistenceType.h"
 #include "mozilla/dom/quota/QuotaManager.h"
 #include "mozilla/dom/quota/QuotaObject.h"
+#include "mozilla/net/IOActivityMonitor.h"
 #include "mozilla/IOInterposer.h"
+#include "nsEscape.h"
+#include "mozilla/StaticPrefs_storage.h"
+
+#ifdef XP_WIN
+#  include "mozilla/StaticPrefs_dom.h"
+#endif
 
 // The last VFS version for which this file has been updated.
 #define LAST_KNOWN_VFS_VERSION 3
@@ -20,23 +26,14 @@
 // The last io_methods version for which this file has been updated.
 #define LAST_KNOWN_IOMETHODS_VERSION 3
 
-/**
- * This preference is a workaround to allow users/sysadmins to identify
- * that the profile exists on an NFS share whose implementation
- * is incompatible with SQLite's default locking implementation.
- * Bug 433129 attempted to automatically identify such file-systems,
- * but a reliable way was not found and it was determined that the fallback
- * locking is slower than POSIX locking, so we do not want to do it by default.
-*/
-#define PREF_NFS_FILESYSTEM   "storage.nfs_filesystem"
-
 namespace {
 
 using namespace mozilla;
 using namespace mozilla::dom::quota;
+using namespace mozilla::net;
 
 struct Histograms {
-  const char *name;
+  const char* name;
   const Telemetry::HistogramID readB;
   const Telemetry::HistogramID writeB;
   const Telemetry::HistogramID readMS;
@@ -44,27 +41,25 @@ struct Histograms {
   const Telemetry::HistogramID syncMS;
 };
 
-#define SQLITE_TELEMETRY(FILENAME, HGRAM) \
-  { FILENAME, \
-    Telemetry::MOZ_SQLITE_ ## HGRAM ## _READ_B, \
-    Telemetry::MOZ_SQLITE_ ## HGRAM ## _WRITE_B, \
-    Telemetry::MOZ_SQLITE_ ## HGRAM ## _READ_MS, \
-    Telemetry::MOZ_SQLITE_ ## HGRAM ## _WRITE_MS, \
-    Telemetry::MOZ_SQLITE_ ## HGRAM ## _SYNC_MS \
+#define SQLITE_TELEMETRY(FILENAME, HGRAM)             \
+  {                                                   \
+    FILENAME, Telemetry::MOZ_SQLITE_##HGRAM##_READ_B, \
+        Telemetry::MOZ_SQLITE_##HGRAM##_WRITE_B,      \
+        Telemetry::MOZ_SQLITE_##HGRAM##_READ_MS,      \
+        Telemetry::MOZ_SQLITE_##HGRAM##_WRITE_MS,     \
+        Telemetry::MOZ_SQLITE_##HGRAM##_SYNC_MS       \
   }
 
-Histograms gHistograms[] = {
-  SQLITE_TELEMETRY("places.sqlite", PLACES),
-  SQLITE_TELEMETRY("cookies.sqlite", COOKIES),
-  SQLITE_TELEMETRY("webappsstore.sqlite", WEBAPPS),
-  SQLITE_TELEMETRY(nullptr, OTHER)
-};
+Histograms gHistograms[] = {SQLITE_TELEMETRY("places.sqlite", PLACES),
+                            SQLITE_TELEMETRY("cookies.sqlite", COOKIES),
+                            SQLITE_TELEMETRY("webappsstore.sqlite", WEBAPPS),
+                            SQLITE_TELEMETRY(nullptr, OTHER)};
 #undef SQLITE_TELEMETRY
 
 /** RAII class for measuring how long io takes on/off main thread
  */
 class IOThreadAutoTimer {
-public:
+ public:
   /**
    * IOThreadAutoTimer measures time spent in IO. Additionally it
    * automatically determines whether IO is happening on the main
@@ -78,12 +73,14 @@ public:
    * IOInterposer. Filename will be reported as NULL, and reference will be
    * either "sqlite-mainthread" or "sqlite-otherthread".
    */
-  explicit IOThreadAutoTimer(Telemetry::HistogramID aId,
-    IOInterposeObserver::Operation aOp = IOInterposeObserver::OpNone)
-    : start(TimeStamp::Now()),
-      id(aId)
+  explicit IOThreadAutoTimer(
+      Telemetry::HistogramID aId,
+      IOInterposeObserver::Operation aOp = IOInterposeObserver::OpNone)
+      : start(TimeStamp::Now()),
+        id(aId)
 #if defined(MOZ_GECKO_PROFILER) && !defined(XP_WIN)
-      , op(aOp)
+        ,
+        op(aOp)
 #endif
   {
   }
@@ -95,28 +92,28 @@ public:
    * @param aOp IO Operation to report through the IOInterposer.
    */
   explicit IOThreadAutoTimer(IOInterposeObserver::Operation aOp)
-    : start(TimeStamp::Now()),
-      id(Telemetry::HistogramCount)
+      : start(TimeStamp::Now()),
+        id(Telemetry::HistogramCount)
 #if defined(MOZ_GECKO_PROFILER) && !defined(XP_WIN)
-      , op(aOp)
+        ,
+        op(aOp)
 #endif
   {
   }
 
-  ~IOThreadAutoTimer()
-  {
+  ~IOThreadAutoTimer() {
     TimeStamp end(TimeStamp::Now());
     uint32_t mainThread = NS_IsMainThread() ? 1 : 0;
     if (id != Telemetry::HistogramCount) {
-      Telemetry::AccumulateTimeDelta(static_cast<Telemetry::HistogramID>(id + mainThread),
-                                     start, end);
+      Telemetry::AccumulateTimeDelta(
+          static_cast<Telemetry::HistogramID>(id + mainThread), start, end);
     }
     // We don't report SQLite I/O on Windows because we have a comprehensive
     // mechanism for intercepting I/O on that platform that captures a superset
     // of the data captured here.
 #if defined(MOZ_GECKO_PROFILER) && !defined(XP_WIN)
     if (IOInterposer::IsObservedOperation(op)) {
-      const char* main_ref  = "sqlite-mainthread";
+      const char* main_ref = "sqlite-mainthread";
       const char* other_ref = "sqlite-otherthread";
 
       // Create observation
@@ -128,7 +125,7 @@ public:
 #endif /* defined(MOZ_GECKO_PROFILER) && !defined(XP_WIN) */
   }
 
-private:
+ private:
   const TimeStamp start;
   const Telemetry::HistogramID id;
 #if defined(MOZ_GECKO_PROFILER) && !defined(XP_WIN)
@@ -141,7 +138,7 @@ struct telemetry_file {
   sqlite3_file base;
 
   // histograms pertaining to this file
-  Histograms *histograms;
+  Histograms* histograms;
 
   // quota object for this file
   RefPtr<QuotaObject> quotaObject;
@@ -150,217 +147,60 @@ struct telemetry_file {
   // sqlite3_file_control() and FCNTL_CHUNK_SIZE.
   int fileChunkSize;
 
+  // The filename
+  char* location;
+
   // This contains the vfs that actually does work
   sqlite3_file pReal[1];
 };
 
-const char*
-DatabasePathFromWALPath(const char *zWALName)
-{
-  /**
-   * Do some sketchy pointer arithmetic to find the parameter key. The WAL
-   * filename is in the middle of a big allocated block that contains:
-   *
-   *   - Random Values
-   *   - Main Database Path
-   *   - \0
-   *   - Multiple URI components consisting of:
-   *     - Key
-   *     - \0
-   *     - Value
-   *     - \0
-   *   - \0
-   *   - Journal Path
-   *   - \0
-   *   - WAL Path (zWALName)
-   *   - \0
-   *
-   * Because the main database path is preceded by a random value we have to be
-   * careful when trying to figure out when we should terminate this loop.
-   */
-  MOZ_ASSERT(zWALName);
-
-  nsDependentCSubstring dbPath(zWALName, strlen(zWALName));
-
-  // Chop off the "-wal" suffix.
-  NS_NAMED_LITERAL_CSTRING(kWALSuffix, "-wal");
-  MOZ_ASSERT(StringEndsWith(dbPath, kWALSuffix));
-
-  dbPath.Rebind(zWALName, dbPath.Length() - kWALSuffix.Length());
-  MOZ_ASSERT(!dbPath.IsEmpty());
-
-  // We want to scan to the end of the key/value URI pairs. Skip the preceding
-  // null and go to the last char of the journal path.
-  const char* cursor = zWALName - 2;
-
-  // Make sure we just skipped a null.
-  MOZ_ASSERT(!*(cursor + 1));
-
-  // Walk backwards over the journal path.
-  while (*cursor) {
-    cursor--;
-  }
-
-  // There should be another null here.
-  cursor--;
-  MOZ_ASSERT(!*cursor);
-
-  // Back up one more char to the last char of the previous string. It may be
-  // the database path or it may be a key/value URI pair.
-  cursor--;
-
-#ifdef DEBUG
-  {
-    // Verify that we just walked over the journal path. Account for the two
-    // nulls we just skipped.
-    const char *journalStart = cursor + 3;
-
-    nsDependentCSubstring journalPath(journalStart,
-                                      strlen(journalStart));
-
-    // Chop off the "-journal" suffix.
-    NS_NAMED_LITERAL_CSTRING(kJournalSuffix, "-journal");
-    MOZ_ASSERT(StringEndsWith(journalPath, kJournalSuffix));
-
-    journalPath.Rebind(journalStart,
-                        journalPath.Length() - kJournalSuffix.Length());
-    MOZ_ASSERT(!journalPath.IsEmpty());
-
-    // Make sure that the database name is a substring of the journal name.
-    MOZ_ASSERT(journalPath == dbPath);
-  }
-#endif
-
-  // Now we're either at the end of the key/value URI pairs or we're at the
-  // end of the database path. Carefully walk backwards one character at a
-  // time to do this safely without running past the beginning of the database
-  // path.
-  const char *const dbPathStart = dbPath.BeginReading();
-  const char *dbPathCursor = dbPath.EndReading() - 1;
-  bool isDBPath = true;
-
-  while (true) {
-    MOZ_ASSERT(*dbPathCursor, "dbPathCursor should never see a null char!");
-
-    if (isDBPath) {
-      isDBPath = dbPathStart <= dbPathCursor &&
-                 *dbPathCursor == *cursor &&
-                 *cursor;
-    }
-
-    if (!isDBPath) {
-      // This isn't the database path so it must be a value. Scan past it and
-      // the key also.
-      for (size_t stringCount = 0; stringCount < 2; stringCount++) {
-        // Scan past the string to the preceding null character.
-        while (*cursor) {
-          cursor--;
-        }
-
-        // Back up one more char to the last char of preceding string.
-        cursor--;
-      }
-
-      // Reset and start again.
-      dbPathCursor = dbPath.EndReading() - 1;
-      isDBPath = true;
-
-      continue;
-    }
-
-    MOZ_ASSERT(isDBPath);
-    MOZ_ASSERT(*cursor);
-
-    if (dbPathStart == dbPathCursor) {
-      // Found the full database path, we're all done.
-      MOZ_ASSERT(nsDependentCString(cursor) == dbPath);
-      return cursor;
-    }
-
-    // Change the cursors and go through the loop again.
-    cursor--;
-    dbPathCursor--;
-  }
-
-  MOZ_CRASH("Should never get here!");
-}
-
-already_AddRefed<QuotaObject>
-GetQuotaObjectFromNameAndParameters(const char *zName,
-                                    const char *zURIParameterKey)
-{
+already_AddRefed<QuotaObject> GetQuotaObjectFromName(const char* zName) {
   MOZ_ASSERT(zName);
-  MOZ_ASSERT(zURIParameterKey);
 
-  const char *persistenceType =
-    sqlite3_uri_parameter(zURIParameterKey, "persistenceType");
-  if (!persistenceType) {
+  const char* directoryLockIdParam =
+      sqlite3_uri_parameter(zName, "directoryLockId");
+  if (!directoryLockIdParam) {
     return nullptr;
   }
 
-  const char *group = sqlite3_uri_parameter(zURIParameterKey, "group");
-  if (!group) {
-    NS_WARNING("SQLite URI had 'persistenceType' but not 'group'?!");
-    return nullptr;
-  }
+  nsresult rv;
+  const int64_t directoryLockId =
+      nsDependentCString(directoryLockIdParam).ToInteger64(&rv);
+  MOZ_RELEASE_ASSERT(NS_SUCCEEDED(rv));
 
-  const char *origin = sqlite3_uri_parameter(zURIParameterKey, "origin");
-  if (!origin) {
-    NS_WARNING("SQLite URI had 'persistenceType' and 'group' but not "
-               "'origin'?!");
-    return nullptr;
-  }
-
-  QuotaManager *quotaManager = QuotaManager::Get();
+  QuotaManager* quotaManager = QuotaManager::Get();
   MOZ_ASSERT(quotaManager);
 
-  return quotaManager->GetQuotaObject(
-    PersistenceTypeFromText(nsDependentCString(persistenceType)),
-    nsDependentCString(group),
-    nsDependentCString(origin),
-    NS_ConvertUTF8toUTF16(zName));
+  return quotaManager->GetQuotaObject(directoryLockId,
+                                      NS_ConvertUTF8toUTF16(zName));
 }
 
-void
-MaybeEstablishQuotaControl(const char *zName,
-                           telemetry_file *pFile,
-                           int flags)
-{
+void MaybeEstablishQuotaControl(const char* zName, telemetry_file* pFile,
+                                int flags) {
   MOZ_ASSERT(pFile);
   MOZ_ASSERT(!pFile->quotaObject);
 
   if (!(flags & (SQLITE_OPEN_URI | SQLITE_OPEN_WAL))) {
     return;
   }
-
-  MOZ_ASSERT(zName);
-
-  const char *zURIParameterKey = (flags & SQLITE_OPEN_WAL) ?
-                                 DatabasePathFromWALPath(zName) :
-                                 zName;
-
-  MOZ_ASSERT(zURIParameterKey);
-
-  pFile->quotaObject =
-    GetQuotaObjectFromNameAndParameters(zName, zURIParameterKey);
+  pFile->quotaObject = GetQuotaObjectFromName(zName);
 }
 
 /*
 ** Close a telemetry_file.
 */
-int
-xClose(sqlite3_file *pFile)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xClose(sqlite3_file* pFile) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
-  { // Scope for IOThreadAutoTimer
+  {  // Scope for IOThreadAutoTimer
     IOThreadAutoTimer ioTimer(IOInterposeObserver::OpClose);
     rc = p->pReal->pMethods->xClose(p->pReal);
   }
-  if( rc==SQLITE_OK ){
+  if (rc == SQLITE_OK) {
     delete p->base.pMethods;
     p->base.pMethods = nullptr;
     p->quotaObject = nullptr;
+    delete[] p->location;
 #ifdef DEBUG
     p->fileChunkSize = 0;
 #endif
@@ -371,13 +211,14 @@ xClose(sqlite3_file *pFile)
 /*
 ** Read data from a telemetry_file.
 */
-int
-xRead(sqlite3_file *pFile, void *zBuf, int iAmt, sqlite_int64 iOfst)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xRead(sqlite3_file* pFile, void* zBuf, int iAmt, sqlite_int64 iOfst) {
+  telemetry_file* p = (telemetry_file*)pFile;
   IOThreadAutoTimer ioTimer(p->histograms->readMS, IOInterposeObserver::OpRead);
   int rc;
   rc = p->pReal->pMethods->xRead(p->pReal, zBuf, iAmt, iOfst);
+  if (rc == SQLITE_OK && IOActivityMonitor::IsActive()) {
+    IOActivityMonitor::Read(nsDependentCString(p->location), iAmt);
+  }
   // sqlite likes to read from empty files, this is normal, ignore it.
   if (rc != SQLITE_IOERR_SHORT_READ)
     Telemetry::Accumulate(p->histograms->readB, rc == SQLITE_OK ? iAmt : 0);
@@ -387,11 +228,9 @@ xRead(sqlite3_file *pFile, void *zBuf, int iAmt, sqlite_int64 iOfst)
 /*
 ** Return the current file-size of a telemetry_file.
 */
-int
-xFileSize(sqlite3_file *pFile, sqlite_int64 *pSize)
-{
+int xFileSize(sqlite3_file* pFile, sqlite_int64* pSize) {
   IOThreadAutoTimer ioTimer(IOInterposeObserver::OpStat);
-  telemetry_file *p = (telemetry_file *)pFile;
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   rc = p->pReal->pMethods->xFileSize(p->pReal, pSize);
   return rc;
@@ -400,11 +239,11 @@ xFileSize(sqlite3_file *pFile, sqlite_int64 *pSize)
 /*
 ** Write data to a telemetry_file.
 */
-int
-xWrite(sqlite3_file *pFile, const void *zBuf, int iAmt, sqlite_int64 iOfst)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
-  IOThreadAutoTimer ioTimer(p->histograms->writeMS, IOInterposeObserver::OpWrite);
+int xWrite(sqlite3_file* pFile, const void* zBuf, int iAmt,
+           sqlite_int64 iOfst) {
+  telemetry_file* p = (telemetry_file*)pFile;
+  IOThreadAutoTimer ioTimer(p->histograms->writeMS,
+                            IOInterposeObserver::OpWrite);
   int rc;
   if (p->quotaObject) {
     MOZ_ASSERT(INT64_MAX - iOfst >= iAmt);
@@ -413,13 +252,20 @@ xWrite(sqlite3_file *pFile, const void *zBuf, int iAmt, sqlite_int64 iOfst)
     }
   }
   rc = p->pReal->pMethods->xWrite(p->pReal, zBuf, iAmt, iOfst);
+  if (rc == SQLITE_OK && IOActivityMonitor::IsActive()) {
+    IOActivityMonitor::Write(nsDependentCString(p->location), iAmt);
+  }
+
   Telemetry::Accumulate(p->histograms->writeB, rc == SQLITE_OK ? iAmt : 0);
   if (p->quotaObject && rc != SQLITE_OK) {
-    NS_WARNING("xWrite failed on a quota-controlled file, attempting to "
-               "update its current size...");
+    NS_WARNING(
+        "xWrite failed on a quota-controlled file, attempting to "
+        "update its current size...");
     sqlite_int64 currentSize;
     if (xFileSize(pFile, &currentSize) == SQLITE_OK) {
-      p->quotaObject->MaybeUpdateSize(currentSize, /* aTruncate */ true);
+      DebugOnly<bool> res =
+          p->quotaObject->MaybeUpdateSize(currentSize, /* aTruncate */ true);
+      MOZ_ASSERT(res);
     }
   }
   return rc;
@@ -428,11 +274,9 @@ xWrite(sqlite3_file *pFile, const void *zBuf, int iAmt, sqlite_int64 iOfst)
 /*
 ** Truncate a telemetry_file.
 */
-int
-xTruncate(sqlite3_file *pFile, sqlite_int64 size)
-{
+int xTruncate(sqlite3_file* pFile, sqlite_int64 size) {
   IOThreadAutoTimer ioTimer(Telemetry::MOZ_SQLITE_TRUNCATE_MS);
-  telemetry_file *p = (telemetry_file *)pFile;
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   Telemetry::AutoTimer<Telemetry::MOZ_SQLITE_TRUNCATE_MS> timer;
   if (p->quotaObject) {
@@ -440,7 +284,7 @@ xTruncate(sqlite3_file *pFile, sqlite_int64 size)
       // Round up to the smallest multiple of the chunk size that will hold all
       // the data.
       size =
-        ((size + p->fileChunkSize - 1) / p->fileChunkSize) * p->fileChunkSize;
+          ((size + p->fileChunkSize - 1) / p->fileChunkSize) * p->fileChunkSize;
     }
     if (!p->quotaObject->MaybeUpdateSize(size, /* aTruncate */ true)) {
       return SQLITE_FULL;
@@ -456,10 +300,13 @@ xTruncate(sqlite3_file *pFile, sqlite_int64 size)
       MOZ_ASSERT(newSize == size);
 #endif
     } else {
-      NS_WARNING("xTruncate failed on a quota-controlled file, attempting to "
-                 "update its current size...");
+      NS_WARNING(
+          "xTruncate failed on a quota-controlled file, attempting to "
+          "update its current size...");
       if (xFileSize(pFile, &size) == SQLITE_OK) {
-        p->quotaObject->MaybeUpdateSize(size, /* aTruncate */ true);
+        DebugOnly<bool> res =
+            p->quotaObject->MaybeUpdateSize(size, /* aTruncate */ true);
+        MOZ_ASSERT(res);
       }
     }
   }
@@ -469,21 +316,18 @@ xTruncate(sqlite3_file *pFile, sqlite_int64 size)
 /*
 ** Sync a telemetry_file.
 */
-int
-xSync(sqlite3_file *pFile, int flags)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
-  IOThreadAutoTimer ioTimer(p->histograms->syncMS, IOInterposeObserver::OpFSync);
+int xSync(sqlite3_file* pFile, int flags) {
+  telemetry_file* p = (telemetry_file*)pFile;
+  IOThreadAutoTimer ioTimer(p->histograms->syncMS,
+                            IOInterposeObserver::OpFSync);
   return p->pReal->pMethods->xSync(p->pReal, flags);
 }
 
 /*
 ** Lock a telemetry_file.
 */
-int
-xLock(sqlite3_file *pFile, int eLock)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xLock(sqlite3_file* pFile, int eLock) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   rc = p->pReal->pMethods->xLock(p->pReal, eLock);
   return rc;
@@ -492,10 +336,8 @@ xLock(sqlite3_file *pFile, int eLock)
 /*
 ** Unlock a telemetry_file.
 */
-int
-xUnlock(sqlite3_file *pFile, int eLock)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xUnlock(sqlite3_file* pFile, int eLock) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   rc = p->pReal->pMethods->xUnlock(p->pReal, eLock);
   return rc;
@@ -504,10 +346,8 @@ xUnlock(sqlite3_file *pFile, int eLock)
 /*
 ** Check if another file-handle holds a RESERVED lock on a telemetry_file.
 */
-int
-xCheckReservedLock(sqlite3_file *pFile, int *pResOut)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xCheckReservedLock(sqlite3_file* pFile, int* pResOut) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc = p->pReal->pMethods->xCheckReservedLock(p->pReal, pResOut);
   return rc;
 }
@@ -515,10 +355,8 @@ xCheckReservedLock(sqlite3_file *pFile, int *pResOut)
 /*
 ** File control method. For custom operations on a telemetry_file.
 */
-int
-xFileControl(sqlite3_file *pFile, int op, void *pArg)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xFileControl(sqlite3_file* pFile, int op, void* pArg) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   // Hook SQLITE_FCNTL_SIZE_HINT for quota-controlled files and do the necessary
   // work before passing to the SQLite VFS.
@@ -545,9 +383,8 @@ xFileControl(sqlite3_file *pFile, int op, void *pArg)
   if (op == SQLITE_FCNTL_SIZE_HINT && p->quotaObject && rc == SQLITE_OK) {
     sqlite3_int64 hintSize = *static_cast<sqlite3_int64*>(pArg);
     if (p->fileChunkSize > 0) {
-      hintSize =
-        ((hintSize + p->fileChunkSize - 1) / p->fileChunkSize) *
-          p->fileChunkSize;
+      hintSize = ((hintSize + p->fileChunkSize - 1) / p->fileChunkSize) *
+                 p->fileChunkSize;
     }
     sqlite3_int64 currentSize;
     MOZ_ASSERT(xFileSize(pFile, &currentSize) == SQLITE_OK);
@@ -560,10 +397,8 @@ xFileControl(sqlite3_file *pFile, int op, void *pArg)
 /*
 ** Return the sector-size in bytes for a telemetry_file.
 */
-int
-xSectorSize(sqlite3_file *pFile)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xSectorSize(sqlite3_file* pFile) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   rc = p->pReal->pMethods->xSectorSize(p->pReal);
   return rc;
@@ -572,10 +407,8 @@ xSectorSize(sqlite3_file *pFile)
 /*
 ** Return the device characteristic flags supported by a telemetry_file.
 */
-int
-xDeviceCharacteristics(sqlite3_file *pFile)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xDeviceCharacteristics(sqlite3_file* pFile) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   rc = p->pReal->pMethods->xDeviceCharacteristics(p->pReal);
   return rc;
@@ -584,89 +417,83 @@ xDeviceCharacteristics(sqlite3_file *pFile)
 /*
 ** Shared-memory operations.
 */
-int
-xShmLock(sqlite3_file *pFile, int ofst, int n, int flags)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xShmLock(sqlite3_file* pFile, int ofst, int n, int flags) {
+  telemetry_file* p = (telemetry_file*)pFile;
   return p->pReal->pMethods->xShmLock(p->pReal, ofst, n, flags);
 }
 
-int
-xShmMap(sqlite3_file *pFile, int iRegion, int szRegion, int isWrite, void volatile **pp)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xShmMap(sqlite3_file* pFile, int iRegion, int szRegion, int isWrite,
+            void volatile** pp) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   rc = p->pReal->pMethods->xShmMap(p->pReal, iRegion, szRegion, isWrite, pp);
   return rc;
 }
 
-void
-xShmBarrier(sqlite3_file *pFile){
-  telemetry_file *p = (telemetry_file *)pFile;
+void xShmBarrier(sqlite3_file* pFile) {
+  telemetry_file* p = (telemetry_file*)pFile;
   p->pReal->pMethods->xShmBarrier(p->pReal);
 }
 
-int
-xShmUnmap(sqlite3_file *pFile, int delFlag){
-  telemetry_file *p = (telemetry_file *)pFile;
+int xShmUnmap(sqlite3_file* pFile, int delFlag) {
+  telemetry_file* p = (telemetry_file*)pFile;
   int rc;
   rc = p->pReal->pMethods->xShmUnmap(p->pReal, delFlag);
   return rc;
 }
 
-int
-xFetch(sqlite3_file *pFile, sqlite3_int64 iOff, int iAmt, void **pp)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xFetch(sqlite3_file* pFile, sqlite3_int64 iOff, int iAmt, void** pp) {
+  telemetry_file* p = (telemetry_file*)pFile;
   MOZ_ASSERT(p->pReal->pMethods->iVersion >= 3);
   return p->pReal->pMethods->xFetch(p->pReal, iOff, iAmt, pp);
 }
 
-int
-xUnfetch(sqlite3_file *pFile, sqlite3_int64 iOff, void *pResOut)
-{
-  telemetry_file *p = (telemetry_file *)pFile;
+int xUnfetch(sqlite3_file* pFile, sqlite3_int64 iOff, void* pResOut) {
+  telemetry_file* p = (telemetry_file*)pFile;
   MOZ_ASSERT(p->pReal->pMethods->iVersion >= 3);
   return p->pReal->pMethods->xUnfetch(p->pReal, iOff, pResOut);
 }
 
-int
-xOpen(sqlite3_vfs* vfs, const char *zName, sqlite3_file* pFile,
-          int flags, int *pOutFlags)
-{
+int xOpen(sqlite3_vfs* vfs, const char* zName, sqlite3_file* pFile, int flags,
+          int* pOutFlags) {
   IOThreadAutoTimer ioTimer(Telemetry::MOZ_SQLITE_OPEN_MS,
                             IOInterposeObserver::OpCreateOrOpen);
   Telemetry::AutoTimer<Telemetry::MOZ_SQLITE_OPEN_MS> timer;
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   int rc;
-  telemetry_file *p = (telemetry_file *)pFile;
-  Histograms *h = nullptr;
+  telemetry_file* p = (telemetry_file*)pFile;
+  Histograms* h = nullptr;
   // check if the filename is one we are probing for
-  for(size_t i = 0;i < sizeof(gHistograms)/sizeof(gHistograms[0]);i++) {
+  for (size_t i = 0; i < sizeof(gHistograms) / sizeof(gHistograms[0]); i++) {
     h = &gHistograms[i];
     // last probe is the fallback probe
-    if (!h->name)
-      break;
-    if (!zName)
-      continue;
-    const char *match = strstr(zName, h->name);
-    if (!match)
-      continue;
+    if (!h->name) break;
+    if (!zName) continue;
+    const char* match = strstr(zName, h->name);
+    if (!match) continue;
     char c = match[strlen(h->name)];
     // include -wal/-journal too
-    if (!c || c == '-')
-      break;
+    if (!c || c == '-') break;
   }
   p->histograms = h;
 
   MaybeEstablishQuotaControl(zName, p, flags);
 
   rc = orig_vfs->xOpen(orig_vfs, zName, p->pReal, flags, pOutFlags);
-  if( rc != SQLITE_OK )
-    return rc;
-  if( p->pReal->pMethods ){
-    sqlite3_io_methods *pNew = new sqlite3_io_methods;
-    const sqlite3_io_methods *pSub = p->pReal->pMethods;
+  if (rc != SQLITE_OK) return rc;
+
+  if (zName) {
+    p->location = new char[7 + strlen(zName) + 1];
+    strcpy(p->location, "file://");
+    strcpy(p->location + 7, zName);
+  } else {
+    p->location = new char[8];
+    strcpy(p->location, "file://");
+  }
+
+  if (p->pReal->pMethods) {
+    sqlite3_io_methods* pNew = new sqlite3_io_methods;
+    const sqlite3_io_methods* pSub = p->pReal->pMethods;
     memset(pNew, 0, sizeof(*pNew));
     // If the io_methods version is higher than the last known one, you should
     // update this VFS adding appropriate IO methods for any methods added in
@@ -707,18 +534,13 @@ xOpen(sqlite3_vfs* vfs, const char *zName, sqlite3_file* pFile,
   return rc;
 }
 
-int
-xDelete(sqlite3_vfs* vfs, const char *zName, int syncDir)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+int xDelete(sqlite3_vfs* vfs, const char* zName, int syncDir) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   int rc;
   RefPtr<QuotaObject> quotaObject;
 
-  if (StringEndsWith(nsDependentCString(zName), NS_LITERAL_CSTRING("-wal"))) {
-    const char *zURIParameterKey = DatabasePathFromWALPath(zName);
-    MOZ_ASSERT(zURIParameterKey);
-
-    quotaObject = GetQuotaObjectFromNameAndParameters(zName, zURIParameterKey);
+  if (StringEndsWith(nsDependentCString(zName), "-wal"_ns)) {
+    quotaObject = GetQuotaObjectFromName(zName);
   }
 
   rc = orig_vfs->xDelete(orig_vfs, zName, syncDir);
@@ -729,150 +551,157 @@ xDelete(sqlite3_vfs* vfs, const char *zName, int syncDir)
   return rc;
 }
 
-int
-xAccess(sqlite3_vfs *vfs, const char *zName, int flags, int *pResOut)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+int xAccess(sqlite3_vfs* vfs, const char* zName, int flags, int* pResOut) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xAccess(orig_vfs, zName, flags, pResOut);
 }
 
-int
-xFullPathname(sqlite3_vfs *vfs, const char *zName, int nOut, char *zOut)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+int xFullPathname(sqlite3_vfs* vfs, const char* zName, int nOut, char* zOut) {
+#if defined(XP_WIN)
+  // SQLite uses GetFullPathnameW which also normailizes file path. If a file
+  // component ends with a dot, it would be removed. However, it's not desired.
+  //
+  // And that would result SQLite uses wrong database and quotaObject.
+  // Note that we are safe to avoid the GetFullPathnameW call for \\?\ prefixed
+  // paths.
+  // And note that this hack will be removed once the issue is fixed directly in
+  // SQLite.
+
+  // zName that starts with "//?/" is the case when a file URI was passed and
+  // zName that starts with "\\?\" is the case when a normal path was passed
+  // (not file URI).
+  if (StaticPrefs::dom_quotaManager_overrideXFullPathname() &&
+      ((zName[0] == '/' && zName[1] == '/' && zName[2] == '?' &&
+        zName[3] == '/') ||
+       (zName[0] == '\\' && zName[1] == '\\' && zName[2] == '?' &&
+        zName[3] == '\\'))) {
+    MOZ_ASSERT(nOut >= vfs->mxPathname);
+    MOZ_ASSERT(nOut > strlen(zName));
+
+    size_t index = 0;
+    while (zName[index] != '\0') {
+      if (zName[index] == '/') {
+        zOut[index] = '\\';
+      } else {
+        zOut[index] = zName[index];
+      }
+
+      index++;
+    }
+    zOut[index] = '\0';
+
+    return SQLITE_OK;
+  }
+#endif
+
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xFullPathname(orig_vfs, zName, nOut, zOut);
 }
 
-void*
-xDlOpen(sqlite3_vfs *vfs, const char *zFilename)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+void* xDlOpen(sqlite3_vfs* vfs, const char* zFilename) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xDlOpen(orig_vfs, zFilename);
 }
 
-void
-xDlError(sqlite3_vfs *vfs, int nByte, char *zErrMsg)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+void xDlError(sqlite3_vfs* vfs, int nByte, char* zErrMsg) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   orig_vfs->xDlError(orig_vfs, nByte, zErrMsg);
 }
 
-void
-(*xDlSym(sqlite3_vfs *vfs, void *pHdle, const char *zSym))(void){
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+void (*xDlSym(sqlite3_vfs* vfs, void* pHdle, const char* zSym))(void) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xDlSym(orig_vfs, pHdle, zSym);
 }
 
-void
-xDlClose(sqlite3_vfs *vfs, void *pHandle)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+void xDlClose(sqlite3_vfs* vfs, void* pHandle) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   orig_vfs->xDlClose(orig_vfs, pHandle);
 }
 
-int
-xRandomness(sqlite3_vfs *vfs, int nByte, char *zOut)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+int xRandomness(sqlite3_vfs* vfs, int nByte, char* zOut) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xRandomness(orig_vfs, nByte, zOut);
 }
 
-int
-xSleep(sqlite3_vfs *vfs, int microseconds)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+int xSleep(sqlite3_vfs* vfs, int microseconds) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xSleep(orig_vfs, microseconds);
 }
 
-int
-xCurrentTime(sqlite3_vfs *vfs, double *prNow)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+int xCurrentTime(sqlite3_vfs* vfs, double* prNow) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xCurrentTime(orig_vfs, prNow);
 }
 
-int
-xGetLastError(sqlite3_vfs *vfs, int nBuf, char *zBuf)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+int xGetLastError(sqlite3_vfs* vfs, int nBuf, char* zBuf) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xGetLastError(orig_vfs, nBuf, zBuf);
 }
 
-int
-xCurrentTimeInt64(sqlite3_vfs *vfs, sqlite3_int64 *piNow)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+int xCurrentTimeInt64(sqlite3_vfs* vfs, sqlite3_int64* piNow) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xCurrentTimeInt64(orig_vfs, piNow);
 }
 
-static
-int
-xSetSystemCall(sqlite3_vfs *vfs, const char *zName, sqlite3_syscall_ptr pFunc)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+static int xSetSystemCall(sqlite3_vfs* vfs, const char* zName,
+                          sqlite3_syscall_ptr pFunc) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xSetSystemCall(orig_vfs, zName, pFunc);
 }
 
-static
-sqlite3_syscall_ptr
-xGetSystemCall(sqlite3_vfs *vfs, const char *zName)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+static sqlite3_syscall_ptr xGetSystemCall(sqlite3_vfs* vfs, const char* zName) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xGetSystemCall(orig_vfs, zName);
 }
 
-static
-const char *
-xNextSystemCall(sqlite3_vfs *vfs, const char *zName)
-{
-  sqlite3_vfs *orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
+static const char* xNextSystemCall(sqlite3_vfs* vfs, const char* zName) {
+  sqlite3_vfs* orig_vfs = static_cast<sqlite3_vfs*>(vfs->pAppData);
   return orig_vfs->xNextSystemCall(orig_vfs, zName);
 }
 
-} // namespace
+}  // namespace
 
 namespace mozilla {
 namespace storage {
 
-const char *GetVFSName()
-{
-  return "telemetry-vfs";
+const char* GetTelemetryVFSName(bool exclusive) {
+  return exclusive ? "telemetry-vfs-excl" : "telemetry-vfs";
 }
 
-sqlite3_vfs* ConstructTelemetryVFS()
-{
+UniquePtr<sqlite3_vfs> ConstructTelemetryVFS(bool exclusive) {
 #if defined(XP_WIN)
-#define EXPECTED_VFS     "win32"
-#define EXPECTED_VFS_NFS "win32"
+#  define EXPECTED_VFS "win32"
+#  define EXPECTED_VFS_EXCL "win32"
 #else
-#define EXPECTED_VFS     "unix"
-#define EXPECTED_VFS_NFS "unix-excl"
+#  define EXPECTED_VFS "unix"
+#  define EXPECTED_VFS_EXCL "unix-excl"
 #endif
 
   bool expected_vfs;
-  sqlite3_vfs *vfs;
-  if (Preferences::GetBool(PREF_NFS_FILESYSTEM)) {
-    vfs = sqlite3_vfs_find(EXPECTED_VFS_NFS);
-    expected_vfs = (vfs != nullptr);
-  }
-  else {
+  sqlite3_vfs* vfs;
+  if (!exclusive) {
+    // Use the non-exclusive VFS.
     vfs = sqlite3_vfs_find(nullptr);
     expected_vfs = vfs->zName && !strcmp(vfs->zName, EXPECTED_VFS);
+  } else {
+    vfs = sqlite3_vfs_find(EXPECTED_VFS_EXCL);
+    expected_vfs = (vfs != nullptr);
   }
   if (!expected_vfs) {
     return nullptr;
   }
 
-  sqlite3_vfs *tvfs = new ::sqlite3_vfs;
-  memset(tvfs, 0, sizeof(::sqlite3_vfs));
+  auto tvfs = MakeUnique<::sqlite3_vfs>();
+  memset(tvfs.get(), 0, sizeof(::sqlite3_vfs));
   // If the VFS version is higher than the last known one, you should update
   // this VFS adding appropriate methods for any methods added in the version
   // change.
   tvfs->iVersion = vfs->iVersion;
   MOZ_ASSERT(vfs->iVersion <= LAST_KNOWN_VFS_VERSION);
-  tvfs->szOsFile = sizeof(telemetry_file) - sizeof(sqlite3_file) + vfs->szOsFile;
+  tvfs->szOsFile =
+      sizeof(telemetry_file) - sizeof(sqlite3_file) + vfs->szOsFile;
   tvfs->mxPathname = vfs->mxPathname;
-  tvfs->zName = GetVFSName();
+  tvfs->zName = GetTelemetryVFSName(exclusive);
   tvfs->pAppData = vfs;
   tvfs->xOpen = xOpen;
   tvfs->xDelete = xDelete;
@@ -899,15 +728,13 @@ sqlite3_vfs* ConstructTelemetryVFS()
   return tvfs;
 }
 
-already_AddRefed<QuotaObject>
-GetQuotaObjectForFile(sqlite3_file *pFile)
-{
+already_AddRefed<QuotaObject> GetQuotaObjectForFile(sqlite3_file* pFile) {
   MOZ_ASSERT(pFile);
 
-  telemetry_file *p = (telemetry_file *)pFile;
+  telemetry_file* p = (telemetry_file*)pFile;
   RefPtr<QuotaObject> result = p->quotaObject;
   return result.forget();
 }
 
-} // namespace storage
-} // namespace mozilla
+}  // namespace storage
+}  // namespace mozilla
