@@ -57,6 +57,7 @@ module.exports = {
     BoxObject: false,
     BroadcastChannel: false,
     BrowsingContext: false,
+    CanonicalBrowsingContext: false,
     CDATASection: false,
     CSS: false,
     CSS2Properties: false,
@@ -103,6 +104,7 @@ module.exports = {
     ChromeWorker: false,
     Clipboard: false,
     ClipboardEvent: false,
+    ClonedErrorHolder: false,
     CloseEvent: false,
     CommandEvent: false,
     Comment: false,
@@ -122,6 +124,7 @@ module.exports = {
     DOMError: false,
     DOMException: false,
     DOMImplementation: false,
+    DOMLocalization: false,
     DOMMatrix: false,
     DOMMatrixReadOnly: false,
     DOMParser: false,
@@ -170,11 +173,14 @@ module.exports = {
     Flex: false,
     FlexItemValues: false,
     FlexLineValues: false,
+    FluentBundle: false,
+    FluentResource: false,
     FocusEvent: false,
     FontFace: false,
     FontFaceSet: false,
     FontFaceSetLoadEvent: false,
     FormData: false,
+    FrameCrashedEvent: false,
     FrameLoader: false,
     GainNode: false,
     Gamepad: false,
@@ -275,7 +281,6 @@ module.exports = {
     HashChangeEvent: false,
     Headers: false,
     HeapSnapshot: false,
-    HiddenPluginEvent: false,
     History: false,
     IDBCursor: false,
     IDBCursorWithValue: false,
@@ -306,11 +311,15 @@ module.exports = {
     InstallTriggerImpl: false,
     IntersectionObserver: false,
     IntersectionObserverEntry: false,
+    IOUtils: false,
+    JSProcessActorChild: false,
+    JSProcessActorParent: false,
     JSWindowActorChild: false,
     JSWindowActorParent: false,
     KeyEvent: false,
     KeyboardEvent: false,
     KeyframeEffect: false,
+    Localization: false,
     Location: false,
     MIDIAccess: false,
     MIDIConnectionEvent: false,
@@ -325,6 +334,7 @@ module.exports = {
     MatchPatternSet: false,
     MediaCapabilities: false,
     MediaCapabilitiesInfo: false,
+    MediaControlService: false,
     MediaDeviceInfo: false,
     MediaDevices: false,
     MediaElementAudioSourceNode: false,
@@ -391,8 +401,8 @@ module.exports = {
     PaintRequestList: false,
     PannerNode: false,
     ParentProcessMessageManager: false,
-    ParentSHistory: false,
     Path2D: false,
+    PathUtils: false,
     PaymentAddress: false,
     PaymentMethodChangeEvent: false,
     PaymentRequest: false,
@@ -417,9 +427,16 @@ module.exports = {
     Permissions: false,
     PlacesBookmark: false,
     PlacesBookmarkAddition: false,
+    PlacesBookmarkMoved: false,
+    PlacesBookmarkRemoved: false,
     PlacesEvent: false,
+    PlacesHistoryCleared: false,
     PlacesObservers: false,
+    PlacesPurgeCaches: false,
+    PlacesRanking: false,
     PlacesVisit: false,
+    PlacesVisitRemoved: false,
+    PlacesVisitTitle: false,
     PlacesWeakCallbackWrapper: false,
     Plugin: false,
     PluginArray: false,
@@ -606,10 +623,8 @@ module.exports = {
     StreamFilterDataEvent: false,
     StructuredCloneHolder: false,
     StructuredCloneTester: false,
-    StyleRuleChangeEvent: false,
     StyleSheet: false,
     StyleSheetApplicableStateChangeEvent: false,
-    StyleSheetChangeEvent: false,
     StyleSheetList: false,
     SubtleCrypto: false,
     SyncMessageSender: false,
@@ -647,6 +662,7 @@ module.exports = {
     UIEvent: false,
     URL: false,
     URLSearchParams: false,
+    UserInteraction: false,
     UserProximityEvent: false,
     VRDisplay: false,
     VRDisplayCapabilities: false,
@@ -734,6 +750,8 @@ module.exports = {
     WebrtcGlobalInformation: false,
     WheelEvent: false,
     Window: false,
+    WindowGlobalChild: false,
+    WindowGlobalParent: false,
     WindowRoot: false,
     Worker: false,
     Worklet: false,
@@ -747,7 +765,6 @@ module.exports = {
     XPathResult: false,
     XSLTProcessor: false,
     XULCommandEvent: false,
-    XULDocument: false,
     XULElement: false,
     XULFrameElement: false,
     XULMenuElement: false,
