@@ -8,7 +8,7 @@
 // ------------------------------------------------------------------------------
 
 var rule = require("../lib/rules/no-define-cc-etc");
-var RuleTester = require("eslint/lib/testers/rule-tester");
+var RuleTester = require("eslint").RuleTester;
 
 const ruleTester = new RuleTester({ parserOptions: { ecmaVersion: 9 } });
 
@@ -22,12 +22,12 @@ function invalidCode(code, varNames) {
   }
   return {
     code,
-    errors: varNames.map(name => [
-      {
+    errors: varNames.map(name => {
+      return {
         message: `${name} is now defined in global scope, a separate definition is no longer necessary.`,
         type: "VariableDeclarator",
-      },
-    ]),
+      };
+    }),
   };
 }
 
