@@ -12,7 +12,11 @@
 #include "nsISupportsImpl.h"
 #include "nsString.h"
 
+namespace mozilla {
+namespace baseprofiler {
 class SpliceableJSONWriter;
+}  // namespace baseprofiler
+}  // namespace mozilla
 
 // This class contains information that's relevant to a single page only
 // while the page information is important and registered with the profiler,
@@ -24,15 +28,15 @@ class SpliceableJSONWriter;
 class PageInformation final {
  public:
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(PageInformation)
-  PageInformation(uint64_t aBrowsingContextID, uint64_t aInnerWindowID,
+  PageInformation(uint64_t aTabID, uint64_t aInnerWindowID,
                   const nsCString& aUrl, uint64_t aEmbedderInnerWindowID);
 
   size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
   bool Equals(PageInformation* aOtherPageInfo) const;
-  void StreamJSON(SpliceableJSONWriter& aWriter) const;
+  void StreamJSON(mozilla::baseprofiler::SpliceableJSONWriter& aWriter) const;
 
   uint64_t InnerWindowID() const { return mInnerWindowID; }
-  uint64_t BrowsingContextID() const { return mBrowsingContextID; }
+  uint64_t TabID() const { return mTabID; }
   const nsCString& Url() const { return mUrl; }
   uint64_t EmbedderInnerWindowID() const { return mEmbedderInnerWindowID; }
 
@@ -45,7 +49,7 @@ class PageInformation final {
   }
 
  private:
-  const uint64_t mBrowsingContextID;
+  const uint64_t mTabID;
   const uint64_t mInnerWindowID;
   const nsCString mUrl;
   const uint64_t mEmbedderInnerWindowID;
